/*
  JS used on the user interface, and the admin interface
*/

var constantsShared = {
  CLASS_POPUP: '.popup'
}

/*
  === Init Class ===
  Used for initializing shared template resources
*/
var init = {
  /*
    Intialize material design bootstrap JS plugin (material.min.js), used
    for visual effects on checkboxes and other form inputs.
  */
  materialDesign: function() {
    $.material.init();
  },
};

/*
  === USER AJAX CLASS ===
  Ajax functions specific to the user view
*/
var sharedAjax = {
  /*
    Calls get_template_ajax.php endpoint to get Smarty template HTML with
    relevant attributes, and then insert it into a page.

    @param template Name of Smarty template
    @param attributes Object containing Smarty attributes that will be passed to
    smarty template (note that name must be what smarty is expecting)
    @param renderId HTML id of location to append or prepend returned HTML to
    @param shouldAppend If true, append HTML, else, prepend
    @param callback Function called on ajax finish
  */
  template: function(template, attributes, renderId, shouldAppend, callback) {
    var data = {};
    data['template'] = template;
    data['attributes'] = attributes;
    data['attributes']['ajax'] = true;

    // Convert attributes to JSON
    data['attributes'] = JSON.stringify(data['attributes']);

    // TODO replace this with a templateFromPhp Call
    $.ajax({
      type: 'POST',
      url: '/utilities/get_template_ajax.php',
      data: data,
      success: function(templateHTML) {
        sharedAjax.templateAppendHelper(templateHTML, renderId, shouldAppend, callback);
      }
    });
  },
  /*
    Similar to ajax.template function, but data passed to the template will come
    from the php endpoint, and not the ajax data passed up. Data passed to endpoint
    can be used however is appropriate in the endpoint.

    @param phpEndpoint PHP file to get template data from
    @param data Data object to pass to endpoint as POST data
    @param renderId
    @param shouldAppend
    @param callback
  */
  templateFromPhp: function(phpEndpoint, data, renderId, shouldAppend, callback) {
    data.ajax = true;
    $.ajax({
      type: 'POST',
      data: data,
      url: phpEndpoint,
      success: function(templateHTML) {
        sharedAjax.templateAppendHelper(templateHTML, renderId, shouldAppend, callback);
      }
    });
  },
  /*
    Helper for template and templateFromPhp. Appends or prepends templateHTML
    received.

    @param templateHTML HTML received from endpoint
    @param renderId
    @param shouldAppend
    @param callback
  */
  templateAppendHelper: function(templateHTML, renderId, shouldAppend, callback) {
    var render = $(renderId);
    if (shouldAppend) {
      render.append(templateHTML);
    } else {
      render.prepend(templateHTML);
    }
    if (callback) {
      callback();
    }

    // TODO I don't think we need to call this every time
    init.materialDesign(); // initialize the material JS library
  },

  /*
    Load more of a template that includes a media offset, with ajax
  */
  ajaxTemplatesOnButtonClick: function(loadMoreButtonId, loadMoreButtonWrapperId, ajaxLocationId, mediaOffset, childrenCount, phpEndpoint, data, afterAjaxCallback) {
    var loadMoreButton = $(loadMoreButtonId);
    mediaOffset = parseInt(mediaOffset);
    loadMoreButton.on('click', function() {
      var currentOffset = parseInt(loadMoreButton.attr('value'));
      var newOffset = currentOffset + mediaOffset;
      var nextOffset = newOffset + mediaOffset;
      var isAdmin = ('popup' in window);
      loadMoreButton.attr('value', newOffset);

      data['offset'] = newOffset;

      if (isAdmin) {
        popup.loadingGif();
      }

      // Load more media icons from the ajax endpoint section of index.php
      sharedAjax.templateFromPhp(
        phpEndpoint,
        data,
        ajaxLocationId,
        true,
        function() {
          if (isAdmin) {
            popup.removeLoadingGif();
          }
          if (afterAjaxCallback) {
            afterAjaxCallback();
          }
        }
      );

      // Remove button if no more to load
      if (nextOffset >= childrenCount) {
        $(loadMoreButtonWrapperId).remove();
      }
    });
  }
}

/*
  === Shared Popup class ===
*/
var sharedPopup = {
  /*
    Hide popup.
  */
  hide: function() {
    util.enableBodyScroll();
    util.fadeOutCSS(constantsShared['CLASS_POPUP']);
  },
  hideOnClick: function(buttonId) {
    $(buttonId).on('click', function() {
      sharedPopup.hide();
    });
  },
}

/*
  === Utility class ===
  For navigation, and general purposes.
*/
var util = {
  preventBodyScroll: function() {
    util.bodyScroll(false);
  },
  enableBodyScroll: function() {
    util.bodyScroll(true);
  },
  /*
    Prevent from scrolling, or enable scrolling, by adding CSS class.
    Helper for preventBodyScroll and enableBodyScroll.

    @param allowScroll If true, 'no-scroll' class will be added, else, it will
    be removed
  */
  bodyScroll: function(allowScroll) {
    if (allowScroll) {
      $('body').removeClass('no-scroll');
    } else {
      $('body').addClass('no-scroll');
    }
  },
  /*
    Fade out with jQuery.
  */
  fadeOut: function(fadeId) {
    var fade = $(fadeId);
    fade.fadeOut('slow');
  },
  /*
    Fade out with CSS3 transition (not supported on very old browsers).
  */
  fadeOutCSS: function(fadeId, keepOnPage) {
    var fade = $(fadeId);
    fade.addClass('fade-out');
    setTimeout(function () {
      if (keepOnPage) {
        fade.toggle(false);
      } else {
        fade.remove();
      }
    }, 500);
  },
  /*
    Navigate page back on item click, or if browser doesn't support back,
    go to the index page.
  */
  goBack: function() {
    if (typeof window.history.back === 'function') {
      window.history.back();
    } else { // Go home if browser doesn't support back
      util.redirect('/');
    }
  },
  // refreshPageWithParam removed
  refreshPage: function() {
    window.location = window.location.href;
  },
  redirect: function(href, newTab) {
    if (newTab) {
      window.open(href, '_blank');
    } else {
      window.location.href = href;
    }
  },
  /*
    Remove '#' symbol from the front of a string, if one is there.
    Used when taking a string that was used for a jQuery selector and handing it
    to a Smarty ajax call.
  */
  removePound: function(string) {
    if (string && string.length > 0 && string.charAt(0) === '#') {
      return string.substr(1);
    }
    return string;
  },
  addPound: function(string) {
    if (string.charAt(0) !== '#') {
      return '#' + string;
    }
    return string;
  },
  // scrollTo removed
  /*
    @return true if the device is an iOS device, false otherwise.
  */
  iOSDevice: function() {
    var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
    return iOS;
  },
  /*
    @return true if HTML5 audio player is supported, false otherwise.
  */
  supportsHTMLAudio: function() {
    return Modernizr.audio;
  },
  /*
    @return true if HTML5 video player is supported, false otherwise.
  */
  supportsHTMLVideo: function() {
    return Modernizr.video;
  },
  /*
    Adds a class to a selection, then removes after a time

    @param classNameToAdd
    @param objectId
    @param timeoutMs
  */
  addClassAndRemoveAfterTime: function(classNameToAdd, objectId, timeoutMs) {
    $(objectId).addClass(classNameToAdd);
    var removeClass = function() {
      var objectIdRemove = objectId;
      var timeoutRemove = timeoutMs;
      var classNameToRemove = classNameToAdd;
      window.setTimeout(function() {
        $(objectIdRemove).removeClass(classNameToRemove);
      }, timeoutRemove);
    }
    removeClass();
  }
}
