<?php

namespace Renew;

class DbManager {

	// Constants
	const SQLITE_DATE_FORMAT_STRING = '%Y-%m-%d %H:%M:%f';
	const SQLITE_STRFTIME_STRING = "STRFTIME('" . self::SQLITE_DATE_FORMAT_STRING . "', 'NOW')";

	/**
	* @var SQLite3 Database path
	* @access protected
	* */
	protected $dbPath;

	/**
	* @var SQLite3 Database Object of the media items, created from the $db_path
	* @access protected
	* */
	protected $dbHandle; // null if the handle is not open

	/**
	* @var The busy timeout for the sqlite3 database
	* @access protected
	* */
	protected $dbBusyTimeoutMs; // the busy timeout of the sqlite3 database

	/**
	* @var This will contain the string that is the clause to get items where the date time is between two values
	* @access protected
	* */
	protected $dateTimeBetweenClause;
	protected $dateTimeBetweenAndClause;
	protected $dateTimeBetweenWhereClause;

  public function __construct($dbPath) {
		$this->dbPath = $dbPath;
		$this->dbHandle = NULL;
		$this->dbBusyTimeoutMs = 15000;
		$this->dateTimeBetweenClause = '';
		$this->dateTimeBetweenAndClause = '';
		$this->dateTimeBetweenWhereClause = '';
		$this->openDb();
  }

	function __destruct(){
		$this->closeDb(); // close the database before you destruct
	}

	/**
	* @param Integer is the busy timeout in milliseconds
	*/
	public function setBusyTimeoutms($timeMs){
		$this->$dbBusyTimeoutMs = $timeMs;
	}

	/**
	* @return Integer of the busy timeout for the database in milliseconds
	*/
	public function getBusyTimeoutMs(){
		return $this->$dbBusyTimeoutMs;
	}

	public function setDateTimeBetweenClause($timeStart = '', $timeEnd = '', $dateTimeColName = 'date_time'){
		try {
			if (strlen($timeStart) == 0 || strlen($timeEnd) == 0) {
				$this->dateTimeBetweenClause = '';
				$this->dateTimeBetweenAndClause = '';
				$this->dateTimeBetweenWhereClause = '';
			}
			else {
				$this->dateTimeBetweenClause = sprintf("%s BETWEEN '%s' AND '%s'", $dateTimeColName, $timeStart, $timeEnd);
				$this->dateTimeBetweenAndClause = ' AND ' . $this->dateTimeBetweenClause ;
				$this->dateTimeBetweenWhereClause = ' WHERE ' . $this->dateTimeBetweenClause ;
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			$retVal = FALSE;
		}
	}

	/**
	* @return TRUE if dbHandle is/was open, and FALSE on failure
	*/
	public function openDb($pregMatch = TRUE, $pregReplace = TRUE) {
		$retVal = FALSE;
		try {
			if (!$this->isDbOpen()){
				$this->dbHandle = new \SQLite3($this->dbPath);
				$this->dbHandle->busyTimeout($this->dbBusyTimeoutMs);
				$this->dbHandle->exec("PRAGMA recursive_triggers = ON;");
				if ($pregMatch){
					if ($this->dbHandle->createFunction('preg_match', 'preg_match', 2) === FALSE)
						error_log("error creating sqlite3 function preg_match");
				}
				if ($pregReplace){
					if ($this->dbHandle->createFunction('preg_replace', 'preg_replace', 3) === FALSE)
						error_log("error creating sqlite3 function preg_replace");
				}
			}
			$retVal = TRUE;
		}
		catch (Exception $e){
			error_log($e);
			$retVal = FALSE;
		}
		return $retVal;
	}

	/**
	* @return TRUE if dbHandle is open, and FALSE if closed
	*/
	public function isDbOpen(){
		return !is_null($this->dbHandle);
	}

	/**
	* @return TRUE if dbHandle is/was closed, and FALSE on failure
	*/
	public function closeDb() {
		$retVal = FALSE;
		try {
			if ($this->isDbOpen()){
				$this->dbHandle->close();
				$this->dbHandle = NULL;
			}
			$retVal = TRUE;
		}
		catch (Exception $e) {
			error_log($e);
			$retVal = FALSE;
		}
		return $retVal;
	}

	public function executeSqlStmtStringRetArrayOfAssocArrays($stmtStr, $keyCol = NULL){
		try {
			$this->openDb();
			$stmt = $this->dbHandle->prepare($stmtStr);
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt, $keyCol);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}

	}

	/**
	* if no $keyCol is provided, then the results are returned as a simple array of associative arrays
	* if a $keyCol is provided, then the results are returned as an associative array of associative arrays where the first key is the value of the $keyCol
	* @param $sqlStmt SQLite3 statement
	* @param $keyCol String representing the column you would liek to key the associtave arrays by
	* @return array of associative arrays on success, and FALSE on failure
	*/
	public function executeSqlStmtRetArrayOfAssocArrays($stmt, $keyCol = NULL, $valueCol = NULL){
		$retVal = FALSE;
		try{
			if (!$this->isDbOpen()){
				error_log("error, trying to execute stmt when database is not open");
				$retVal = FALSE;
			} else {
				$resultArray = array();
				$sqlite3Result = $stmt->execute();

				if ($keyCol == NULL) {
					while ($result = $sqlite3Result->fetchArray(SQLITE3_ASSOC)) {
						$resultArray[] = $result;
					}
				} else {
					while ($result = $sqlite3Result->fetchArray(SQLITE3_ASSOC)) {
						if ($valueCol != NULL) {
							$resultArray[$result[$keyCol]] = $result[$valueCol];
						} else {
							$resultArray[$result[$keyCol]] = $result;
						}
					}
				}
		 		$retVal = $resultArray;
			}
		} catch (Exception $e) {
			error_log($e);
			$retVal = FALSE;
		}
		return $retVal;
	}

	public function executeSqlStmtRetFirstAssociativeArrayIfExists($stmt, $keyCol = NULL){
		try {
			$retArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, $keyCol);
			if (count($retArray) < 1) {
				return FALSE;
			}
			return $retArray[0];
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}

	}

	/**
	* @param SQLite3 statement
	* @return Boolean TRUE on success, and FALSE on failure
	*/
	public function executeSqlStmt($stmt){
		$retVal = FALSE;
		try{
			# will rturn a SQLite3Result on success, and FALSE on failure
			$result = $stmt->execute();
			$retVal = ($result !== FALSE);
		}
		catch (Exception $e){
			error_log($e);
			$retVal = FALSE;
		}
		return $retVal;
	}

	/**
	* @param String that is a sql query
	* @return Boolean TRUE on success, and FALSE on failure
	*/
	public function queryExecute($stmtStr){
		$retVal = FALSE;
		try{
			if (!$this->isDbOpen()){
				$this->openDb();
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			return $this->executeSqlStmt($stmt);
		}
		catch (Exception $e){
			error_log($e);
			$retVal = FALSE;
		}
		return $retVal;
	}


	/**
	* Updates cols for thrse rows that have that value for that col
	*
	* @param $tableName a string representing the name of the table to update
	* @param $colName a string representing the col name of the id
	* @param $colValue the value to update for
	* @param $attrs is an associative array where the keys represent the cols in the table and the values represent the new values to update to.
	* @return TRUE on success and FALSE on failure
	* @author CV
	**/
	public function updateTableAttributesForWithColnameAndColvalue($tableName, $colName, $colValue, $attrs, $likeValue=FALSE){
		try {
			if (!$this->isDbOpen()){
				$this->openDb();
			}
			if (count($attrs) === 0){
				return TRUE;
			}
			$stmtStr = 'UPDATE '.$tableName.' SET';
			foreach ($attrs as $key => $value) {
				$stmtStr .= ' ' . $key . ' = :' . $key . ', ';
			}
			$stmtStr = rtrim($stmtStr, ', ');
			if ($likeValue) {
				$stmtStr .= ' WHERE ' . $colName . ' LIKE ' . ":col_value";
			} else {
				$stmtStr .= ' WHERE ' . $colName . ' = :col_value';
			}
			$stmt = $this->dbHandle->prepare($stmtStr);

			$stmt->bindValue(':col_value', $colValue);
			foreach ($attrs as $key => $value) {
				$stmt->bindValue(':'.$key, $value);
			}

			$success = $this->executeSqlStmt($stmt);
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	*
	* @param $tableName a string representing the name of the table to update
	* @param $attrs is an associative array where the keys represent the cols in the table and the values represent the new values to insert
	* @return TRUE on success and FALSE on failure
	* @author CV
	**/
	public function insertOrIgnoreIntoTableAttributes($tableName, $attrs){
		return $this->_insertIntoTableAttributes($tableName, $attrs, ' OR IGNORE');
	}
	public function insertOrRepalceIntoTableAttributes($tableName, $attrs){
		return $this->_insertIntoTableAttributes($tableName, $attrs, ' OR REPLACE');
	}
	public function insertIntoTableAttributes($tableName, $attrs){
		return $this->_insertIntoTableAttributes($tableName, $attrs, '');
	}
	private function _insertIntoTableAttributes($tableName, $attrs, $orClause = ''){
		try {
			$colsStr = '(';
			$valuesStr = '(';
			foreach ($attrs as $key => $value) {
				$colsStr .= $key.', ';
				$valuesStr .= ':'.$key.', ';
			}
			$colsStr = rtrim($colsStr, ', ');
			$valuesStr = rtrim($valuesStr, ', ');
			$colsStr = $colsStr.')';
			$valuesStr = $valuesStr.')';
			$stmtStr = 'INSERT' .$orClause.' INTO '.$tableName.' '.$colsStr.' VALUES '.$valuesStr;
			$stmt = $this->dbHandle->prepare($stmtStr);
			foreach ($attrs as $key => $value) {
				$stmt->bindValue(':'.$key, $value);
			}
			$success = $this->executeSqlStmt($stmt);
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* For a settings table that has a key column and a value column
	* @param $key String represents the key in a settings database
	* @param $value String represents the value associated with that key
	* @return Boolean TRUE on success, and FALSE on failure
	*/
	public function settingsSetKeyValue($key, $value) {
		try{
			$stmt = $this->dbHandle->prepare( "INSERT OR REPLACE INTO settings (key, value) VALUES (:key, :value)");
			$stmt->bindValue(':key', $key);
			$stmt->bindValue(':value', $value);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e){
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Sets settings table that has a key column and a value column, with an array
	* @param $array Associative array of keys and values
	* @return Boolean TRUE on success, and FALSE on failure
	* @author JP
	*/
	public function settingsSetKeyValueArray($array) {
		$returnValue = TRUE;
      foreach ($array as $key => $value) {
        $success = $this->settingsSetKeyValue($key, $value);
				if ($success === FALSE) { // If we fail at any point
					$returnValue = FALSE;
				}
      }
    return $returnValue;
	}

	/**
	* For a settings table that has a key column and a value column
	* @param $key String represents the key in a settings database
	* @return String that is the value on success, and FALSE on failure
	*/
	public function settingsGetValueForKey($key) {
		try{
			$stmt = $this->dbHandle->prepare("SELECT value FROM settings WHERE key = :key");
			$stmt->bindValue(':key', $key);
			$results = $stmt->execute();
			return $results->fetchArray(SQLITE3_ASSOC)["value"];
		} catch (Exception $e){
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get values from a certain table over a specified time range
	* @param string $tableName is the name of the table
	* @param string $selectStmt is the select part of the sql stmt (optional)
	* @param string $clause is the clause that can be appended to the end of the slq stmt
	* @return array of associateve arrays on success or FALSE on failure
	*/
	public function tableSelectOverTimeRanges($tableName, $selectStmt = "SELECT COUNT(*)", $clause = "" ) {
		try {
			$clauseAnd="";
			$clauseWhere="";

			if (strlen($clause) > 0) {
				$clauseAnd="AND $clause";
				$clauseWhere="WHERE $clause";
			}

			$stmtStr = "SELECT * FROM ";
			$stmtStr = "$stmtStr ($selectStmt as '4hours' FROM $tableName WHERE date_time >= date('now', '-4 hours') $clauseAnd), ";
			$stmtStr = "$stmtStr ($selectStmt as '24hours' FROM $tableName WHERE date_time >= date('now', '-24 hours') $clauseAnd), ";
			$stmtStr = "$stmtStr ($selectStmt as '7days' FROM $tableName WHERE date_time >= date('now', '-7 days') $clauseAnd), ";
			$stmtStr = "$stmtStr ($selectStmt as 'all' FROM $tableName $clauseWhere)";
			$stmt = $this->dbHandle->prepare($stmtStr);
			//error_log($stmtStr);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* @param $associativeArray Associative array of data from Database
	* @param $keyName key for select
	* @param $valueName value for select
	* @param $callbackAppendstringToKey takes in an item from $associative array,
	* and can return an additional string for that item added to the value if desired.
	* Useful for giving users messages about certain selections.
	* @param $appendStringFirst if TRUE, value returned from callback will appear
	* at the front of the key
	**/
	public static function formatAssociativeArrayForSelect($associativeArray, $keyName, $valueName, $callbackAppendStringToKey = FALSE, $appendStringFirst = FALSE) {
		$answer = array();
		foreach ($associativeArray as $item) {
		  $key = $item[$keyName];
		  $value = $item[$valueName];
			if ($callbackAppendStringToKey) {
				$appendString = $callbackAppendStringToKey($item);
				if ($appendStringFirst) {
					$key = $appendString . $key;
				} else {
					$key .= $appendString;
				}
			}
		  $answer[$key] = $value;
		}
		return $answer;
	}

	/**
	* Returns an array of values, from an array of associative arrays with 'key', 'value' parameters
	* 
	* @param $arrayOfAssociativeArrays Array of Associative Arrays
	* @param $keyName The key to get values from
	* @return Array of values
	**/
	public static function formatArrayOfAssociativeArraysAsValueArray($arrayOfAssociativeArrays, $keyName) {
		$answerArray = array();
		foreach ($arrayOfAssociativeArrays as $associativeArray) {
			$answerArray[] = $associativeArray[$keyName];
		}
		return $answerArray;
	}

	public static function formatAssociativeArrayConvertBooleans($associativeArray) {
		foreach($associativeArray as $key => $value) {
			if ($value === 'true' || $value === 'false') { // || is_numeric($value)
				$value = json_decode($value);
			}
			$associativeArray[$key] = $value;
		};
		return $associativeArray;
	}
}
