<?php
 namespace Renew;
 require_once('DbManager.php');

 class DbManagerBluetooth extends DbManager {

    /**
     * counts the total number of pushes based on the supplied pushType
     *
     * @param string $pushType the push type (success, rejected, failed)
     * @return integer total number of pushes
     * @author JPul, CV
     **/
    public function statsCountPushes($pushtype)
    {
		try {
			$pushType = strtolower($pushtype);
			$acceptedPushTypes = array('success', 'rejected', 'failed', 'all');
			if (!in_array($pushtype, $acceptedPushTypes)) {
				return FALSE; // invalid type
			}
			if ($pushType == 'all') {
				$stmt = $this->dbHandle->prepare("SELECT COUNT(*) as count FROM bluetooth_interaction_stats");
			} else {
				$stmt = $this->dbHandle->prepare("SELECT COUNT(*) as count FROM bluetooth_interaction_stats WHERE interaction_type = :status");
				$stmt->bindValue(':status', $pushType);
			}
			$results = $stmt->execute();
			$row = $results->fetchArray();
			return $row['count'];
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
    /**
     * Finds the total number of unique devices based on the given $bluetoothActivityType
     *
     * @param string $bluetoothActivityType the activity type of the bluetooth (scanned, distributed)
     * @return integer total number of unique devices
     * @access public
     * @author JPul, CV
     **/
    public function statsCountUniqueDevices($bluetoothActivityType){
  		try {
  			$bluetoothActivityType = strtolower($bluetoothActivityType);
  			if ($bluetoothActivityType == 'success') {
  				$stmt = $this->dbHandle->prepare(
  					"SELECT COUNT(DISTINCT device_id) as count" .
  					" FROM bluetooth_interaction_stats" .
  					" WHERE interaction_type='success'"
  				);
  			} elseif ($bluetoothActivityType == 'all') {
  				$stmt = $this->dbHandle->prepare("SELECT COUNT(DISTINCT device_id) as count FROM bluetooth_interaction_stats");
  			} else {
  				return FALSE; // invalid activity type
  			}
  			$results = $stmt->execute();
  			$row = $results->fetchArray();
  			return $row['count'];
  		} catch (Exception $e) {
  			error_log($e);
  			return FALSE;
  		}
    }

    /**
    * Resets the data in the table
    *
    * @return Boolean TRUE on Success and FALSE on Failure
    * @access public
    * @author CV
    **/
    public function statsDeleteAll() {
      return $this->queryExecute("DELETE FROM bluetooth_interaction_stats");
    }

		public function statsGetOverTimeSuccessfulPushes() {
			return $this->statsGetOverTime('success');
		}

		public function statsGetOverTimeSuccessfulPushesUniqueDevices() {
			return $this->statsGetOverTime('success-devices');
		}

		public function statsGetOverTimeUniqueDevices() {
			return $this->statsGetOverTime('devices');
		}

		private function statsGetOverTime($interactionType) {
			try {
				$selectStmt = "SELECT COUNT(*)";
				$clause="";
				if ($interactionType == 'devices') {
					$selectStmt = "SELECT COUNT(DISTINCT device_id)";
				}
				elseif ($interactionType == 'success') {
					$clause="interaction_type == 'success'";
				}
				elseif ($interactionType == 'success-devices') {
          $selectStmt = "SELECT COUNT(DISTINCT device_id)";
					$clause="interaction_type == 'success'";
				}
				else {
					return FALSE;
				}
				return $this->tableSelectOverTimeRanges('bluetooth_interaction_stats', $selectStmt, $clause);
			} catch (Exception $e) {
				error_log($e);
				return FALSE;
			}
		}

    /**
     * Get the top $length files based on Successful pushes
     *
     * @param integer $length the total number of results you want
     * @return array of file_name and its total pushes
     * @author JPul, CV
     **/
    public function statsTopFiles($length) {
		try {
			$topFilesArray = array();
			$stmt = $this->dbHandle->prepare(
				"SELECT file_path, Count(*) as pushes from bluetooth_interaction_stats" .
				" WHERE interaction_type = 'success'" .
				" GROUP BY file_path" .
				" ORDER BY pushes DESC" .
				" LIMIT :lmt"
			);
			$stmt->bindValue(':lmt', intval($length));
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				array_push($topFilesArray, $row);
			}
			return $topFilesArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
    /**
     * Get the latest rejected devices, returns $length amount of devices
     *
     * @param integer $length The number of devices you want returned
     * @return array
     * @access public
     * @link http://shiflett.org/blog/2011/jun/sorting-multi-dimensional-arrays-in-php
     * @author JPul
     **/
    public function statsLatestRejectedDevices($length) {
		try {
			$devices = $this->statsFindAllBlacklistedDevices();
			return array_slice($devices, 0, $length);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}

    }
    /**
     * Returns an array of files that were pushed with the devices and the status of the push.
     * SUM code came from:
     * http://softwaresalariman.blogspot.com/2008/05/pivot-table-hack-in-sqlite3-and-mysql.html
     *
     * @return array
     * @access public
     * @author JPul
     **/
    public function statsFindAllPushedFiles() {
		try {
			$pushedFiles = array();
			$stmt = $this->dbHandle->prepare(
				"SELECT file_path " .
				", SUM(CASE WHEN interaction_type = 'success' THEN 1 END) as total_successes" .
				", SUM(CASE WHEN interaction_type = 'rejected' THEN 1 END) as total_rejected" .
				", SUM(CASE WHEN interaction_type = 'failed' THEN 1 END) as total_failed" .
				" FROM bluetooth_interaction_stats" .
				" WHERE file_id >= 0" .
				" GROUP BY file_path" .
				" ORDER BY file_path ASC"
			);
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				$row['total_successes'] = $this->nullToZero($row['total_successes']);
				$row['total_rejected'] = $this->nullToZero($row['total_rejected']);
				$row['total_failed'] = $this->nullToZero($row['total_failed']);
				array_push($pushedFiles, $row);
			}
			return $pushedFiles;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
	/**
	 *
	 * @return array
	 * @access public
	 * @author JPul
	 **/
	 public function statsFindAllPushedDevices() {
		try {
			$pushedFiles = array();
			$stmt = $this->dbHandle->prepare(
	            "SELECT device_id, device_name " .
	            ", SUM(CASE WHEN interaction_type = 'success' THEN 1 END) as total_successes" .
	            ", SUM(CASE WHEN interaction_type = 'rejected' THEN 1 END) as total_rejected" .
	            ", SUM(CASE WHEN interaction_type = 'failed' THEN 1 END) as total_failed" .
	            ", SUM(CASE WHEN interaction_type = 'scan' THEN 1 END) as total_scanned" .
	            " FROM bluetooth_interaction_stats" .
	            " GROUP BY device_id" .
	            " ORDER BY total_successes DESC"
			);
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				$row['total_successes'] = $this->nullToZero($row['total_successes']);
				$row['total_rejected'] = $this->nullToZero($row['total_rejected']);
				$row['total_failed'] = $this->nullToZero($row['total_failed']);
				array_push($pushedFiles, $row);
			}
			return $pushedFiles;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function statsFindAllSuccessfulPushedFiles() {
			try {
				$pushedFiles = array();
				$stmt = $this->dbHandle->prepare(
					"SELECT file_path, file_id " .
					", SUM(CASE WHEN interaction_type = 'success' THEN 1 END) as total_successes" .
					" FROM bluetooth_interaction_stats" .
					" WHERE (file_id >= 0 AND interaction_type='success')" .
					" GROUP BY file_path" .
					" ORDER BY total_successes DESC"
				);
				$results = $stmt->execute();
				while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
					$row['total_successes'] = $this->nullToZero($row['total_successes']);
					array_push($pushedFiles, $row);
				}
				return $pushedFiles;
			} catch (Exception $e) {
				error_log($e);
				return FALSE;
			}
    }


    public function statsFindAllSuccessfulPushedDevices() {
		try {
			$pushedFiles = array();
			$stmt = $this->dbHandle->prepare(
				"SELECT device_id, device_name " .
				", SUM(CASE WHEN interaction_type = 'success' THEN 1 END) as total_successes" .
				" FROM bluetooth_interaction_stats" .
				" WHERE interaction_type='success'" .
				" GROUP BY device_id" .
				" ORDER BY total_successes DESC"
			);
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				$row['total_successes'] = $this->nullToZero($row['total_successes']);
				array_push($pushedFiles, $row);
			}
			return $pushedFiles;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	public function statsFindAllSuccessfulAndScans() {
		try {
			$pushedFiles = array();
			$stmt = $this->dbHandle->prepare(
				"SELECT *  " .
				" FROM bluetooth_interaction_stats" .
				" WHERE (interaction_type='success' OR interaction_type='scan')" .
				" GROUP BY device_id" .
				" ORDER BY date_time DESC"
			);
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				array_push($pushedFiles, $row);
			}
			return $pushedFiles;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }

    /**
     * find all the rejected devices
     *
     * @return array An array of the rejected devices
     * @access public
     * @author JPul
     **/
    public function statsFindAllBlacklistedDevices() {
		try {
			$rejectedDevices = array();
			# grab some bluetooth settings here ...
			$maxPushRetry = $this->settingsGetValueForKey('max_push_retry');
			$rejectDelay = $this->settingsGetValueForKey('reject_delay');
			$rejectDelayStr = "-" . $rejectDelay . " hours";
			/**
			 * Retrieve the devices for the file
			 *
			 * @author JPul
			   **/
			$stmt = $this->dbHandle->prepare(
				"SELECT device_id, device_name, date_time, COUNT(interaction_id) AS reject_count" .
				" FROM bluetooth_interaction_stats" .
				" WHERE (interaction_type = 'rejected' OR interaction_type ='failed')" .
				" AND date_time BETWEEN DATETIME('now', :reject_delay_str) AND DATETIME('now')" .
				" GROUP BY device_id" .
				" HAVING reject_count >= :max_push_retry" .
				" ORDER BY date_time DESC"
			);
			$stmt->bindValue(':reject_delay_str', $rejectDelayStr, SQLITE3_TEXT);
			$stmt->bindValue(':max_push_retry', $maxPushRetry, SQLITE3_INTEGER); // use intval so it doesn't quote it when it puts it in the statement
			$results = $stmt->execute();
			while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
				array_push($rejectedDevices, $row);
			}
			return $rejectedDevices;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
    /**
     * Resets the tables for the specific Bluetooth stats
     *
     * @param array $resetTypes The table type to reset (pushes, scans, blacklist)
     * @return void
     * @access public
     * @author JPul
     **/
    public function statsReset($resetTypes = array()) {
		try {
			foreach ($resetTypes as $resetType) {
				if (!in_array($resetType, array('pushes', 'scans', 'blacklist', "all"))) {
					return FALSE; // invalid type in reset method
				}
			}
			$success = TRUE;
			if (in_array('blacklist', $resetTypes)) {
				$rejectDelay = $this->settingsGetValueForKey('reject_delay');
				$rejectDelayStr = "-" . $rejectDelay . " hours";
				$stmt = $this->dbHandle->prepare(
					"UPDATE bluetooth_interaction_stats" .
					" SET interaction_type = CASE" .
					"   WHEN interaction_type = 'rejected' THEN 'cleared_rejected'" .
					"   WHEN interaction_type = 'failed' THEN 'cleared_failed'" .
					"   ELSE interaction_type" .
					"   END" .
					" WHERE date_time BETWEEN DATETIME('now', :reject_delay_str) AND DATETIME('now')" .
					""
				);
				$stmt->bindValue(':reject_delay_str', $rejectDelayStr, SQLITE3_TEXT);
				$success &= $this->executeSqlStmt($stmt);
			}
			if (in_array('pushes', $resetTypes)) {
			  $stmt = $this->dbHandle->prepare(
					"UPDATE bluetooth_interaction_stats" .
					" SET interaction_type = CASE" .
					"   WHEN interaction_type = 'success' THEN 'cleared_success'" .
					"   ELSE interaction_type" .
					"   END"
			  );
			  $success &= $this->executeSqlStmt($stmt);
			}
			if (in_array('scans', $resetTypes)) {
				$stmt = $this->dbHandle->prepare(
					"UPDATE bluetooth_interaction_stats" .
					" SET interaction_type = CASE" .
					"   WHEN interaction_type = 'scan' THEN 'cleared_scan'" .
					"   ELSE interaction_type" .
					"   END"
				);
			  $success &= $this->executeSqlStmt($stmt);
			}
			if (in_array('all', $resetTypes)) {
				$stmt = $this->dbHandle->prepare("DELETE FROM bluetooth_interaction_stats");
				$success &= $this->executeSqlStmt($stmt);
			}
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
    /**
     * resets the backlisted device with the given $deviceId
     *
     * @param string $deviceId the unique id for the device
     * @return void
     * @access public
     * @author JPul
     **/
    public function statsResetBlacklistedDevice($device_id) {
		try {
			if ($device_id == '') {
				throw new \InvalidArgumentException("You are missing the device id to reset.");
				return;
			}
			# grab some bluetooth settings here ...
			//$max_push_retry = $this->getSetting('max_push_retry');
			$rejectDelay = $this->settingsGetValueForKey('reject_delay');
			$rejectDelayStr = "-" . $rejectDelay . " hours";
			$stmt = $this->dbHandle->prepare(
				"UPDATE bluetooth_interaction_stats" .
				" SET interaction_type = CASE" .
				"   WHEN interaction_type = 'rejected' THEN 'cleared_rejected'" .
				"   WHEN interaction_type = 'failed' THEN 'cleared_failed'" .
				"   ELSE interaction_type" .
				"   END" .
				" WHERE device_id=:device_id" .
				" AND date_time BETWEEN DATETIME('now', :reject_delay_str) AND DATETIME('now')" .
				""
			);
			$stmt->bindValue(':device_id', $device_id, SQLITE3_TEXT);
			$stmt->bindValue(':reject_delay_str', $rejectDelayStr, SQLITE3_TEXT);
			$stmt->execute();
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
    /**
     * Checks if the push if the file to the device id rejected
     *
     * @param array $device the device from the device table to test
     * @return boolean
     * @author JPul
     **/
	 /*
    private function statsIsRejected($device) {
		try {
			$pushedOn = new \DateTime($device['pushed']);
			$now = new \DateTime();
			$hoursPassed = ($now->format('U') - $pushedOn->format('U'))/3600;
			if ($device['reject_count'] >= $this->settingsGetValueForKey('max_push_retry')) {
				return TRUE;
			}
			if ($hoursPassed < $this->settingsGetValueForKey('reject_delay')) {
				return TRUE;
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
    }
	*/
    /**
     * Checks if a number is null then sets it to 0
     *
     * @param mixed $number the integer or null value to check
     * @return integer
     * @access private
     * @author JPul
     **/
    private function nullToZero($number) {
		try {

		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
        if (is_null($number) === true) {
            return 0;
        } else {
            return $number;
        }
    }
}
