<?php

namespace Renew;
require_once('DbManager.php');
require_once('DateHelper.php');
require_once('DiskInfo.php');
require_once('StringHelper.php');
require_once('SystemHelper.php');
require_once('FilesHelper.php');
require_once('ArrayHelper.php');

// define this class for use below, it will skip all directories and files that are hidden (ie begin with a dot ('.'))
class RecursiveDotFileFilterIterator extends \RecursiveFilterIterator {
	public function accept() {
		return '.' !== substr($this->current()->getFileName(), 0 , 1);
	}
}

class DbManagerMedia extends DbManager {

	/**
	* @var
	* @access protected
	**/
	protected $userImagesDirAbsPath; // all formats of images are stored in subdirs under this dir
	protected $mediaDirPath;
	protected $lastErrorsArray;

	const ERR_TITLE_EXISTS = 'ERR_TITLE_EXISTS';
	const ERR_IMAGE_EXISTS = 'ERR_TITLE_EXISTS';
	const ERR_DB_FAILED = 'ERR_DB_FAILED';
	const ERR_CONVERSION_FAILED = 'ERR_CONVERSION_FAILED';
	const ERR_IMAGE_DNE = 'ERR_IMAGE_DNE';
	const ERR_THUMBNAIL_NOT_SUPPORTED = 'ERR_THUMBNAIL_NOT_SUPPORTED';
	const ERR_THUMBNAIL_FAILED = 'ERR_THUMBNAIL_FAILED';

	const IMAGE_DIR_4X3 = "4x3";
	const IMAGE_DIR_16X9 = "16x9";
	const IMAGE_DIR_BG = "bg";
	const IMAGE_DIR_TMP = "tmp";

	const IMAGE_KEY_ABSPATH_4X3 = "abspath_4x3"; // 240x180, quality 85
	const IMAGE_KEY_ABSPATH_16X9 = "abspath_16x9"; // 1280x720, quality 60
	const IMAGE_KEY_ABSPATH_BG = "abspath_bg"; // maxw 1280 maxh 1280, quality 70

	const MEDIA_MIME_DIRECTORY = 'directory';
	const MEDIA_MIME_LANGUAGE_DIRECTORY = 'languagedirectory';
	const MEDIA_MIME_PART1 = 'mime_part1';
	const MEDIA_MIME_PART1_VIDEO = 'video';
	const MEDIA_PARENT_PATH_REL = 'parent_path_rel';
	const MEDIA_ROOT_DIRECTORY_TITLE = 'Home';
	const MEDIA_ROOT_PARENT_PATH_REL = '';
	const MEDIA_ROOT_DIRECTORY_FILE_ID = 0;
	const MEDIA_LANGUAGE_ISO_CUSTOM_LANGUAGE_NAME = 'und';
	
	const DOWNLOADER_SOURCE_JF = 'jf';
	const ARCLIGHT_SUBTYPE_FEATURE_FILM = 'featureFilm';
	
	// default values
	const LIMIT_SEARCH_DEFAULT = 30;
	const FILE_ID_DEFAULT = '0'; 


	/**
	*
	* @param $dbPath the absolute path to the media sql database
	* @param $userImagesDirAbsPath the absolute path to the user images directory
	* @author CV
	*/
	public function __construct($dbPath, $mediaDirPath, $userImagesDirAbsPath) {
		parent::__construct($dbPath);
		$this->mediaDirPath = rtrim($mediaDirPath, DIRECTORY_SEPARATOR);
		$this->mediaDirPath = $this->mediaDirPath.'/';
		$this->userImagesDirAbsPath = rtrim($userImagesDirAbsPath, DIRECTORY_SEPARATOR);
		$this->lastErr = NULL;
	}

	public function getLastErr(){
		return $this->lastErr;
	}

	/**
	*
	* @return associative array of all of the images, where the key is the image_id
	*/
	public function imagesGetAll($offset = NULL, $limit = NULL) {
		$this->openDb();
			// hide all of the thumbnail images (those with from_media_file_id > 0)
			$stmtStr = 'SELECT * FROM images WHERE from_media_file_id < 1 ORDER BY image_id DESC';
			$hasOffset = (isset($limit) && isset($offset));
			if ($hasOffset) {
				$stmtStr .= " LIMIT :lim OFFSET :off";
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			if ($hasOffset) {
				$stmt->bindValue(':off', $offset);
				$stmt->bindValue(':lim', $limit);
			}
			$array = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, "image_id");
		$this->closeDb(); // TODO, should we really close this? generally we do this on the endpoints.
		return $array;
	}

	public function imagesDeleteAll() {
		$this->openDb();
		$success = TRUE;
		$imagesArray = $this->imagesGetAll();
		if ($imagesArray === FALSE) {
			return FALSE;
		}
		foreach($imagesArray as $image) {
			$success &= $this->imagesDeleteImageWithId($image['image_id']);
		}
		return $success;
	}

	/**
	* Delte an image from the database and from the filesystem
	*
	* @return TRUE on success, and an integer representing an ERR code on failure
	* @access public
	* @author CV
	**/
	public function imagesDeleteImageWithId($imageId) {
		return $this->imagesDeleteImage($imageId, NULL);
	}
	public function imagesDeleteImageWithTitle($imageTitle) {
		return $this->imagesDeleteImage(NULL, $imageTitle);
	}
	// this function is protected because it may not be obvious how to use it to the outside world
	protected function imagesDeleteImage($imageId, $imageTitle){
		$imageItem = $this->imagesGetImage($imageId, $imageTitle, TRUE);
		if ($imageItem === FALSE){
			return self::ERR_IMAGE_DNE; // the image DNE
		}
		$imageId = $imageItem['image_id'];
		$this->openDb();
		$stmt = $this->dbHandle->prepare("DELETE FROM images WHERE image_id = :image_id ");
		$stmt->bindValue(":image_id", $imageId);
		$success = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		$stmt = $this->dbHandle->prepare("UPDATE media2 SET image_id = 0 WHERE image_id = :image_id ");
		$stmt->bindValue(":image_id", $imageId);
		$success = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		$this->closeDb();
		if ( $success === FALSE ){
			return self::ERR_DB_FAILED;
		}
		// remove the files from the filesystem
		unlink($imageItem[self::IMAGE_KEY_ABSPATH_4X3]);
		unlink($imageItem[self::IMAGE_KEY_ABSPATH_16X9]);
		unlink($imageItem[self::IMAGE_KEY_ABSPATH_BG]);

		return TRUE;
	}

	public function imagesDeleteImageFromMediaId($fileId) {
		try {
			$image = $this->imagesGetImageFromMediaFileId($fileId);
			if ($image !== FALSE && isset($image['image_id'])) {
				return $this->imagesDeleteImageWithId($image['image_id']);
			}
			else {
				return FALSE;
			}
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function mediaSetDefaultsForFileNameWithFileIdIfAvailable($fileId) {
		try {
			$fileInfo = $this->mediaGetFileWithId($fileId, TRUE);
			if ($fileInfo === FALSE) {
				error_log("ERROR mediaSetDefaultsForFileNameWithFileIdIfAvailable fileId returned FALSE problem");
				return FALSE;
			}

			$fileName = $fileInfo['file_name'];
			$fileExtension = strtolower($fileInfo['file_extension']);
			$parentPathRel = $fileInfo['parent_path_rel'];
			$imageToApply = FALSE;
			$attrs = array();
			$arclightJsonInfo = $fileInfo['downloader_json'];
			if ($arclightJsonInfo !== '') {
				$arclightJsonInfo = json_decode($arclightJsonInfo, TRUE);
			}

			// Handle case of people loading with arclight.json file from USB, with images folder on USB
			if ($arclightJsonInfo !== '' && isset($arclightJsonInfo['image_title'])) {
				$imageToApply = $this->imagesGetImageWithTitle($arclightJsonInfo['image_title']);
			}
			// Jfilm
			else if (preg_match('/^JESUS Film\[[a-z][a-z][a-z]\]$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - JESUS Film');
				$attrs['featured'] = 1;
			}
			else if (preg_match('/^Magdalena\[[a-z][a-z][a-z]\]$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - Magdalena');
			}
			// rule for matching Jesus Film Segments folders
			else if (preg_match('/^JESUS Film Segments$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - JESUS Film Segments');
			}
			// rule for matching More Than Dreams folders
			else if (preg_match('/^More Than Dreams$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - More Than Dreams');
			}
			else if (preg_match('/^The Hope\[[a-z][a-z][a-z]\]$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - The Hope');
			}
			// naming rule for FCBH Bible Apps (APKs)
			else if (preg_match('/^.*-[0-9]+\.[0-9]+.[0-9]+$/i', $fileName) && $fileExtension === 'apk') {
				$imageToApply = $this->imagesGetImageWithTitle('Default - Bible App');
				$attrs['featured'] = 1;
			}
			// naming rule for FCBH Bible Apps (APKs)
			else if (preg_match('/^God Provides.*$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - God Provides');
			}
			// naming rule for The Savior Film
			else if (preg_match('/^The Savior.*$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - The Savior');
			}
			// naming rule for audio Bible files
			else if (preg_match('/^.+\[[A-Z0-9]{8}\]\//i', $parentPathRel) || preg_match('/^.+\[[A-Z0-9]{8}\]$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - Audio Bible');
			}
			else if (preg_match('/^The Story\[[a-z][a-z][a-z]\]$/i', $fileName)) {
				$imageToApply = $this->imagesGetImageWithTitle('Default - The Story');
			}
			// naming convention for LUMO gospel films
			else if (preg_match('/^Gospel of (Matthew|Mark|Luke|John)\[[a-z][a-z][a-z]\]$/i', $fileName) && $fileExtension === 'mp4') {
				$imageToApply = $this->imagesGetImageWithTitle('Default - Lumo Gospel Films');
			}
			// naming rule for JESUS Film's story of Jesus for children
			else if (preg_match('/^The Story of JESUS for Children\[[a-z][a-z][a-z]\]$/i', $fileName) && $fileExtension === 'mp4') {
				$imageToApply = $this->imagesGetImageWithTitle('Default - The Story of JESUS for Children');
			}
			// naming rule for the Gospel of John from JESUS film
			else if (preg_match('/^Life of Jesus \(Gospel of John\)\[[a-z][a-z][a-z]\]$/i', $fileName) && $fileExtension === 'mp4') {
				$imageToApply = $this->imagesGetImageWithTitle('Default - Life of Jesus (Gospel of John)');
			}
			if ($imageToApply !== FALSE) {
				$attrs['image_id'] = $imageToApply['image_id'];
			}

			$this->mediaUpdateAttrsForFileWithId($fileId, $attrs);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function imagesSetImageTitleForId($imageId, $imageTitle) {
		try {
			$attrs = array();
			$attrs['image_title'] = $imageTitle;
			return $this->updateTableAttributesForWithColnameAndColvalue('images', 'image_id', $imageId, $attrs);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Set the from_media_file_id column for a particular image_id
	*
	* @return boolean TRUE on success, and an FALSE on failure
	* @access private
	* @author CV
	**/
	public function imagesSetFromMediaFileIdForImageId($imageId, $fromMediaFileId){
		try {
			$this->openDb();
			$stmtStr = "UPDATE images SET from_media_file_id = :from_media_file_id WHERE image_id = :image_id";
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':from_media_file_id', $fromMediaFileId);
			$stmt->bindValue(':image_id', $imageId);
			return $this->executeSqlStmt($stmt);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Take a video file and create a thumbnail image for it
	*
	* @return Integer that is the last inserted rowid on success, and FALSE on failure with an err written to lastErr.
	* @access public
	* @author CV
	**/
	public function imagesAddThumbnailVideo($videoPath, $randomThumbnail = TRUE) {
		try {

			$ffmpegthumbnailerBin = '/usr/bin/ffmpegthumbnailer';
			if (!file_exists($ffmpegthumbnailerBin)) {
				// add support for mac development environment
				$ffmpegthumbnailerBin = '/usr/local/bin/ffmpegthumbnailer';
				if (@readlink($ffmpegthumbnailerBin) === FALSE) {
					error_log('NO FF!');
					$this->lastErr = self::ERR_THUMBNAIL_NOT_SUPPORTED;
					return FALSE;
				}
			}

			// TODO, please empty this directory at each boot
			$imagesTmpDir = $this->userImagesDirAbsPath . '/' . self::IMAGE_DIR_TMP;
			$this->fileMkdirIfNotExists($imagesTmpDir);

			// ffmpegthumbnailer exists
			// try to get the thumbnail from the video at 50% through the video
			$pctIn = 50;
			if ($randomThumbnail === TRUE) {
				$pctIn = rand(10, 90);
			}
			$fileInfo = $this->fileGetInfoWithPath($videoPath);
			$tmpImagePath = $imagesTmpDir . '/'  .$fileInfo['file_name'] . '.jpg';
			$videoPathAbs = $fileInfo['path_abs'];
			$cmd = "rwo_createVideoThumbnail.sh " . escapeshellarg($videoPathAbs) . " " . escapeshellarg($tmpImagePath) . " \"{$pctIn}\"  2>&1";
			$returnArray = SystemHelper::RunCmd($cmd, FALSE);

			if ($returnArray === FALSE) {
				$this->lastErr = self::ERR_THUMBNAIL_FAILED;
				return FALSE;
			}

			return $this->imagesAddImage($fileInfo['file_name'], $tmpImagePath);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Take a video file and create a thumbnail image for it
	*
	* @return Integer that is the last inserted rowid on success, and FALSE on failure with an err written to lastErr.
	* @access public
	* @author CV
	**/
	public function imagesAddThumbnailForVideoWithId($fileId, $randomThumbnail = FALSE) {
		try {
			$fileInfo = $this->mediaGetFileWithId($fileId, TRUE);
			if ($fileInfo === FALSE) {
				error_log("ERROR imagesAddThumbnailForVideoWithId trying to add thumbnail error retrieving file");
				return FALSE;
			}
			if ($fileInfo['mime_part1'] !== 'video') {
				error_log("ERROR imagesAddThumbnailForVideoWithId trying to add thumbnail for file of incorrect type");
				return FALSE;
			}
			// delete the old thumbnail
			$this->imagesDeleteImageFromMediaId($fileId);

			$pathAbs = $this->mediaPathToPathAbs($fileInfo['path_rel']);

			$imageId = $this->imagesAddThumbnailVideo($pathAbs, $randomThumbnail);
			if ($imageId === FALSE) {
				error_log("ERROR imagesAddThumbnailForVideoWithId adding thumbnail for video ... ");
				return FALSE;
			}

			$success = $this->imagesSetFromMediaFileIdForImageId($imageId, $fileId);
			if ($success === FALSE) {
				error_log("ERROR imagesAddThumbnailForVideoWithId adding thumbnail for video ... ");
				return FALSE;
			}

			return TRUE;
			// $attrs = array();
			// $attrs['image_id'] = intval($imageId);
			// return TRUE; $this->mediaUpdateAttrsForFileWithId($fileId, $attrs);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}

	}

	/**
	* Add an image to the dtabase, and move it to the proper location, and do a conversion
	*
	* @return Integer that is the last inserted rowid on success, and FALSE on failure with an err written to lastErr.
	* @access public
	* @author CV
	**/
	public function imagesAddImageSetTitleToFileName($imagePath, $allowDuplicateTitles = TRUE) {
		try {
			$fileInfo = $this->fileGetInfoWithPathAbs($imagePath);
			return $this->imagesAddImage($fileInfo['file_name'], $imagePath, $allowDuplicateTitles);
		} catch (\Exception $e) {
			$this->lastErr = self::ERR_IMAGE_DNE;
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Add a new image to the database
	 * @return Integer imageId
	 */
	public function imagesAddImage($imageTitle, $imagePath, $allowDuplicateTitles = TRUE) {
		try {
			if (!file_exists($imagePath)) {
				$this->lastErr = self::ERR_IMAGE_DNE;
				return FALSE; // the file does not exist
			}
			// this may rename your image title
			$exists = $this->imagesGetImageWithTitle($imageTitle);
			if ($exists !== FALSE){
				if ($allowDuplicateTitles === FALSE) {
					$this->lastErr = self::ERR_TITLE_EXISTS; // same tite
					return FALSE;
				}
				$index = 1;
				while ($exists !== FALSE) {
					$imageTitleNew = $imageTitle . "(" . $index . ")";
					$exists = $this->imagesGetImageWithTitle($imageTitleNew);
					$index = $index + 1;
				}
				$imageTitle = $imageTitleNew;
			}

			$this->openDb();
				$stmt = $this->dbHandle->prepare("INSERT OR IGNORE INTO images (image_title) VALUES (:image_title)");
				$stmt->bindValue(":image_title", $imageTitle);
				$dbSuccess = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
				$lastInsertRowId = $this->dbHandle->lastInsertRowID(); // we can return this on success
			$this->closeDb();

			if ($dbSuccess === FALSE){
				unlink($imagePath); // remove the original file
				$this->lastErr = self::ERR_DB_FAILED; // the sql statement failed
				return FALSE;
			}

			self::SetFilePermissions($imagePath, 0777);
			// TODO DEV ENVIRONMENT - This is a lot of dirnames. Remove this for the hardware, non-dev version?
			$basePath = dirname(dirname(dirname(dirname(dirname(__FILE__))))); // for dev environment
			$cmd = "\"rwo_convertWebImages.sh\" " . escapeshellarg($imagePath) . " 2>&1";
			$returnArray = SystemHelper::RunCmd($cmd, FALSE);

			// Temp image paths
			$imagePathBg="{$imagePath}.bg.jpeg";
			$imagePath4x3="{$imagePath}.4x3.jpeg";
			$imagePath16x9="{$imagePath}.16x9.jpeg";

			$conversionSuccess = FALSE;
			// check all converted files are there
			if (file_exists($imagePathBg)
					&& file_exists($imagePath4x3)
					&& file_exists($imagePath16x9)) {
				$conversionSuccess = TRUE;
			}

			if ($conversionSuccess === FALSE){
				if (file_exists($imagePath)) unlink($imagePath); // remove the original file
				// delete any converted files
				if (file_exists($imagePathBg)) unlink($imagePathBg);
				if (file_exists($imagePath4x3)) unlink($imagePath4x3);
				if (file_exists($imagePath16x9)) unlink($imagePath16x9);
				$this->imagesDeleteImageWithTitle($imageTitle);
				$this->lastErr = self::ERR_CONVERSION_FAILED;
				return FALSE;
			}

			// now time to move the files
			$imageItem = $this->imagesGetImage(NULL, $imageTitle, TRUE); // get image by title w/ abs_path s
			unlink($imagePath); // remove the original file
			$imageDestBg = $imageItem[self::IMAGE_KEY_ABSPATH_BG];
			$imageDest4x3 = $imageItem[self::IMAGE_KEY_ABSPATH_4X3];
			$imageDest16x9 = $imageItem[self::IMAGE_KEY_ABSPATH_16X9];

			$this->fileMkdirIfNotExists(dirname($imageDestBg));
			$this->fileMkdirIfNotExists(dirname($imageDest4x3));
			$this->fileMkdirIfNotExists(dirname($imageDest16x9));

			rename($imagePathBg, $imageDestBg);
			rename($imagePath4x3, $imageDest4x3);
			rename($imagePath16x9, $imageDest16x9);
			self::SetFilePermissions($imageDestBg);
			self::SetFilePermissions($imageDest4x3);
			self::SetFilePermissions($imageDest16x9);

			return $lastInsertRowId;
		} catch (Exception $e) {
			$this->lastErr = self::ERR_IMAGE_DNE;
			error_log($e);
			return FALSE;
		}
	}

	/**
	* @return associative array of the parameters for the image, and FALSE on failure
	* @access protected
	* @author CV
	**/
	public function imagesGetImageWithTitle($imageTitle) {
		return $this->imagesGetImage(NULL, $imageTitle);
	}

	public function imagesGetImageWithId($imageId) {
		return $this->imagesGetImage($imageId, NULL);
	}

	public function imagesGetImageFromMediaFileId($fromMediaFileId) {
		return $this->imagesGetImage(NULL, NULL, FALSE, $fromMediaFileId);
	}

	public function imagesGetImageIdFromMediaFileId($fromMediaFileId) {
		try {
			$image = $this->imagesGetImageFromMediaFileId($fromMediaFileId);
	    if ($image == FALSE) {
	      return FALSE;
	    }
	    else {
	      return $image['image_id'];
	    }
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	protected function imagesGetImage($imageId, $imageTitle = NULL, $includeAbsPaths = FALSE, $fromMediaFileId = NULL){
		try {
			$stmt = NULL;
			$this->openDb();
				if ($imageId !== NULL){
					$stmt = $this->dbHandle->prepare("SELECT * FROM images WHERE image_id = :image_id");
					$stmt->bindValue(":image_id", $imageId);
				}
				elseif ($imageTitle !== NULL) {
					$stmt = $this->dbHandle->prepare("SELECT * FROM images WHERE image_title = :image_title");
					$stmt->bindValue(":image_title", $imageTitle);
				}
				elseif ($fromMediaFileId !== NULL) {
					$stmt = $this->dbHandle->prepare("SELECT * FROM images WHERE from_media_file_id = :from_media_file_id");
					$stmt->bindValue(":from_media_file_id", $fromMediaFileId);
				}
				else {
					$this->closeDb();
					return FALSE; // FAIL, not enough parameters to distinguish
				}
				$array = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			$this->closeDb();
			if ($array === FALSE) {
				return FALSE; // this means it failed
			}
			if ( count($array) < 1 ){
				return FALSE; // the array was empty, the item did not exist in the database
			}
			$imageItem=$array[0];
			// check if you should include the abs paths for the image
			if ($includeAbsPaths){
				$imageItem = $this->imagesFillInAbsPathsForImageItem($imageItem);
			}
			return $imageItem; // return the item
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	protected function imagesFillInAbsPathsForImagesArray($imagesArray){
		try {
			foreach ($imagesArray as $imageItem) {
				$imageItem = $this->imagesFillInAbsPathsForImageItem($imageItem);
			}
			return $imagesArray;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	protected function imagesFillInAbsPathsForImageItem($imageItem){
		$imageItem[self::IMAGE_KEY_ABSPATH_4X3] = $this->userImagesDirAbsPath . '/' . self::IMAGE_DIR_4X3 . '/' . $imageItem["image_id"] . ".jpeg";
		$imageItem[self::IMAGE_KEY_ABSPATH_16X9] = $this->userImagesDirAbsPath . '/' . self::IMAGE_DIR_16X9 . '/' . $imageItem["image_id"] . ".jpeg";
		$imageItem[self::IMAGE_KEY_ABSPATH_BG] = $this->userImagesDirAbsPath . '/' . self::IMAGE_DIR_BG . '/' . $imageItem["image_id"] . ".jpeg";
		return $imageItem;
	}

	public function mediaGetDirPath(){
		return $this->mediaDirPath;
	}

	public function mediaPathToPathRel($path) {
		return preg_replace('/^'.preg_quote($this->mediaDirPath, '/').'/', '', $path);
	}

	public function mediaPathToPathAbs($path) {
		$pathRel = $this->mediaPathToPathRel($path);
		$pathAbs = $this->mediaGetDirPath().$pathRel;
		return $pathAbs;
	}

	/**
	* Insert or replace metadata in the database recursively for all media
	* Any media that has been added before will simply be undeleted
	*
	* @return boolean TRUE on success and FALSE on failure
	* @author CV
	*/
	public function mediaScanForMedia() {
		$retVal= TRUE;
		try {
			// get the recursive iterator for the current directory, includes folders, but does not include $dir, aand does not include any hidden directories
			$mediaDirIter = new \RecursiveDirectoryIterator($this->mediaDirPath, \RecursiveDirectoryIterator::SKIP_DOTS);
			$mediaDirIterIter = new \RecursiveIteratorIterator(new \Renew\RecursiveDotFileFilterIterator($mediaDirIter), \RecursiveIteratorIterator::SELF_FIRST, \RecursiveIteratorIterator::CATCH_GET_CHILD);
			// Go ahead and set all of the media in the database to "deleted", and we are essentially going to "add it back in", one by one
			$this->openDb();
			$stmt = $this->dbHandle->prepare("UPDATE media2 SET is_deleted = 1 WHERE is_deleted = 0");
			$this->executeSqlStmt($stmt);
			// iterate throug al files recursively adding them back into the database or undeleteing them
			foreach ($mediaDirIterIter as $path => $file) {
				$this->mediaAddOrUndeleteFileWithPath($path, $file);
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
		// return the status
		return $retVal;
	}

	/**
	* Adds or undeletes a piece of media in the database
	* if the file in the database with the same parent_path_rel, file_name, fille_extension, and hash exists, undelete
	* Otherwise, add a new entry in the media database
	* @param $path can either be an absolute path, or a path relative to the media directory to the file
	* @param $file is optional, an SplFileInfo object, if it is NULL, then later function can create the object.
	* @param $arclightJsonObject an ArrayList that contains arclightJson information for the file, optionally
	* @return the last inserted rowId on success and FALSE on failure
	* @author CV
	**/
	public function mediaAddOrUndeleteFileWithPath($path, $file = NULL, $arclightJsonObject = NULL) {
		try {
			$retVal = FALSE;
			$mediaItem = $this->mediaGetFileWithPath($path);
			$hash = $this->fileHash($path);
			// check if it is the same media item
			if ($mediaItem !== FALSE && $mediaItem['hash'] == $hash && $mediaItem['is_deleted'] !== 0 && $arclightJsonObject === NULL) {
				// it's the same media item, let's undelete the item
				$this->openDb();
				$stmt = $this->dbHandle->prepare("UPDATE media2 SET is_deleted = 0 WHERE file_id = :file_id");
				$stmt->bindValue(':file_id', $mediaItem['file_id']);
				$retVal = $this->executeSqlStmt($stmt);
				if ($retVal !== FALSE) {
					$retVal = $mediaItem['file_id'];
				}
			}	else {
				$languageIso = NULL;
				if ($arclightJsonObject !== NULL && isset($arclightJsonObject['languageIso'])) {
					$languageIso = $arclightJsonObject['languageIso'];
				}
				$retVal = $this->mediaAddFileWithPathToDb($path, $file, $hash, TRUE, $languageIso, $arclightJsonObject);
			}
			return $retVal;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Adds a file to the media database
	*
	* @param $path can either be an absolute path, or a path relative to the piece of media that is not in the library yet, but should be
	* @param $parentPathId is the integer file_id of the parent directory to put this file inside of
	* @return an integer which is the newly created fileid on success and FALSE on failure
	**/
	public function mediaAddFileWithPathToMediaDirAndDb($pathAbs, $parentDirId = 0, $updateArray = FALSE) {
		try {
			if ($parentDirId == NULL) {
				$parentDirId = 0;
			}
			$parentPathRel = '';
			if ($parentDirId != 0) {
				$parentFile = $this->mediaGetFileWithId($parentDirId);
				if($parentFile == FALSE || $parentFile['mime_part1'] != self::MEDIA_MIME_DIRECTORY){
					return FALSE;
				}
				$parentPathRel = $parentFile['parent_path_rel'] . $parentFile['file_name'] . '/';
			}
			$fileInfo = $this->fileGetInfoWithPathAbs($pathAbs);
			if ($fileInfo == FALSE || $fileInfo['mime_part1'] == self::MEDIA_MIME_DIRECTORY) {
				return FALSE;
			}
			$newPathAbs = $this->mediaPathToPathAbs(self::GetNewPathRelFromParentPathAndFileInfo($parentPathRel, $fileInfo, FALSE));

			// check to see if there is a file already by that same name at that location
			// if there is, append a timestamp to the file and then check again
			if (file_exists($newPathAbs)) {
				$newPathAbs = $this->mediaPathToPathAbs(self::GetNewPathRelFromParentPathAndFileInfo($parentPathRel, $fileInfo, TRUE));
				if (file_exists($newPathAbs)) {
					return FALSE; // something weird, even the file with the timestamp exists!
				}
			}
			// move the file into it's proper location
			rename($pathAbs, $newPathAbs);
			$fileId = $this->mediaAddFileWithPathToDb($newPathAbs);
			// if it failed to add to the DB, then move it back
			if ($fileId === FALSE) {
				rename($newPathAbs, $pathAbs);
			} else {
				self::SetFilePermissions($newPathAbs);
				if ($updateArray !== FALSE) {
					$this->mediaUpdateAttrsForFileWithId($fileId, $updateArray);
				}
			}
			return $fileId; // returns FALSE or the rowid of the new file
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Adds a file to the media database
	*
	* @param $path can either be an absolute path, or a path relative to the media directory to the file
	* @param $file is optional, an SplFileInfo object, if it is NULL, then later function can create the object.
	* @param $hash is optional a string that is the hash for the content of the file, if it is null the function can take the hash for you.
	* @param $arclightJsonObject an ArrayList with information if this file comes from Arclight
	* @return mixed the last inserted rowid on success and FALSE on failure
	**/
	private function mediaAddFileWithPathToDb($path, $file = NULL, $hash = NULL, $makeVideoThumb = TRUE, $languageIso = NULL, $arclightJsonObject = NULL) {
		try {
			$fileInfo = $this->fileGetInfoWithPath($path, $file);
			if ($fileInfo === FALSE) {
				return FALSE;
			}
			if ($hash == NULL || strlen($hash) < 1) {
				$hash = $this->fileHash($fileInfo['path_abs']);
			}
			$imageId = 0;
			if ($makeVideoThumb === TRUE && $fileInfo['mime_part1'] === 'video') {
				$imageId = $this->imagesAddThumbnailVideo($fileInfo['path_abs']);
				if ($imageId === FALSE) {
					$imageId = 0;
				}
			}
			$title = $fileInfo['file_name'];

			// override the langauge iso
			if (preg_match('/^[a-z][a-z][a-z]$/', $languageIso)) {
				$fileInfo['language_iso'] = $languageIso;
			}

			// check for language directories
			if (strlen($fileInfo['language_iso']) === 3 && $fileInfo['mime_part2'] === self::MEDIA_MIME_DIRECTORY && $fileInfo['parent_path_rel'] === self::MEDIA_ROOT_PARENT_PATH_REL) {
				$fileInfo['mime_part2'] = self::MEDIA_MIME_LANGUAGE_DIRECTORY;
				$languageIso = $fileInfo['language_iso'];

				// check if the title has brackets iso at the end (fileGetInfoWithPathAbs gets iso as fileInfo['language_iso'] from brackets)
				if (preg_match('/^.+\['.$languageIso.'\]$/', $title)) {
					$title = substr($title, 0, -5);
				} else {
					// if you are adding a language folder that is simply "[pes]"
					// give the langauge folder its localized title
					$language = $this->languagesGetWithIso($languageIso);
					$title = $language['name_english'];
					if ($language['name_localized'] !== NULL) {
						$title = $language['name_localized'];
					}
				}
			}
			$downloaderSource = $arclightJsonObject === NULL ? '' : self::DOWNLOADER_SOURCE_JF;
			$downloaderJson = '';
			$description = '';
			$featured = 0;
			if ($arclightJsonObject !== NULL) {
				// downloader_json
				$downloaderJsonArray = array();
				if (isset($arclightJsonObject['mediaComponentId'])) {
					$downloaderJsonArray['media_component_id'] = urldecode($arclightJsonObject['mediaComponentId']);
				}
				if (isset($arclightJsonObject['mediaLanguageId'])) {
					$downloaderJsonArray['media_language_id'] = urldecode($arclightJsonObject['mediaLanguageId']);
				}
				if (isset($arclightJsonObject['imageTitle'])) {
					$downloaderJsonArray['image_title'] = urldecode($arclightJsonObject['imageTitle']);
				}
				if (!empty($downloaderJsonArray)) {
					$downloaderJson = json_encode($downloaderJsonArray);
				}

				// additional attributesfrom arclight
				if (isset($arclightJsonObject['subType']) && $arclightJsonObject['subType'] === self::ARCLIGHT_SUBTYPE_FEATURE_FILM) {
					$featured = 1;
				}
				if (isset($arclightJsonObject['title'])) {
					$title = urldecode($arclightJsonObject['title']);
				}
				if (isset($arclightJsonObject['longDescription'])) {
					$description = urldecode($arclightJsonObject['longDescription']);
				}
			}
			$stmtStr = "INSERT INTO media2 "
						."(parent_path_rel, hash, file_name, file_extension, mime_part1, mime_part2, "
						."wifi_status, bluetooth_status, usb_status, image_id, description, title, "
						."featured, sort_order, datetime_added, datetime_edited, language_iso, "
						."file_size_bytes, file_length_seconds, is_deleted, promoted_media_file_id, greeting_title, downloader_source, downloader_json) "
						."VALUES (:parent_path_rel, :hash, :file_name, :file_extension, :mime_part1, :mime_part2, "
						.":wifi_status, :bluetooth_status, :usb_status, :image_id, :description, :title, "
						.":featured, :sort_order, :datetime_added, :datetime_edited, :language_iso, "
						.":file_size_bytes, :file_length_seconds, :is_deleted, :promoted_media_file_id, :greeting_title, :downloader_source, :downloader_json)";
			$this->openDb();
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':parent_path_rel', $fileInfo['parent_path_rel']);
			$stmt->bindValue(':hash', $hash);
			$stmt->bindValue(':file_name', $fileInfo['file_name']);
			$stmt->bindValue(':file_extension', $fileInfo['file_extension']);
			$stmt->bindValue(':mime_part1', $fileInfo['mime_part1']);
			$stmt->bindValue(':mime_part2', $fileInfo['mime_part2']);
			$stmt->bindValue(':wifi_status', 2);
			$stmt->bindValue(':bluetooth_status', 0);
			$stmt->bindValue(':usb_status', 2);
			$stmt->bindValue(':image_id', intval($imageId));
			$stmt->bindValue(':description', $description);
			$stmt->bindValue(':title', $title);
			$stmt->bindValue(':featured', $featured);
			$stmt->bindValue(':sort_order', NULL);
			$stmt->bindValue(':datetime_added', \DateHelper::DateTimeStringSQLite3());
			$stmt->bindValue(':datetime_edited', '');
			$stmt->bindValue(':language_iso', $fileInfo['language_iso']);
			$stmt->bindValue(':file_size_bytes', $fileInfo['file_size_bytes']);
			$stmt->bindValue(':file_length_seconds', 0);
			$stmt->bindValue(':is_deleted', 0);
			$stmt->bindValue(':promoted_media_file_id', 0);
			$stmt->bindValue(':greeting_title', '');
			$stmt->bindValue(':downloader_source', $downloaderSource);
			$stmt->bindValue(':downloader_json', $downloaderJson);
			$retVal = $this->executeSqlStmt($stmt);
			if ($retVal !== FALSE){
				// get the last inserted rowid
				$retVal = $this->dbHandle->lastInsertRowID();
				// if a thumbnail was made, then go ahead and mark which file it was made from
				if ($makeVideoThumb === TRUE && $imageId !== 0) {
					$this->imagesSetFromMediaFileIdForImageId($imageId, $retVal);
				}
			}
			return $retVal;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
		return FALSE;
	}

	/**
	* Updates cols for that piece of media in the media database
	*
	* @param $fileId the ID for that particualr file in the media database
	* @param $attrs is an associative array where the keys represent the cols in the media database and the values represent the new values to update to.
	* @return TRUE on success and FALSE on failure
	* @author CV
	**/
	public function mediaUpdateAttrsForFileWithId($fileId, $attrs){
		try {
			unset($attrs['file_id']); // file_id should never be updated

			// check to see if it is a directory and if it needs udpate of sub files/directories
			$pulledFileAttrs=$this->mediaGetFileWithId($fileId, TRUE);

			// delete the old image if it's a thumbnail image that is being unset
			if (isset($attrs['image_id']) && $pulledFileAttrs['image_id'] !== $attrs['image_id']) {
				$thumbnailId = $this->imagesGetImageFromMediaFileId($fileId);
				if ($pulledFileAttrs['image_id'] === $thumbnailId) {
					$this->imagesDeleteImageWithId($thumbnailId);
				}
			}

			if ($pulledFileAttrs['mime_part1'] === 'directory'){
				// only update these attributes for a Directory
				$attrsForDirectory=array();
				if (isset($attrs['wifi_status']) && $attrs['wifi_status'] != $pulledFileAttrs['wifi_status']) {
					$attrsForDirectory['wifi_status'] = $attrs['wifi_status'];
				}
				if (isset($attrs['bluetooth_status']) && $attrs['bluetooth_status'] != $pulledFileAttrs['bluetooth_status']) {
					$attrsForDirectory['bluetooth_status'] = $attrs['bluetooth_status'];
				}
				if (isset($attrs['usb_status']) && $attrs['usb_status'] != $pulledFileAttrs['usb_status']) {
					$attrsForDirectory['usb_status'] = $attrs['usb_status'];
				}

				if (count($attrsForDirectory) > 0) {
					$pathRel=$pulledFileAttrs['path_rel'];
					$pathRelLike="$pathRel%";
					$this->mediaUpdteAttrsForFilesWithParentPathRelLike($pathRelLike, $attrsForDirectory);
				}

			}
			return $this->updateTableAttributesForWithColnameAndColvalue('media2', 'file_id', $fileId, $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	// TODO is the being used any more?
	public function mediaUpdteAttrsForFilesWithParentPathRelLike($parentPathRelLike, $attrs) {
		try {
			// these attributs should never be udpated for all under a certain parent_path_rel
			unset($attrs['file_id']);
			unset($attrs['file_name']);
			unset($attrs['file_extension']);
			unset($attrs['mime_part1']);
			unset($attrs['mime_part2']);
			unset($attrs['sort_order']);
			unset($attrs['datetime_added']);
			unset($attrs['file_size_bytes']);
			unset($attrs['file_length_seconds']);
			unset($attrs['parent_path_rel']);
			if (count($attrs) < 1) {
				return TRUE; // nothing to update
			}
			return $this->updateTableAttributesForWithColnameAndColvalue('media2', 'parent_path_rel', $parentPathRelLike, $attrs, TRUE);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}

	}


	/**
	* Get the path relative to the media directory for a file with a give ID.
	*
	* @param $fileId the ID for that particualr file in the media database
	* @return String that is the path relative to the media directory on success and FALSE on failure
	* @author CV
	**/
	public function mediaGetPathRelForId($fileId){
		try {
			$this->openDb();
			$stmt = $this->dbHandle->prepare("SELECT parent_path_rel, file_name, file_extension, mime_part1, mime_part2 FROM media2 WHERE file_id = :file_id");
			$stmt->bindValue(':file_id', $fileId);
			$sqlite3Result = $stmt->execute();
			$resultArray = $sqlite3Result->fetchArray(SQLITE3_ASSOC);
			return $this->mediaGetPathRelForFileObject($resultArray);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function mediaGetPathRelForFileObject($fileObject){
		try {
			$pathRel = $fileObject['parent_path_rel'] . $fileObject['file_name'];
			if (strlen($fileObject['file_extension']) > 0) {
				$pathRel = $pathRel . '.' .$fileObject['file_extension'];
			}
			elseif ($fileObject['mime_part1'] == self::MEDIA_MIME_DIRECTORY) {
				$pathRel .= '/';
			}
			return $pathRel;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Ge the path relative to the media directory for a file with a give ID.
	*
	* @param $parentPath the path of the parent directory for which you want to get media under (pass blank for root)
	* @param $depth a non-zero value representing the depth at which you want to get media under
	* @param $limitsArray the count limits you want to impose at each corresponding depth
	* @param $offset the offset you want to get at the first depth
	* @return A multidimensional array of attributes, children and their children sorted first by sort_order and then by title on success, and FALSE on failure
	* @author CV
	**/
	public function mediaGetHomepageWithParentPath($parentPath, $offset = 0, $limitsArray = [3, 10, 30], $depth = 3) {
		try {
			if ($depth < 1)
				return FALSE;
			$resultArray = [];
			$limit = $limitsArray[$depth - 1];
			$this->openDb();
			$stmt = $this->dbHandle->prepare("SELECT * FROM media2 WHERE parent_path_rel = :parent_path_rel AND is_deleted = 0 AND wifi_status > 0 AND mime_part2 != :language_directory_mime ORDER BY sort_order IS NULL, sort_order, title ASC LIMIT :lim OFFSET :off");
			$stmt->bindValue(':parent_path_rel', $parentPath);
			$stmt->bindValue(':lim', $limit);
			$stmt->bindValue(':off', $offset);
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$sqlite3Result = $stmt->execute();
			while ($result = $sqlite3Result->fetchArray(SQLITE3_ASSOC)) {
				$countOnlyWifiEnabledInChildren = TRUE;
				$result = $this->mediaAddFileDetailsToFile($result, $countOnlyWifiEnabledInChildren);
				if ($result['mime_part1'] === self::MEDIA_MIME_DIRECTORY) {
					$nextDepth = $depth - 1;
					if ($result['children_count'] > 0 && $nextDepth > 0) {
						$result['children'] = $this->mediaGetHomepageWithParentPath($result['path_rel'], 0, $limitsArray, $nextDepth);
					}
				}
				// This creates issues with count being messed up
				// // Don't return folders with 0 children
				// if (!isset($result['children_count']) || $result['children_count'] > 0) {
				// 	$resultArray[] = $result;
				// }
				$resultArray[] = $result;
			}
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		TODO Comment
	*/
	public function mediaGetHomepageWithParentId($parentId, $offset = 0, $limitsArray = [3, 10, 30], $depth = 3)  {
		try {
			$this->openDb();
			$parentPath = $this->mediaGetPathRelForId($parentId);
			return $this->mediaGetHomepageWithParentPath($parentPath, $offset, $limitsArray, $depth);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get all media that is set to is_global
	 * @return Array of global media
	 */
	public function mediaGetGlobalMedia($isFeatured = FALSE) {
		try {
			$stmtStr = 'SELECT * FROM media2 WHERE is_global > 0 AND wifi_status > 0 AND is_deleted = 0 ';
			if ($isFeatured === TRUE) {
				$stmtStr .= 'AND featured > 0 ';
			}
			$stmtStr .= 'ORDER BY sort_order IS NULL, sort_order, title ASC;';
			$stmt = $this->dbHandle->prepare($stmtStr);
			$fileArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			return $this->mediaAddFileDetailsToFileArray($fileArray, TRUE);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get all language directories with iso
	 * @param $languageIso
	 */
	public function mediaGetLanguageDirectoriesWithLanguageIso($languageIso) {
		try {
			$stmtStr = "SELECT * FROM media2 WHERE mime_part2 == :language_directory_mime AND language_iso = :language_iso AND mime_part1 = 'directory' AND parent_path_rel = '' AND is_deleted = 0;";
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$stmt->bindValue(':language_iso', $languageIso);
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function mediaGetLanguageDirectories($getForHomepage = FALSE, $includePromotedMediaTitle = FALSE) {
		try {
			$this->openDb();
			$selectFields = '*';
			$getForHomepageString = '';
			if ($getForHomepage === TRUE) {
				$getForHomepageString = 'AND wifi_status > 0 ';
				if ($selectFields === '*') {
					$selectFields = 'file_name, title, file_id, language_iso, mime_part2';
				}
			}
			$stmtStr = '';
			$stmtStrEnding = "";
			if ($includePromotedMediaTitle) {
				// This selects * all fields from media2, as well as the title from the promoted media item
				// Join media2 table to itself so that we can get the title of the promoted media item.
				$stmtStr = "SELECT table_1.*, table_2.title AS promoted_media_title from media2 AS table_1 LEFT JOIN media2 AS table_2 ON table_1.promoted_media_file_id = table_2.file_id WHERE table_1.mime_part2 == :language_directory_mime {$getForHomepageString}AND table_1.mime_part1 = 'directory' AND table_1.parent_path_rel = '' AND table_1.is_deleted = 0 ORDER BY table_1.sort_order IS NULL, table_1.sort_order, table_1.title ASC";
			} else {
				$stmtStr = "SELECT {$selectFields} FROM media2 WHERE mime_part2 == :language_directory_mime {$getForHomepageString}AND mime_part1 = 'directory' AND parent_path_rel = '' AND is_deleted = 0 ORDER BY sort_order IS NULL, sort_order, title ASC";
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$stmt->bindValue(':select_fields', $selectFields);
			$sqlite3Result = $stmt->execute();
			$resultArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($getForHomepage === FALSE) {
				$countOnlyWifiEnabledInChildren = FALSE;
				$resultArray = $this->mediaAddFileDetailsToMediaArray($resultArray, $countOnlyWifiEnabledInChildren);
			}
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		TODO Comment
	*/
	public function mediaGetAllWithParentPath($parentPath, $offset = NULL, $limit = NULL) {
		try {
			$this->openDb();
			// Eventually, add in english name to folders: . "(SELECT languages_iso639_3.Ref_Name FROM languages_iso639_3 WHERE languages_iso639_3.Id = media2.language_iso LIMIT 1) AS language_eng "
			$stmtStr = "SELECT *, (SELECT images.image_id FROM images WHERE from_media_file_id = media2.file_id LIMIT 1) AS image_id_thumbnail FROM media2 WHERE parent_path_rel = :parent_path_rel AND is_deleted = 0 AND mime_part2 != :language_directory_mime ORDER BY sort_order IS NULL, sort_order, title ASC";
			$hasOffset = (isset($offset) && isset($limit));
			if ($hasOffset) {
				$stmtStr .= " LIMIT :lim OFFSET :off";
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			if ($hasOffset) {
				$stmt->bindValue(':lim', $limit);
				$stmt->bindValue(':off', $offset);
			}
			$stmt->bindValue(':parent_path_rel', $parentPath);
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$resultArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			// Include children count in directories
			$countOnlyWifiEnabledInChildren = FALSE;
			$resultArray = $this->mediaAddFileDetailsToMediaArray($resultArray, $countOnlyWifiEnabledInChildren);
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function mediaGetAllWithParentId($parentId, $offset = NULL, $limit = NULL) {
		try {
			$this->openDb();
			$parentPath = $this->mediaGetPathRelForId($parentId);
			return $this->mediaGetAllWithParentPath($parentPath, $offset, $limit);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		TODO Comment
	*/
	public function mediaAddFileDetailsToMediaArray($resultArray, $countOnlyWifiEnabledInChildren = FALSE) {
		foreach ($resultArray as $key => $file) {
			$resultArray[$key] = $this->mediaAddFileDetailsToFile($file, $countOnlyWifiEnabledInChildren);
		}
		return $resultArray;
	}

	public function mediaAddPathRelToMediaArray($resultArray) {
		foreach ($resultArray as $key => $file) {
			$resultArray[$key]['path_rel'] = $this->MediaGetPathRelWithFile($file);
		}
		return $resultArray;
	}

	/*
		TODO Comment
	*/
	public function mediaAddFileDetailsToFile($file, $countOnlyWifiEnabledInChildren = FALSE) {
		// this is the relative path to the file with filename and extension if needed
		$file['path_rel'] = $this->MediaGetPathRelWithFile($file);

		$isRoot = self::MediaFileIdIsRootDirectory($file['file_id']);
		if ($isRoot || $file['mime_part1'] === self::MEDIA_MIME_DIRECTORY) { // Children count if directory
			$childrenCount = $this->mediaCountChildrenWithParentPath($file['path_rel'], $countOnlyWifiEnabledInChildren);
			$file['children_count'] = $childrenCount;
		} else { // Else, file size and length
			$file['file_size'] = \Renew\DiskInfo::BytesToHumanReadableSize($file['file_size_bytes']);
			$file['file_length'] = \Renew\StringHelper::SecondsToFormattedTime($file['file_length_seconds']);
		}
		return $file;
	}

	public function mediaAddFileDetailsToFileArray($fileArray, $countOnlyWifiEnabledInChildren = FALSE) {
		for($i = 0; $i < count($fileArray); $i++) {
			$fileArray[$i] = $this->mediaAddFileDetailsToFile($fileArray[$i], $countOnlyWifiEnabledInChildren);
		}
		return $fileArray;
	}

	public function mediaGetAllBluetoothEnabledMediaAddPathRel() {
		return $this->mediaGetMediaInfoHelper("WHERE bluetooth_status > 0", '*', 'ORDER BY title ASC', TRUE);
	}

	// NOT USED
	// public function mediaGetAllBluetoothDisabledMediaAddPathRel() {
	// 	return $this->mediaGetMediaInfoHelper("WHERE bluetooth_status = 0 AND mime_part1 != 'directory'", '*', 'ORDER BY title ASC', TRUE);
	// }

	// NOT USED
	// public function mediaGetAllNonFeaturedMediaAddPathRel() {
	// 	return $this->mediaGetMediaInfoHelper("WHERE featured = 0", '*', 'ORDER BY title ASC', TRUE);
	// }

	// NOT USED
	// public function mediaGetAllMediaAddPathRel() {
	// 	return $this->mediaGetMediaInfoHelper(FALSE, '*', 'ORDER BY title ASC', TRUE);
	// }
	
	// NOT USED
	// public function mediaGetAllMediaInfoInDirectoryWithFilename($directoryFilename) {
	// 	$directoryFilename = \SQLite3::escapeString($directoryFilename);
	// 	return $this->mediaGetMediaInfoHelper("WHERE parent_path_rel LIKE '{$directoryFilename}%'");
	// }

	public function mediaGetAllDirectoriesAddPathRel() {
		$answer = $this->mediaGetMediaInfoHelper("WHERE mime_part1 = 'directory'", "*", "ORDER BY parent_path_rel ASC, title ASC", TRUE);

		// Add root directory info
		array_unshift($answer, array(
			'path_rel' => self::MEDIA_ROOT_DIRECTORY_TITLE,
			'file_id' => self::MEDIA_ROOT_DIRECTORY_FILE_ID,
		));

		return $answer;
	}

	/*
		Legacy. Do not use this for getting all media on the device (it overflows ram). Use the media search function that uses limit and ajax.
		Also, if you calculate the number of children in each directory it can take a LONG time
	*/
	private function mediaGetMediaInfoHelper($where = FALSE, $options = "*", $orderByClause = "ORDER BY title ASC", $onlyPathRel = FALSE) {
		try {
			$this->openDb();
			$langaugeDirectoryMime = self::MEDIA_MIME_LANGUAGE_DIRECTORY;
			$defaultWhereOptions = "is_deleted = 0 AND mime_part2 != '{$langaugeDirectoryMime}' ";
			$stmtStr = "SELECT " . $options ." FROM media2 ";
			if ($where !== FALSE) {
				$stmtStr .= $where . " AND {$defaultWhereOptions}";
			} else { // if the where is FALSE, then make sure you are not returning media that has been deleted
				$stmtStr .= "WHERE {$defaultWhereOptions}";
			}

			if (strlen($orderByClause) > 0) {
				$stmtStr .= $orderByClause;
			}

			$stmt = $this->dbHandle->prepare($stmtStr);
			$resultArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);

			// TODO: if we get only media enabled over wifi, change this to true
			$countOnlyWifiEnabledInChildren = FALSE;
			if ($onlyPathRel === TRUE) {
				$resultArray = $this->mediaAddPathRelToMediaArray($resultArray);
			} else {
				$resultArray = $this->mediaAddFileDetailsToMediaArray($resultArray, $countOnlyWifiEnabledInChildren);
			}
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		TODO Comment
	*/
	public static function MediaFormatDirectoryArrayForSelect($directoriesArray) {
		return self::MediaFormatMediaArrayForSelect($directoriesArray, 'path_rel', 'file_id');
	}

	/*
		TODO Comment
	*/
	public static function MediaFormatLanguageDirectoryArrayForSelect($languageDirectoriesArray) {
		return self::MediaFormatMediaArrayForSelect(
			$languageDirectoriesArray,
			'title',
			'file_id',
			function($languageDirectory) { // Add filename to end of titles
				return ' (' . $languageDirectory['file_name'] . ')';
			}
		);
	}

	/*
		TODO Comment
	*/
	public static function MediaFormatMediaArrayForSelectWithDirectoryString($mediaInfoArray) {
		return self::MediaFormatMediaArrayForSelect(
			$mediaInfoArray,
			'path_rel',
			'file_id',
			function($mediaInfo) {
				$appendString = '';
				if ($mediaInfo['mime_part1'] === DbManagerMedia::MEDIA_MIME_DIRECTORY) {
					$appendString = ' ('._('folder').')';
				}
				return $appendString;
			}
		);
	}

	/*
		TODO Comment
	*/
	public static function MediaFormatMediaArrayForSelectBluetooth($mediaInfoArray) {
		return self::MediaFormatMediaArrayForSelect(
				$mediaInfoArray,
				'path_rel',
				'file_id',
				function($mediaInfo) {
					$fileSize = (int) $mediaInfo['file_size_bytes'];
					$appendString = ' (' . \Renew\DiskInfo::BytesToHumanReadableSize($fileSize) . ')';
					/*
					if ($fileSize <= self::BLUETOOTH_MAX_RECOMMENDED_BYTES) {
					       $appendString .= ' - ' . strtoupper(_('Small File'));
					}
					*/
					return $appendString;
			}
		);
	}

	/**
	 * Return true if media item is language directory
	 * @param mediaItem file in database
	 * @return Boolean
	 */
	public static function MediaItemIsLanguageDirectory($mediaItem) {
		return self::MediaItemMimePart2Helper($mediaItem, self::MEDIA_MIME_LANGUAGE_DIRECTORY);
	}

	/**
	 * Return true if media item is directory
	 * @param mediaItem file in database
	 * @return Boolean
	 */
	public static function MediaItemIsDirectory($mediaItem) {
		return self::MediaItemMimePart2Helper($mediaItem, self::MEDIA_MIME_DIRECTORY);
	}

	/**
	 * Return true if $mediaItem['mime_part2] === $mimePart2
	 * @param $mediaItem
	 * @param $mimPart2
	 * @return Boolean
	 */
	public static function MediaItemMimePart2Helper($mediaItem, $mimePart2) {
		if ($mediaItem && isset($mediaItem['mime_part2']) && $mediaItem['mime_part2']) {
			return $mediaItem['mime_part2'] === $mimePart2;
		}
		return FALSE;
	}

	/**
	 * Remove global media items from media item array
	 * @param $mediaItemArray Array mediaItem object array
	 */
	public static function MediaItemArrayRemoveGlobal($mediaItemArray) {
		for($i = 0; $i < count($mediaItemArray); $i++) {
			if ($mediaItemArray[$i]['is_global'] !== 0) {
				array_splice($mediaItemArray, $i, 1); // Remove item if global
			}
			// TODO: This hides stuff in nested folders, but then the counts are wrong. Should we just remove this?
			// else if (self::MediaItemIsDirectory($mediaItemArray[$i]) && isset($mediaItemArray[$i]['children'])) {
			// 	// Hide global media from nested children media
			// 	$mediaItemArray[$i]['children'] = array_filter($mediaItemArray[$i]['children'], function($currentMediaItem) {
			// 		return $currentMediaItem['is_global'] === 0;
			// 	});
			// }
		}
		return $mediaItemArray;
	}

	public static function MediaFormatMediaItemForSelect($mediaItem, $keyName = 'title', $valueName = 'file_id', $callbackAppendStringToKey = FALSE) {
		return self::formatAssociativeArrayForSelect(array($mediaItem), $keyName, $valueName, $callbackAppendStringToKey);
	}

	public static function MediaFormatMediaArrayForSelect($mediaInfoArray, $keyName = 'title', $valueName = 'file_id', $callbackAppendStringToKey = FALSE) {
		return self::formatAssociativeArrayForSelect($mediaInfoArray, $keyName, $valueName, $callbackAppendStringToKey);
	}

	public static function MediaFileIdIsRootDirectory($fileId) {
		return ((int) $fileId) === DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID;
	}

	/**
	* Get relative path from parent relative path and fileInfo object
	*
	* @param $parentPathRel
	* @param $fileInfo
	* @param $shouldAddTimestamp If TRUE, a timestamp will be added to the file name
	**/
	public static function GetNewPathRelFromParentPathAndFileInfo($parentPathRel, $fileInfo, $shouldAddTimestamp) {
		$dateString = '';
		if ($shouldAddTimestamp === TRUE) {
			$date = new \DateTime();
			$dateString = $date->format('YmdHis');
		}
		$newPathRel = $parentPathRel . $fileInfo['file_name'] . $dateString;
		if (strlen($fileInfo['file_extension']) > 0) {
			$newPathRel = $newPathRel . '.' . $fileInfo['file_extension'];
		}
		return $newPathRel;
	}

	/*
		TODO Comment
	*/
	public function mediaGetBreadcrumbsWithPath($path) {
		// Termination condition
		if ($path === '') {
			return array();
		}

		$file = $this->mediaGetFileWithPath($path);
		$answer = array($file['file_id'] => $file['title']);

		$path = substr($path, 0, strlen($path) - 1); // remove '/' at end
		$newPath = substr($path, 0, strrpos($path, '/')); // Get new parent path, but removing current dir

		// Recursive call
		return $this->mediaGetBreadcrumbsWithPath($newPath) + $answer; // Merge arrays
	}

	/**
	* Get the number of children on the homapge given a parent path rel
	*
	* @param $parentPathRel the path of the parent directory for which you want to get media under
	* @return A number of immediate children for the given parent path on success, and FALSE on failure
	* @author CV
	**/
	public function mediaCountChildrenWithParentPath($parentPath, $countOnlyWifiEnabled = TRUE){
		try {
			$parentPathRel = $this->mediaPathToPathRel($parentPath);
			$this->openDb();
			$stmtAppend = '';
			if ($countOnlyWifiEnabled) {
				$stmtAppend = ' AND wifi_status > 0';
			}
			$stmt = $this->dbHandle->prepare("SELECT COUNT(file_id) AS children_count FROM media2 WHERE parent_path_rel = :parent_path_rel AND is_deleted = 0 AND mime_part2 != :language_directory_mime". $stmtAppend);
			$stmt->bindValue(':parent_path_rel', $parentPathRel);
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$sqlite3Result = $stmt->execute();
			$resultArray = $sqlite3Result->fetchArray(SQLITE3_ASSOC)['children_count'];
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Sets a file with a certain id to deleted in the database
	*
	* @param $fileId integer that is the file_id you want to set to deleted in the database
	* @return TRUE on success and FALSE on failure
	* @author CV
	**/
	public function mediaDeleteFileWithId($fileId, $downloadsDir){
		try {

			$mediaItem = $this->mediaGetFileWithId($fileId, TRUE);
			if ($mediaItem == FALSE) {
				error_log("ERROR in DbManagerMedia->mediaDeleteFileWithId trying to delete a file that does not exist with id: {$fileId}");
				return FALSE;
			}

			// set the database to is_deleted
			$this->openDb();
			$stmt = $this->dbHandle->prepare("UPDATE media2 SET is_deleted = 1 WHERE file_id = :file_id");
			$stmt->bindValue(':file_id', $fileId);
			$success = $this->executeSqlStmt($stmt);

			// delete it's thumbnail if it has one
			$this->imagesDeleteImageFromMediaId($fileId);

			// delete it's corresponding thumbnail ... if there is one
			// $this->imagesDeleteImageFromMediaId($fileId);

			$filePathRel = $mediaItem['path_rel'];
			if ($filePathRel !== FALSE){
				$filePathAbs = $this->mediaPathToPathAbs($filePathRel);
				if (file_exists($filePathAbs)){
					// don't just use the php function unlink because it does not handle directoriesArray
					$cmd = "/usr/bin/sudo /bin/rm -Rf " . escapeshellarg($filePathAbs);
					SystemHelper::RunCmd($cmd);
					// unlink all files with this id in the downloads directory
					$wildcard = DOWNLOADS_DIR . DS . '*_' . $fileId;
					array_map("unlink", glob($wildcard));
				}
			}
			// case for deleting directories
			if ($mediaItem['mime_part1'] == self::MEDIA_MIME_DIRECTORY) {
				$attrsArray=array();
				$attrsArray['is_deleted'] = '1';
				$pathRelLike="$filePathRel%";
				$this->mediaUpdteAttrsForFilesWithParentPathRelLike($pathRelLike, $attrsArray);
			}
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Gets all of the media properties from the database fr a file given the path to that file
	*
	* @param $path a string representing the path to the file you want to get
	* @return Array of key/value pairs for cols/values in the database for the piece of media item on success and FALSE on failure
	* @author CV
	**/
	public function mediaGetFileWithPath($path, $excludeIsDeleted = FALSE) {
		try {
			$fileInfo = $this->fileGetInfoWithPath($path);
			$this->openDb();
			$stmtStr = "SELECT * FROM media2 WHERE parent_path_rel = :parent_path_rel AND file_name = :file_name AND file_extension = :file_extension";
			if ($excludeIsDeleted === TRUE) {
				$stmtStr .= ' AND is_deleted = 0';
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':parent_path_rel', $fileInfo['parent_path_rel']);
			$stmt->bindValue(':file_name', $fileInfo['file_name']);
			$stmt->bindValue(':file_extension', $fileInfo['file_extension']);
			$sqlite3Result = $stmt->execute();
			$resultArray = $sqlite3Result->fetchArray(SQLITE3_ASSOC);
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Gets all of the media properties from the database fr a file given the file_id of that file
	*
	* @param $file_id itneger representing the id of the file you want to get from the database
	* @param $addDetails if TRUE, add file details
	* @param $countOnlyWifiEnabledInChildren if TRUE, in children only count children that are available over WiFi
	* @return Array of key/value pairs for cols/values in the database for the piece of media item on success and FALSE on failure
	**/
	public function mediaGetFileWithId($fileId, $addDetails = FALSE, $countOnlyWifiEnabledInChildren = FALSE){
		try {
			// Allow people to return root directory if add details is selected
			if (self::MediaFileIdIsRootDirectory($fileId)) {
				$file = array(
					'file_id' => DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID,
				);
			} else {
				$this->openDb();
					$stmt = $this->dbHandle->prepare("SELECT * FROM media2 WHERE file_id = :file_id");
				$stmt->bindValue(':file_id', $fileId);
				$file = $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
			}

			if ($file === FALSE) {
				return FALSE;
			}
			if ($addDetails) {
				$file = $this->mediaAddFileDetailsToFile($file, $countOnlyWifiEnabledInChildren);
			}
			return $file;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get file with ID and then format it for a <select> tag as an array with a single item
	 * @param [see mediaGetFileWithId]
	 */
	public function mediaGetFileWithIdFormatAsSelectArray($fileId, $addDetails = FALSE, $countOnlyWifiEnabledInChildren = FALSE) {
		try {
			$mediaFileSelectArray = array();
			if ($fileId !== self::FILE_ID_DEFAULT) {
				$mediaFileSelectArray = self::MediaFormatMediaItemForSelect($this->mediaGetFileWithId($fileId, $addDetails, $countOnlyWifiEnabledInChildren));
			} 
			return $mediaFileSelectArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get featured media
	 * @return ArrayList featured media
	 */
	public function mediaGetFeaturedMedia($includeDetails = FALSE, $wifiEnabledOnly = FALSE, $ignoreDirectoryFilenamesArray = FALSE, $directoryFilename = FALSE) {
		try {
			$this->openDb();
			$stmtStr = "SELECT * FROM media2 WHERE featured = 1 AND is_deleted = 0";
			if ($wifiEnabledOnly === TRUE) {
				$stmtStr .= ' AND wifi_status > 0 AND is_global = 0';
			}
			// TODO Josiah
			// Get featured media in a language directory with title
			if ($directoryFilename !== FALSE) {
				$directoryFilename = \SQLite3::escapeString($directoryFilename);
				$stmtStr .= " AND parent_path_rel LIKE '{$directoryFilename}%'";
			}
			if ($ignoreDirectoryFilenamesArray !== FALSE) {
				foreach($ignoreDirectoryFilenamesArray as $ignoreDirectoryFilename) {
					$ignoreDirectoryFilename = \SQLite3::escapeString($ignoreDirectoryFilename);
					$stmtStr .= " AND parent_path_rel NOT LIKE '{$ignoreDirectoryFilename}%'";
				}
			}
			$stmtStr .= ' ORDER BY sort_order IS NULL, sort_order, title ASC';
			$stmt = $this->dbHandle->prepare($stmtStr);
			$resultArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($includeDetails === TRUE) {
				$resultArray = $this->mediaAddFileDetailsToMediaArray($resultArray, $wifiEnabledOnly);
			}
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		Set these media ids as the only featured ones
		TODO Comment
	*/
	public function mediaSetFeaturedOnlyWithIds($fileIds) {
		return $this->mediaSetAttributeEnabledWithIdArray($fileIds, 'featured', 1);
	}

	/*
		TODO comment
	*/
	public function mediaSetBluetoothEnabledWithIdArray($fileIds) {
		return $this->mediaSetAttributeEnabledWithIdArray($fileIds, 'bluetooth_status', 2);
	}

	private function mediaSetAttributeEnabledWithIdArray($fileIds, $attribute, $enabledValue) {
		try {
			$this->openDb();
			$attributeEscaped = \SQLite3::escapeString($attribute);
			$stmt = $this->dbHandle->prepare("UPDATE media2 SET {$attributeEscaped} = 0");
			if ($this->executeSqlStmt($stmt) === FALSE) {
				return FALSE;
			}
			$attrs = array();
			$attrs[$attribute] = $enabledValue;
			$success = TRUE;
			foreach ($fileIds as $fileId) {
				$success &= $this->mediaUpdateAttrsForFileWithId($fileId, $attrs);
			}
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function mediaSetWifiStatusDisabledForHomepageMedia() {
		try {
			$this->openDb();
			$stmt = $this->dbHandle->prepare("UPDATE media2 SET wifi_status = 0 WHERE parent_path_rel = ''");
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/*
		TODO Comment
	*/
	public static function MediaGetPathRelWithFile($file) {
		if (isset($file) && self::MediaFileIdIsRootDirectory($file['file_id'])) {
			return '';
		} else if (isset($file) && $file['mime_part1'] == self::MEDIA_MIME_DIRECTORY) {
			$pathRel = $file['parent_path_rel'] . $file['file_name'] . '/';
			return $pathRel;
		} else {
			$pathRel = $file['parent_path_rel'] . $file['file_name'];
			if (strlen($file['file_extension'])) {
				$pathRel .= '.' . $file['file_extension'];
			}
			return $pathRel;
		}
		return FALSE;
	}

	/*
		TODO Comment
	*/
	public static function MediaGetFilenameAndExtensionWithFile($file) {
		return $file['file_name'] . '.' . $file['file_extension'];
	}

	/*
		TODO Comment
	*/
	public function mediaMoveFileWithId($fileId, $newParentId) {
		try {
			// get the file
			if ($fileId === $newParentId) {
				error_log('Error: cannot move folder into itself.');
				return FALSE;
			}
			$file = $this->mediaGetFileWithId($fileId);
			if ($file === FALSE) {
				error_log('Error: file undefined');
				return FALSE; // file does not exist
			}
			if ($file['mime_part2'] === self::MEDIA_MIME_LANGUAGE_DIRECTORY) {
				error_log('Error: cannot move language folder');
				return FALSE;
			}
			if ($newParentId == self::MEDIA_ROOT_DIRECTORY_FILE_ID) { // If in root
				$newParentPathRel = '';
			} else {
				// get the directory
				$newParent = $this->mediaGetFileWithId($newParentId);
				if ($newParent === FALSE) {
					error_log('Error: New parent undefined');
					return FALSE; // the new parentId does not exist
				}
				// make sure the new parent is a directory
				if ($newParent['mime_part1'] != self::MEDIA_MIME_DIRECTORY) {
					error_log('Error: New parent is not a directory');
					return FALSE; // the second argument needs to be a file_id to a directory
				}
				$newParentPathRel = $this->mediaGetPathRelForFileObject($newParent);
			}
			// check to see if the new parent is inside of the file
			$filePathRel = $this->mediaGetPathRelForFileObject($file);
			$isDir = FALSE;
			if ($file['mime_part1'] == self::MEDIA_MIME_PART1) {
				$isDir = TRUE;
				if (strpos($newParentPathRel, $filePathRel) === 0) { // check if the new parent starts with the path to this file
					error_log('Error: Directory cannot be moved into itself.');
					return FALSE; // you can not move a directory inside of itself
				}
			}
			if ($this->mediaRenameFileWithId($fileId, $file['file_name'], $newParentPathRel, $file) == FALSE ) {
				error_log('Error: Rename Failed.');
				return FALSE;
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*
		TODO Comment
	*/
	public function mediaRenameFileWithId($fileId, $newName, $newParentPathRel = NULL, $file = NULL) {
		try {
			if ($file === NULl) {
				$file = $this->mediaGetFileWithId($fileId);
			}
			if ($file === FALSE) {
				error_log('Error: File does not exist');
				return FALSE; // file does not exist
			}
			// check if it is a directory
			$isDir = FALSE;
			if ($file['mime_part1'] === self::MEDIA_MIME_DIRECTORY) {
				$isDir = TRUE;
			}
			// check if there should be a new parent_path_rel
			if ($newParentPathRel === NULL) {
				$newParentPathRel = $file['parent_path_rel'];
			}

			$newName = \Renew\StringHelper::FormatFilename($newName);
			$newFilePathRel = $newParentPathRel . $newName;
			if (strlen($file['file_extension']) > 0) {
				$newFilePathRel = $newFilePathRel.'.'.$file['file_extension'];
			}
			$oldFilePathRel = $this->mediaGetPathRelForFileObject($file);
			$oldFilepathAbs = rtrim($this->mediaPathToPathAbs($oldFilePathRel), "/");
			$newFilePathAbs = $this->mediaPathToPathAbs($newFilePathRel);
			if ($oldFilepathAbs == $newFilePathAbs) {
				return TRUE; // it's not actually a move or a rename
			}

			// check for a naming conflict
			if (file_exists($newFilePathAbs)) { // checks weather file or directory exists
				error_log('Error: New file abs path does exist.');
				return FALSE; // a file with that same name already exists in the new parent directory
			}
			// move the file
			rename($oldFilepathAbs, $newFilePathAbs);
			// check for success
			if (!file_exists($newFilePathAbs)) { // checks weather file or directory exists
				error_log('Error: New file abs path does not exist.');
				return FALSE; // the move was unsuccessful
			}
			// set the new parent path rel for the file
			$file['parent_path_rel'] = $newParentPathRel;
			$file['file_name'] = $newName;
			$this->mediaUpdateAttrsForFileWithId($file['file_id'], $file);
			// set the new parent_path_rel for children
			if ($isDir) {
				$escapedFilePathRel = preg_quote($oldFilePathRel, '/');
				$escapedFilePathRel = "/^({$escapedFilePathRel})(.*)/";
				$newFilePathRel = "{$newFilePathRel}/$2";
				$stmt = $this->dbHandle->prepare("UPDATE media2 SET parent_path_rel = preg_replace(:escaped_file_path_rel, :new_file_path_rel, parent_path_rel)");
				$stmt->bindValue(':escaped_file_path_rel', $escapedFilePathRel);
				$stmt->bindValue(':new_file_path_rel', $newFilePathRel);
				$this->executeSqlStmt($stmt);

			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * @return FALSE on failure, rowId of the created language directory on success
	 */
	public function mediaCreateLanguageDirectoryWithLanguageIso($languageIso, $title = FALSE) {
		try {
			$success = TRUE;
			if (!$title) {
				$this->openDb();
					$language = $this->languagesGetWithIso($languageIso);
					if ($language !== FALSE) {
						$englishName = $language['name_english'];
						$localizedName = $language['name_localized'];
					} else {
						return FALSE;
					}
				$this->closeDb();

				if ($localizedName) {
					$title = $localizedName;
				} else {
					$title = $englishName;
				}
			}
			$title = \Renew\StringHelper::FormatFilename($title);
			$directoryTitle = $title . ' [' . $languageIso . ']';
			$rowId = $this->mediaCreateDirectoryWithFilenameAndParentId($directoryTitle, self::MEDIA_ROOT_DIRECTORY_FILE_ID, $languageIso);
			$success &= $rowId;
			if ($success !== FALSE) {
				// TODO: if this fails, delete the directory created?
				$success &= $this->mediaUpdateAttrsForFileWithId($rowId, array('title' => $title));
			}
			if ($success !== FALSE) {
				return $rowId;
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * returns FALSE on failure, and the rowId of the newly created directory on success
	 */
	public function mediaCreateDirectoryWithFilenameAndParentId($directoryTitle, $parentId, $languageIso = FALSE, $returnIdIfAlreadyExists = FALSE) {
		try {
			if ($parentId == NULL) {
				$parentId = 0;
			}
			$parentPathRel = '';
			if ($parentId != 0) {
				$parentFile = $this->mediaGetFileWithId($parentId);
				if($parentFile == FALSE || $parentFile['mime_part1'] != self::MEDIA_MIME_DIRECTORY){
					return FALSE;
				}
				$parentPathRel = $parentFile['parent_path_rel'] . $parentFile['file_name'] . '/';
			}
			$dirPathRel = $parentPathRel . $directoryTitle;
			$dirPathAbs = $this->mediaPathToPathAbs($dirPathRel);

			// If the folder exists, and the folder is in the database, don't create
			$dirMediaItem = $this->mediaGetFileWithPath($dirPathRel, TRUE);
			if (file_exists($dirPathAbs) && $dirMediaItem !== FALSE) {
				if ($returnIdIfAlreadyExists) {
					return $dirMediaItem['file_id'];
				}
				return FALSE; // error, can not create a directory that already exists
			}

			// otherwise, you may have a problem with permissions
			$success = $this->fileMkdirIfNotExists($dirPathAbs);
			if ($success === FALSE) {
				// error, failed to create directory
				return FALSE;
			}

			$rowId = $this->mediaAddFileWithPathToDb($dirPathAbs, NULL, NULL, TRUE, $languageIso);
			if ($rowId === FALSE) {
				rmdir($dirPathAbs);
				return FALSE; // error failed to add file to db
			}
			return $rowId;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Check to see if a file with a certain ID has a specific password
	 *
	 * @param $fileId the ID of the file
	 * @param $password the password to check
	 * @return Boolean TRUE if the password matches, FALSE on error or otherwise
	 */
	public function mediaPasswordsCheckFileIdHasPassword($fileId, $password) {
		try {
			$stmt = $this->dbHandle->prepare('SELECT password FROM media_passwords WHERE file_id = :file_id');
			$stmt->bindValue(':file_id', $fileId);
			$answer = $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
			$passwordMatches = FALSE;
			$hash = $this->mediaPasswordsHashPasswordWithFileId($fileId, $password);
			if ($hash !== FALSE && isset($answer['password']) && $answer['password'] === $hash) {
				$passwordMatches = TRUE;
			}
			return $passwordMatches;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Hash media password
	 *
	 * @param $password the password to hash
	 * @param $file the file to hash the password with
	 * @return String the hashed password
	 * 
	 */
	public function mediaPasswordsHashPasswordWithFileId($fileId, $password) {
		try {
			$file = $this->mediaGetFileWithId($fileId);
			if ($file !== FALSE) {
				$salt = $file['file_id'];
				return hash('sha256', $password.$salt);
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Set a specific password for a fileId
	 *
	 * @param $fileId the ID of the file
	 * @param $password the password to set for the fileId
	 * @return Boolean TRUE on success
	 * 
	 */
	public function mediaPasswordsSetPasswordWithFileId($fileId, $password) {
		try {
			// If password is blank, don't change anything
			if (empty($password)) {
				return TRUE;
			}
			$hash = $this->mediaPasswordsHashPasswordWithFileId($fileId, $password);
			if ($hash !== FALSE) {
				$stmt = $this->dbHandle->prepare('INSERT OR REPLACE INTO media_passwords (file_id, password) VALUES (:file_id, :password)');
				$stmt->bindValue(':file_id', $fileId);
				$stmt->bindValue(':password', $hash);
				return $this->executeSqlStmt($stmt);
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Remove password from table
	 *
	 * @param $fileId the ID of the file
	 * @return Boolean TRUE on success
	 * 
	 */
	public function mediaPasswordsRemovePasswordWithFileId($fileId) {
		try {
			$stmt = $this->dbHandle->prepare('DELETE FROM media_passwords WHERE file_id = :file_id');
			$stmt->bindValue(':file_id', $fileId);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Used to see if there are no media items that have a password (makes authentication faster)
	 * @return TRUE if any media items have a password currently
	 */
	public function mediaPasswordsGetProtectedFileIdArray() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT file_id FROM media2 WHERE is_password_protected != 0');
			$retVal = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($retVal === FALSE) {
				return FALSE;
			}
			$answerArray = array();
			foreach($retVal as $retArray) {
				$fileId = $retArray['file_id'];
				$answerArray[] = $fileId;
			}
			return $answerArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function fileGetInfoWithPath($path, $file = NULL){
		try {
			$pathAbs = $this->mediaPathToPathAbs($path);
			return $this->fileGetInfoWithPathAbs($pathAbs, $file);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function fileGetInfoWithPathAbs($pathAbs, $file = NULL) {
		try {
			$resultArray = array();
			if (!file_exists($pathAbs)){
				return FALSE;
			}
			if ($file == NULL){
				$file = new \SplFileInfo($pathAbs);
			}
			$fileName = '';
			$fileExtension = '';
			$parentPath = $file->getPath();
			$parentPathRel = $this->mediaPathToPathRel($parentPath.'/');
			$languageIso = '';
			$mimePart1 = '';
			$mimePart2 = '';
			$fileSizeBytes = 0;
			// Directory specific
			if ($file->isDir()) {
				$fileName = $file->getFileName();
				$mimePart1 = self::MEDIA_MIME_DIRECTORY;
				$mimePart2 = self::MEDIA_MIME_DIRECTORY;
			}
			// File (non-directory) specific
			else{
				$pathInfo = pathinfo($pathAbs);
				$fileName = $pathInfo['filename']; // file name without extension
				$fileExtension = $pathInfo['extension'];
				// support for hidden files
				if (strlen($fileName) === 0){
					$fileName = $file->getFileName();
					$fileExtension = '';
				}
				if (strlen($fileExtension) > 0) {
					$mime = self::GetMimeTypeForExtension($fileExtension);
				}
				else {
					$mime = mime_content_type($pathAbs);
				}
				// get the mime parts
				$mimeParts = explode("/", $mime);
				if (count($mimeParts) == 2){
					$mimePart1 = $mimeParts[0];
					$mimePart2 = $mimeParts[1];
				}
				try {
					$fileSizeBytes = $file->getSize();
				} catch (\Exception $e) {
					$fileSizeBytes = 0;
				}
			}
			// support for tagging files from Renew with format fileName[iso]
			// ... even for drectories
			if (strlen($fileName) >= 5) {
				$outputArray = array();
				preg_match("/\[([a-z][a-z][a-z])\]$/", $fileName, $outputArray);
				if (count($outputArray) > 1) {
					$languageIso = $outputArray[1];
				}
			}
			$resultArray['file_name'] = $fileName;
			$resultArray['file_extension'] = $fileExtension;
			$resultArray['parent_path'] = $parentPath;
			$resultArray['parent_path_rel'] = $parentPathRel;
			$resultArray['language_iso'] = $languageIso;
			$resultArray['mime_part1'] = $mimePart1;
			$resultArray['mime_part2'] = $mimePart2;
			$resultArray['file_size_bytes'] = $fileSizeBytes;
			$resultArray['path_abs'] = $pathAbs;
			return $resultArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	// returns a string with the hash on success and FALSE on failure
	public function fileHash($path){
		try {
			// TODO DEV ENVIRONMENT - This is a lot of dirnames. Remove this for the hardware, non-dev version?
			$basePath=dirname(dirname(dirname(dirname(dirname(__FILE__))))); // for dev environment
			putenv('PATH=/usr/local/bin:/usr/bin');
			$cmd = "\"{$basePath}/usr/bin/rwo_fastHash.sh\" \"{$path}\"";
			exec($cmd, $output, $return); // run the conversion
			$hash = '';
			if ($return == 0) {
				if (count($output) > 0){
					$hash = $output[0];
				}
			}
			if (strlen($hash) == 0){
				return FALSE;
			}
			return $hash; // success
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function fileMkdirIfNotExists($dirname){
		try {
			$oldMask = umask(0);
			if (!file_exists($dirname)) {
				mkdir($dirname, 0777, TRUE);
			}
			chmod("$dirname", 0777);
			umask($oldMask);
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}

	}

	public function filePathAbsIsInMediaDir($pathAbs){
		try {
			$mediaDir = $this->mediaGetDirPath();
			$escapedMediaDir = preg_quote($mediaDir, '/');
			if (preg_match("/^{$escapedMediaDir}.*/", $pathAbs)) {
				return TRUE;
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	
	public static function SetFilePermissions($filePath, $permissions = 0644) {
		$oldMask = umask(0);
		chmod("{$filePath}", $permissions);
		umask($oldMask);
	}

	public static function GetMimeTypeForExtension($extension) {
			$mimeTypes = array(
			'123' => 'application/vnd.lotus-1-2-3',
			'3dml' => 'text/vnd.in3d.3dml',
			'3ds' => 'image/x-3ds',
			'3g2' => 'video/3gpp2',
			'3gp' => 'video/3gpp',
			'7z' => 'application/x-7z-compressed',
			'aab' => 'application/x-authorware-bin',
			'aac' => 'audio/x-aac',
			'aam' => 'application/x-authorware-map',
			'aas' => 'application/x-authorware-seg',
			'abw' => 'application/x-abiword',
			'ac' => 'application/pkix-attr-cert',
			'acc' => 'application/vnd.americandynamics.acc',
			'ace' => 'application/x-ace-compressed',
			'acu' => 'application/vnd.acucobol',
			'acutc' => 'application/vnd.acucorp',
			'adp' => 'audio/adpcm',
			'aep' => 'application/vnd.audiograph',
			'afm' => 'application/x-font-type1',
			'afp' => 'application/vnd.ibm.modcap',
			'ahead' => 'application/vnd.ahead.space',
			'ai' => 'application/postscript',
			'aif' => 'audio/x-aiff',
			'aifc' => 'audio/x-aiff',
			'aiff' => 'audio/x-aiff',
			'air' => 'application/vnd.adobe.air-application-installer-package+zip',
			'ait' => 'application/vnd.dvb.ait',
			'ami' => 'application/vnd.amiga.ami',
			'apk' => 'application/vnd.android.package-archive',
			'appcache' => 'text/cache-manifest',
			'application' => 'application/x-ms-application',
			'apr' => 'application/vnd.lotus-approach',
			'arc' => 'application/x-freearc',
			'asc' => 'application/pgp-signature',
			'asf' => 'video/x-ms-asf',
			'asm' => 'text/x-asm',
			'aso' => 'application/vnd.accpac.simply.aso',
			'asx' => 'video/x-ms-asf',
			'atc' => 'application/vnd.acucorp',
			'atom' => 'application/atom+xml',
			'atomcat' => 'application/atomcat+xml',
			'atomsvc' => 'application/atomsvc+xml',
			'atx' => 'application/vnd.antix.game-component',
			'au' => 'audio/basic',
			'avi' => 'video/x-msvideo',
			'aw' => 'application/applixware',
			'azf' => 'application/vnd.airzip.filesecure.azf',
			'azs' => 'application/vnd.airzip.filesecure.azs',
			'azw' => 'application/vnd.amazon.ebook',
			'bat' => 'application/x-msdownload',
			'bcpio' => 'application/x-bcpio',
			'bdf' => 'application/x-font-bdf',
			'bdm' => 'application/vnd.syncml.dm+wbxml',
			'bed' => 'application/vnd.realvnc.bed',
			'bh2' => 'application/vnd.fujitsu.oasysprs',
			'bin' => 'application/octet-stream',
			'blb' => 'application/x-blorb',
			'blorb' => 'application/x-blorb',
			'bmi' => 'application/vnd.bmi',
			'bmp' => 'image/bmp',
			'book' => 'application/vnd.framemaker',
			'box' => 'application/vnd.previewsystems.box',
			'boz' => 'application/x-bzip2',
			'bpk' => 'application/octet-stream',
			'btif' => 'image/prs.btif',
			'bz' => 'application/x-bzip',
			'bz2' => 'application/x-bzip2',
			'c' => 'text/x-c',
			'c11amc' => 'application/vnd.cluetrust.cartomobile-config',
			'c11amz' => 'application/vnd.cluetrust.cartomobile-config-pkg',
			'c4d' => 'application/vnd.clonk.c4group',
			'c4f' => 'application/vnd.clonk.c4group',
			'c4g' => 'application/vnd.clonk.c4group',
			'c4p' => 'application/vnd.clonk.c4group',
			'c4u' => 'application/vnd.clonk.c4group',
			'cab' => 'application/vnd.ms-cab-compressed',
			'caf' => 'audio/x-caf',
			'cap' => 'application/vnd.tcpdump.pcap',
			'car' => 'application/vnd.curl.car',
			'cat' => 'application/vnd.ms-pki.seccat',
			'cb7' => 'application/x-cbr',
			'cba' => 'application/x-cbr',
			'cbr' => 'application/x-cbr',
			'cbt' => 'application/x-cbr',
			'cbz' => 'application/x-cbr',
			'cc' => 'text/x-c',
			'cct' => 'application/x-director',
			'ccxml' => 'application/ccxml+xml',
			'cdbcmsg' => 'application/vnd.contact.cmsg',
			'cdf' => 'application/x-netcdf',
			'cdkey' => 'application/vnd.mediastation.cdkey',
			'cdmia' => 'application/cdmi-capability',
			'cdmic' => 'application/cdmi-container',
			'cdmid' => 'application/cdmi-domain',
			'cdmio' => 'application/cdmi-object',
			'cdmiq' => 'application/cdmi-queue',
			'cdx' => 'chemical/x-cdx',
			'cdxml' => 'application/vnd.chemdraw+xml',
			'cdy' => 'application/vnd.cinderella',
			'cer' => 'application/pkix-cert',
			'cfs' => 'application/x-cfs-compressed',
			'cgm' => 'image/cgm',
			'chat' => 'application/x-chat',
			'chm' => 'application/vnd.ms-htmlhelp',
			'chrt' => 'application/vnd.kde.kchart',
			'cif' => 'chemical/x-cif',
			'cii' => 'application/vnd.anser-web-certificate-issue-initiation',
			'cil' => 'application/vnd.ms-artgalry',
			'cla' => 'application/vnd.claymore',
			'class' => 'application/java-vm',
			'clkk' => 'application/vnd.crick.clicker.keyboard',
			'clkp' => 'application/vnd.crick.clicker.palette',
			'clkt' => 'application/vnd.crick.clicker.template',
			'clkw' => 'application/vnd.crick.clicker.wordbank',
			'clkx' => 'application/vnd.crick.clicker',
			'clp' => 'application/x-msclip',
			'cmc' => 'application/vnd.cosmocaller',
			'cmdf' => 'chemical/x-cmdf',
			'cml' => 'chemical/x-cml',
			'cmp' => 'application/vnd.yellowriver-custom-menu',
			'cmx' => 'image/x-cmx',
			'cod' => 'application/vnd.rim.cod',
			'com' => 'application/x-msdownload',
			'conf' => 'text/plain',
			'cpio' => 'application/x-cpio',
			'cpp' => 'text/x-c',
			'cpt' => 'application/mac-compactpro',
			'crd' => 'application/x-mscardfile',
			'crl' => 'application/pkix-crl',
			'crt' => 'application/x-x509-ca-cert',
			'cryptonote' => 'application/vnd.rig.cryptonote',
			'csh' => 'application/x-csh',
			'csml' => 'chemical/x-csml',
			'csp' => 'application/vnd.commonspace',
			'css' => 'text/css',
			'cst' => 'application/x-director',
			'csv' => 'text/csv',
			'cu' => 'application/cu-seeme',
			'curl' => 'text/vnd.curl',
			'cww' => 'application/prs.cww',
			'cxt' => 'application/x-director',
			'cxx' => 'text/x-c',
			'dae' => 'model/vnd.collada+xml',
			'daf' => 'application/vnd.mobius.daf',
			'dart' => 'application/vnd.dart',
			'dataless' => 'application/vnd.fdsn.seed',
			'davmount' => 'application/davmount+xml',
			'dbk' => 'application/docbook+xml',
			'dcr' => 'application/x-director',
			'dcurl' => 'text/vnd.curl.dcurl',
			'dd2' => 'application/vnd.oma.dd2+xml',
			'ddd' => 'application/vnd.fujixerox.ddd',
			'deb' => 'application/x-debian-package',
			'def' => 'text/plain',
			'deploy' => 'application/octet-stream',
			'der' => 'application/x-x509-ca-cert',
			'dfac' => 'application/vnd.dreamfactory',
			'dgc' => 'application/x-dgc-compressed',
			'dic' => 'text/x-c',
			'dir' => 'application/x-director',
			'dis' => 'application/vnd.mobius.dis',
			'dist' => 'application/octet-stream',
			'distz' => 'application/octet-stream',
			'djv' => 'image/vnd.djvu',
			'djvu' => 'image/vnd.djvu',
			'dll' => 'application/x-msdownload',
			'dmg' => 'application/x-apple-diskimage',
			'dmp' => 'application/vnd.tcpdump.pcap',
			'dms' => 'application/octet-stream',
			'dna' => 'application/vnd.dna',
			'doc' => 'application/msword',
			'docm' => 'application/vnd.ms-word.document.macroenabled.12',
			'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
			'dot' => 'application/msword',
			'dotm' => 'application/vnd.ms-word.template.macroenabled.12',
			'dotx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
			'dp' => 'application/vnd.osgi.dp',
			'dpg' => 'application/vnd.dpgraph',
			'dra' => 'audio/vnd.dra',
			'dsc' => 'text/prs.lines.tag',
			'dssc' => 'application/dssc+der',
			'dtb' => 'application/x-dtbook+xml',
			'dtd' => 'application/xml-dtd',
			'dts' => 'audio/vnd.dts',
			'dtshd' => 'audio/vnd.dts.hd',
			'dump' => 'application/octet-stream',
			'dvb' => 'video/vnd.dvb.file',
			'dvi' => 'application/x-dvi',
			'dwf' => 'model/vnd.dwf',
			'dwg' => 'image/vnd.dwg',
			'dxf' => 'image/vnd.dxf',
			'dxp' => 'application/vnd.spotfire.dxp',
			'dxr' => 'application/x-director',
			'ecelp4800' => 'audio/vnd.nuera.ecelp4800',
			'ecelp7470' => 'audio/vnd.nuera.ecelp7470',
			'ecelp9600' => 'audio/vnd.nuera.ecelp9600',
			'ecma' => 'application/ecmascript',
			'edm' => 'application/vnd.novadigm.edm',
			'edx' => 'application/vnd.novadigm.edx',
			'efif' => 'application/vnd.picsel',
			'ei6' => 'application/vnd.pg.osasli',
			'elc' => 'application/octet-stream',
			'emf' => 'application/x-msmetafile',
			'eml' => 'message/rfc822',
			'emma' => 'application/emma+xml',
			'emz' => 'application/x-msmetafile',
			'eol' => 'audio/vnd.digital-winds',
			'eot' => 'application/vnd.ms-fontobject',
			'eps' => 'application/postscript',
			'epub' => 'application/epub+zip',
			'es3' => 'application/vnd.eszigno3+xml',
			'esa' => 'application/vnd.osgi.subsystem',
			'esf' => 'application/vnd.epson.esf',
			'et3' => 'application/vnd.eszigno3+xml',
			'etx' => 'text/x-setext',
			'eva' => 'application/x-eva',
			'evy' => 'application/x-envoy',
			'exe' => 'application/x-msdownload',
			'exi' => 'application/exi',
			'ext' => 'application/vnd.novadigm.ext',
			'ez' => 'application/andrew-inset',
			'ez2' => 'application/vnd.ezpix-album',
			'ez3' => 'application/vnd.ezpix-package',
			'f' => 'text/x-fortran',
			'f4v' => 'video/x-f4v',
			'f77' => 'text/x-fortran',
			'f90' => 'text/x-fortran',
			'fbs' => 'image/vnd.fastbidsheet',
			'fcdt' => 'application/vnd.adobe.formscentral.fcdt',
			'fcs' => 'application/vnd.isac.fcs',
			'fdf' => 'application/vnd.fdf',
			'fe_launch' => 'application/vnd.denovo.fcselayout-link',
			'fg5' => 'application/vnd.fujitsu.oasysgp',
			'fgd' => 'application/x-director',
			'fh' => 'image/x-freehand',
			'fh4' => 'image/x-freehand',
			'fh5' => 'image/x-freehand',
			'fh7' => 'image/x-freehand',
			'fhc' => 'image/x-freehand',
			'fig' => 'application/x-xfig',
			'flac' => 'audio/x-flac',
			'fli' => 'video/x-fli',
			'flo' => 'application/vnd.micrografx.flo',
			'flv' => 'video/x-flv',
			'flw' => 'application/vnd.kde.kivio',
			'flx' => 'text/vnd.fmi.flexstor',
			'fly' => 'text/vnd.fly',
			'fm' => 'application/vnd.framemaker',
			'fnc' => 'application/vnd.frogans.fnc',
			'for' => 'text/x-fortran',
			'fpx' => 'image/vnd.fpx',
			'frame' => 'application/vnd.framemaker',
			'fsc' => 'application/vnd.fsc.weblaunch',
			'fst' => 'image/vnd.fst',
			'ftc' => 'application/vnd.fluxtime.clip',
			'fti' => 'application/vnd.anser-web-funds-transfer-initiation',
			'fvt' => 'video/vnd.fvt',
			'fxp' => 'application/vnd.adobe.fxp',
			'fxpl' => 'application/vnd.adobe.fxp',
			'fzs' => 'application/vnd.fuzzysheet',
			'g2w' => 'application/vnd.geoplan',
			'g3' => 'image/g3fax',
			'g3w' => 'application/vnd.geospace',
			'gac' => 'application/vnd.groove-account',
			'gam' => 'application/x-tads',
			'gbr' => 'application/rpki-ghostbusters',
			'gca' => 'application/x-gca-compressed',
			'gdl' => 'model/vnd.gdl',
			'geo' => 'application/vnd.dynageo',
			'gex' => 'application/vnd.geometry-explorer',
			'ggb' => 'application/vnd.geogebra.file',
			'ggt' => 'application/vnd.geogebra.tool',
			'ghf' => 'application/vnd.groove-help',
			'gif' => 'image/gif',
			'gim' => 'application/vnd.groove-identity-message',
			'gml' => 'application/gml+xml',
			'gmx' => 'application/vnd.gmx',
			'gnumeric' => 'application/x-gnumeric',
			'gph' => 'application/vnd.flographit',
			'gpx' => 'application/gpx+xml',
			'gqf' => 'application/vnd.grafeq',
			'gqs' => 'application/vnd.grafeq',
			'gram' => 'application/srgs',
			'gramps' => 'application/x-gramps-xml',
			'gre' => 'application/vnd.geometry-explorer',
			'grv' => 'application/vnd.groove-injector',
			'grxml' => 'application/srgs+xml',
			'gsf' => 'application/x-font-ghostscript',
			'gtar' => 'application/x-gtar',
			'gtm' => 'application/vnd.groove-tool-message',
			'gtw' => 'model/vnd.gtw',
			'gv' => 'text/vnd.graphviz',
			'gxf' => 'application/gxf',
			'gxt' => 'application/vnd.geonext',
			'h' => 'text/x-c',
			'h261' => 'video/h261',
			'h263' => 'video/h263',
			'h264' => 'video/h264',
			'hal' => 'application/vnd.hal+xml',
			'hbci' => 'application/vnd.hbci',
			'hdf' => 'application/x-hdf',
			'hh' => 'text/x-c',
			'hlp' => 'application/winhlp',
			'hpgl' => 'application/vnd.hp-hpgl',
			'hpid' => 'application/vnd.hp-hpid',
			'hps' => 'application/vnd.hp-hps',
			'hqx' => 'application/mac-binhex40',
			'htke' => 'application/vnd.kenameaapp',
			'htm' => 'text/html',
			'html' => 'text/html',
			'hvd' => 'application/vnd.yamaha.hv-dic',
			'hvp' => 'application/vnd.yamaha.hv-voice',
			'hvs' => 'application/vnd.yamaha.hv-script',
			'i2g' => 'application/vnd.intergeo',
			'icc' => 'application/vnd.iccprofile',
			'ice' => 'x-conference/x-cooltalk',
			'icm' => 'application/vnd.iccprofile',
			'ico' => 'image/x-icon',
			'ics' => 'text/calendar',
			'ief' => 'image/ief',
			'ifb' => 'text/calendar',
			'ifm' => 'application/vnd.shana.informed.formdata',
			'iges' => 'model/iges',
			'igl' => 'application/vnd.igloader',
			'igm' => 'application/vnd.insors.igm',
			'igs' => 'model/iges',
			'igx' => 'application/vnd.micrografx.igx',
			'iif' => 'application/vnd.shana.informed.interchange',
			'imp' => 'application/vnd.accpac.simply.imp',
			'ims' => 'application/vnd.ms-ims',
			'in' => 'text/plain',
			'ink' => 'application/inkml+xml',
			'inkml' => 'application/inkml+xml',
			'install' => 'application/x-install-instructions',
			'iota' => 'application/vnd.astraea-software.iota',
			'ipfix' => 'application/ipfix',
			'ipk' => 'application/vnd.shana.informed.package',
			'irm' => 'application/vnd.ibm.rights-management',
			'irp' => 'application/vnd.irepository.package+xml',
			'iso' => 'application/x-iso9660-image',
			'itp' => 'application/vnd.shana.informed.formtemplate',
			'ivp' => 'application/vnd.immervision-ivp',
			'ivu' => 'application/vnd.immervision-ivu',
			'jad' => 'text/vnd.sun.j2me.app-descriptor',
			'jam' => 'application/vnd.jam',
			'jar' => 'application/java-archive',
			'java' => 'text/x-java-source',
			'jisp' => 'application/vnd.jisp',
			'jlt' => 'application/vnd.hp-jlyt',
			'jnlp' => 'application/x-java-jnlp-file',
			'joda' => 'application/vnd.joost.joda-archive',
			'jpe' => 'image/jpeg',
			'jpeg' => 'image/jpeg',
			'jpg' => 'image/jpeg',
			'jpgm' => 'video/jpm',
			'jpgv' => 'video/jpeg',
			'jpm' => 'video/jpm',
			'js' => 'application/javascript',
			'json' => 'application/json',
			'jsonml' => 'application/jsonml+json',
			'kar' => 'audio/midi',
			'karbon' => 'application/vnd.kde.karbon',
			'kfo' => 'application/vnd.kde.kformula',
			'kia' => 'application/vnd.kidspiration',
			'kml' => 'application/vnd.google-earth.kml+xml',
			'kmz' => 'application/vnd.google-earth.kmz',
			'kne' => 'application/vnd.kinar',
			'knp' => 'application/vnd.kinar',
			'kon' => 'application/vnd.kde.kontour',
			'kpr' => 'application/vnd.kde.kpresenter',
			'kpt' => 'application/vnd.kde.kpresenter',
			'kpxx' => 'application/vnd.ds-keypoint',
			'ksp' => 'application/vnd.kde.kspread',
			'ktr' => 'application/vnd.kahootz',
			'ktx' => 'image/ktx',
			'ktz' => 'application/vnd.kahootz',
			'kwd' => 'application/vnd.kde.kword',
			'kwt' => 'application/vnd.kde.kword',
			'lasxml' => 'application/vnd.las.las+xml',
			'latex' => 'application/x-latex',
			'lbd' => 'application/vnd.llamagraphics.life-balance.desktop',
			'lbe' => 'application/vnd.llamagraphics.life-balance.exchange+xml',
			'les' => 'application/vnd.hhe.lesson-player',
			'lha' => 'application/x-lzh-compressed',
			'link66' => 'application/vnd.route66.link66+xml',
			'list' => 'text/plain',
			'list3820' => 'application/vnd.ibm.modcap',
			'listafp' => 'application/vnd.ibm.modcap',
			'lnk' => 'application/x-ms-shortcut',
			'log' => 'text/plain',
			'lostxml' => 'application/lost+xml',
			'lrf' => 'application/octet-stream',
			'lrm' => 'application/vnd.ms-lrm',
			'ltf' => 'application/vnd.frogans.ltf',
			'lvp' => 'audio/vnd.lucent.voice',
			'lwp' => 'application/vnd.lotus-wordpro',
			'lzh' => 'application/x-lzh-compressed',
			'm13' => 'application/x-msmediaview',
			'm14' => 'application/x-msmediaview',
			'm1v' => 'video/mpeg',
			'm21' => 'application/mp21',
			'm2a' => 'audio/mpeg',
			'm2v' => 'video/mpeg',
			'm3a' => 'audio/mpeg',
			'm3u' => 'audio/x-mpegurl',
			'm3u8' => 'application/vnd.apple.mpegurl',
			'm4a' => 'audio/mp4',
			'm4u' => 'video/vnd.mpegurl',
			'm4v' => 'video/x-m4v',
			'ma' => 'application/mathematica',
			'mads' => 'application/mads+xml',
			'mag' => 'application/vnd.ecowin.chart',
			'maker' => 'application/vnd.framemaker',
			'man' => 'text/troff',
			'mar' => 'application/octet-stream',
			'mathml' => 'application/mathml+xml',
			'mb' => 'application/mathematica',
			'mbk' => 'application/vnd.mobius.mbk',
			'mbox' => 'application/mbox',
			'mc1' => 'application/vnd.medcalcdata',
			'mcd' => 'application/vnd.mcd',
			'mcurl' => 'text/vnd.curl.mcurl',
			'mdb' => 'application/x-msaccess',
			'mdi' => 'image/vnd.ms-modi',
			'me' => 'text/troff',
			'mesh' => 'model/mesh',
			'meta4' => 'application/metalink4+xml',
			'metalink' => 'application/metalink+xml',
			'mets' => 'application/mets+xml',
			'mfm' => 'application/vnd.mfmp',
			'mft' => 'application/rpki-manifest',
			'mgp' => 'application/vnd.osgeo.mapguide.package',
			'mgz' => 'application/vnd.proteus.magazine',
			'mid' => 'audio/midi',
			'midi' => 'audio/midi',
			'mie' => 'application/x-mie',
			'mif' => 'application/vnd.mif',
			'mime' => 'message/rfc822',
			'mj2' => 'video/mj2',
			'mjp2' => 'video/mj2',
			'mk3d' => 'video/x-matroska',
			'mka' => 'audio/x-matroska',
			'mks' => 'video/x-matroska',
			'mkv' => 'video/x-matroska',
			'mlp' => 'application/vnd.dolby.mlp',
			'mmd' => 'application/vnd.chipnuts.karaoke-mmd',
			'mmf' => 'application/vnd.smaf',
			'mmr' => 'image/vnd.fujixerox.edmics-mmr',
			'mng' => 'video/x-mng',
			'mny' => 'application/x-msmoney',
			'mobi' => 'application/x-mobipocket-ebook',
			'mods' => 'application/mods+xml',
			'mov' => 'video/quicktime',
			'movie' => 'video/x-sgi-movie',
			'mp2' => 'audio/mpeg',
			'mp21' => 'application/mp21',
			'mp2a' => 'audio/mpeg',
			'mp3' => 'audio/mpeg',
			'mp4' => 'video/mp4',
			'mp4a' => 'audio/mp4',
			'mp4s' => 'application/mp4',
			'mp4v' => 'video/mp4',
			'mpc' => 'application/vnd.mophun.certificate',
			'mpe' => 'video/mpeg',
			'mpeg' => 'video/mpeg',
			'mpg' => 'video/mpeg',
			'mpg4' => 'video/mp4',
			'mpga' => 'audio/mpeg',
			'mpkg' => 'application/vnd.apple.installer+xml',
			'mpm' => 'application/vnd.blueice.multipass',
			'mpn' => 'application/vnd.mophun.application',
			'mpp' => 'application/vnd.ms-project',
			'mpt' => 'application/vnd.ms-project',
			'mpy' => 'application/vnd.ibm.minipay',
			'mqy' => 'application/vnd.mobius.mqy',
			'mrc' => 'application/marc',
			'mrcx' => 'application/marcxml+xml',
			'ms' => 'text/troff',
			'mscml' => 'application/mediaservercontrol+xml',
			'mseed' => 'application/vnd.fdsn.mseed',
			'mseq' => 'application/vnd.mseq',
			'msf' => 'application/vnd.epson.msf',
			'msh' => 'model/mesh',
			'msi' => 'application/x-msdownload',
			'msl' => 'application/vnd.mobius.msl',
			'msty' => 'application/vnd.muvee.style',
			'mts' => 'model/vnd.mts',
			'mus' => 'application/vnd.musician',
			'musicxml' => 'application/vnd.recordare.musicxml+xml',
			'mvb' => 'application/x-msmediaview',
			'mwf' => 'application/vnd.mfer',
			'mxf' => 'application/mxf',
			'mxl' => 'application/vnd.recordare.musicxml',
			'mxml' => 'application/xv+xml',
			'mxs' => 'application/vnd.triscape.mxs',
			'mxu' => 'video/vnd.mpegurl',
			'n-gage' => 'application/vnd.nokia.n-gage.symbian.install',
			'n3' => 'text/n3',
			'nb' => 'application/mathematica',
			'nbp' => 'application/vnd.wolfram.player',
			'nc' => 'application/x-netcdf',
			'ncx' => 'application/x-dtbncx+xml',
			'nfo' => 'text/x-nfo',
			'ngdat' => 'application/vnd.nokia.n-gage.data',
			'nitf' => 'application/vnd.nitf',
			'nlu' => 'application/vnd.neurolanguage.nlu',
			'nml' => 'application/vnd.enliven',
			'nnd' => 'application/vnd.noblenet-directory',
			'nns' => 'application/vnd.noblenet-sealer',
			'nnw' => 'application/vnd.noblenet-web',
			'npx' => 'image/vnd.net-fpx',
			'nsc' => 'application/x-conference',
			'nsf' => 'application/vnd.lotus-notes',
			'ntf' => 'application/vnd.nitf',
			'nzb' => 'application/x-nzb',
			'oa2' => 'application/vnd.fujitsu.oasys2',
			'oa3' => 'application/vnd.fujitsu.oasys3',
			'oas' => 'application/vnd.fujitsu.oasys',
			'obd' => 'application/x-msbinder',
			'obj' => 'application/x-tgif',
			'oda' => 'application/oda',
			'odb' => 'application/vnd.oasis.opendocument.database',
			'odc' => 'application/vnd.oasis.opendocument.chart',
			'odf' => 'application/vnd.oasis.opendocument.formula',
			'odft' => 'application/vnd.oasis.opendocument.formula-template',
			'odg' => 'application/vnd.oasis.opendocument.graphics',
			'odi' => 'application/vnd.oasis.opendocument.image',
			'odm' => 'application/vnd.oasis.opendocument.text-master',
			'odp' => 'application/vnd.oasis.opendocument.presentation',
			'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
			'odt' => 'application/vnd.oasis.opendocument.text',
			'oga' => 'audio/ogg',
			'ogg' => 'audio/ogg',
			'ogv' => 'video/ogg',
			'ogx' => 'application/ogg',
			'omdoc' => 'application/omdoc+xml',
			'onepkg' => 'application/onenote',
			'onetmp' => 'application/onenote',
			'onetoc' => 'application/onenote',
			'onetoc2' => 'application/onenote',
			'opf' => 'application/oebps-package+xml',
			'opml' => 'text/x-opml',
			'oprc' => 'application/vnd.palm',
			'org' => 'application/vnd.lotus-organizer',
			'osf' => 'application/vnd.yamaha.openscoreformat',
			'osfpvg' => 'application/vnd.yamaha.openscoreformat.osfpvg+xml',
			'otc' => 'application/vnd.oasis.opendocument.chart-template',
			'otf' => 'font/otf',
			'otg' => 'application/vnd.oasis.opendocument.graphics-template',
			'oth' => 'application/vnd.oasis.opendocument.text-web',
			'oti' => 'application/vnd.oasis.opendocument.image-template',
			'otp' => 'application/vnd.oasis.opendocument.presentation-template',
			'ots' => 'application/vnd.oasis.opendocument.spreadsheet-template',
			'ott' => 'application/vnd.oasis.opendocument.text-template',
			'oxps' => 'application/oxps',
			'oxt' => 'application/vnd.openofficeorg.extension',
			'p' => 'text/x-pascal',
			'p10' => 'application/pkcs10',
			'p12' => 'application/x-pkcs12',
			'p7b' => 'application/x-pkcs7-certificates',
			'p7c' => 'application/pkcs7-mime',
			'p7m' => 'application/pkcs7-mime',
			'p7r' => 'application/x-pkcs7-certreqresp',
			'p7s' => 'application/pkcs7-signature',
			'p8' => 'application/pkcs8',
			'pas' => 'text/x-pascal',
			'paw' => 'application/vnd.pawaafile',
			'pbd' => 'application/vnd.powerbuilder6',
			'pbm' => 'image/x-portable-bitmap',
			'pcap' => 'application/vnd.tcpdump.pcap',
			'pcf' => 'application/x-font-pcf',
			'pcl' => 'application/vnd.hp-pcl',
			'pclxl' => 'application/vnd.hp-pclxl',
			'pct' => 'image/x-pict',
			'pcurl' => 'application/vnd.curl.pcurl',
			'pcx' => 'image/x-pcx',
			'pdb' => 'application/vnd.palm',
			'pdf' => 'application/pdf',
			'pfa' => 'application/x-font-type1',
			'pfb' => 'application/x-font-type1',
			'pfm' => 'application/x-font-type1',
			'pfr' => 'application/font-tdpfr',
			'pfx' => 'application/x-pkcs12',
			'pgm' => 'image/x-portable-graymap',
			'pgn' => 'application/x-chess-pgn',
			'pgp' => 'application/pgp-encrypted',
			'pic' => 'image/x-pict',
			'pkg' => 'application/octet-stream',
			'pki' => 'application/pkixcmp',
			'pkipath' => 'application/pkix-pkipath',
			'plb' => 'application/vnd.3gpp.pic-bw-large',
			'plc' => 'application/vnd.mobius.plc',
			'plf' => 'application/vnd.pocketlearn',
			'pls' => 'application/pls+xml',
			'pml' => 'application/vnd.ctc-posml',
			'png' => 'image/png',
			'pnm' => 'image/x-portable-anymap',
			'portpkg' => 'application/vnd.macports.portpkg',
			'pot' => 'application/vnd.ms-powerpoint',
			'potm' => 'application/vnd.ms-powerpoint.template.macroenabled.12',
			'potx' => 'application/vnd.openxmlformats-officedocument.presentationml.template',
			'ppam' => 'application/vnd.ms-powerpoint.addin.macroenabled.12',
			'ppd' => 'application/vnd.cups-ppd',
			'ppm' => 'image/x-portable-pixmap',
			'pps' => 'application/vnd.ms-powerpoint',
			'ppsm' => 'application/vnd.ms-powerpoint.slideshow.macroenabled.12',
			'ppsx' => 'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
			'ppt' => 'application/vnd.ms-powerpoint',
			'pptm' => 'application/vnd.ms-powerpoint.presentation.macroenabled.12',
			'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
			'pqa' => 'application/vnd.palm',
			'prc' => 'application/x-mobipocket-ebook',
			'pre' => 'application/vnd.lotus-freelance',
			'prf' => 'application/pics-rules',
			'ps' => 'application/postscript',
			'psb' => 'application/vnd.3gpp.pic-bw-small',
			'psd' => 'image/vnd.adobe.photoshop',
			'psf' => 'application/x-font-linux-psf',
			'pskcxml' => 'application/pskc+xml',
			'ptid' => 'application/vnd.pvi.ptid1',
			'pub' => 'application/x-mspublisher',
			'pvb' => 'application/vnd.3gpp.pic-bw-var',
			'pwn' => 'application/vnd.3m.post-it-notes',
			'pya' => 'audio/vnd.ms-playready.media.pya',
			'pyv' => 'video/vnd.ms-playready.media.pyv',
			'qam' => 'application/vnd.epson.quickanime',
			'qbo' => 'application/vnd.intu.qbo',
			'qfx' => 'application/vnd.intu.qfx',
			'qps' => 'application/vnd.publishare-delta-tree',
			'qt' => 'video/quicktime',
			'qwd' => 'application/vnd.quark.quarkxpress',
			'qwt' => 'application/vnd.quark.quarkxpress',
			'qxb' => 'application/vnd.quark.quarkxpress',
			'qxd' => 'application/vnd.quark.quarkxpress',
			'qxl' => 'application/vnd.quark.quarkxpress',
			'qxt' => 'application/vnd.quark.quarkxpress',
			'ra' => 'audio/x-pn-realaudio',
			'ram' => 'audio/x-pn-realaudio',
			'rar' => 'application/x-rar-compressed',
			'ras' => 'image/x-cmu-raster',
			'rcprofile' => 'application/vnd.ipunplugged.rcprofile',
			'rdf' => 'application/rdf+xml',
			'rdz' => 'application/vnd.data-vision.rdz',
			'rep' => 'application/vnd.businessobjects',
			'res' => 'application/x-dtbresource+xml',
			'rgb' => 'image/x-rgb',
			'rif' => 'application/reginfo+xml',
			'rip' => 'audio/vnd.rip',
			'ris' => 'application/x-research-info-systems',
			'rl' => 'application/resource-lists+xml',
			'rlc' => 'image/vnd.fujixerox.edmics-rlc',
			'rld' => 'application/resource-lists-diff+xml',
			'rm' => 'application/vnd.rn-realmedia',
			'rmi' => 'audio/midi',
			'rmp' => 'audio/x-pn-realaudio-plugin',
			'rms' => 'application/vnd.jcp.javame.midlet-rms',
			'rmvb' => 'application/vnd.rn-realmedia-vbr',
			'rnc' => 'application/relax-ng-compact-syntax',
			'roa' => 'application/rpki-roa',
			'roff' => 'text/troff',
			'rp9' => 'application/vnd.cloanto.rp9',
			'rpss' => 'application/vnd.nokia.radio-presets',
			'rpst' => 'application/vnd.nokia.radio-preset',
			'rq' => 'application/sparql-query',
			'rs' => 'application/rls-services+xml',
			'rsd' => 'application/rsd+xml',
			'rss' => 'application/rss+xml',
			'rtf' => 'application/rtf',
			'rtx' => 'text/richtext',
			's' => 'text/x-asm',
			's3m' => 'audio/s3m',
			'saf' => 'application/vnd.yamaha.smaf-audio',
			'sbml' => 'application/sbml+xml',
			'sc' => 'application/vnd.ibm.secure-container',
			'scd' => 'application/x-msschedule',
			'scm' => 'application/vnd.lotus-screencam',
			'scq' => 'application/scvp-cv-request',
			'scs' => 'application/scvp-cv-response',
			'scurl' => 'text/vnd.curl.scurl',
			'sda' => 'application/vnd.stardivision.draw',
			'sdc' => 'application/vnd.stardivision.calc',
			'sdd' => 'application/vnd.stardivision.impress',
			'sdkd' => 'application/vnd.solent.sdkm+xml',
			'sdkm' => 'application/vnd.solent.sdkm+xml',
			'sdp' => 'application/sdp',
			'sdw' => 'application/vnd.stardivision.writer',
			'see' => 'application/vnd.seemail',
			'seed' => 'application/vnd.fdsn.seed',
			'sema' => 'application/vnd.sema',
			'semd' => 'application/vnd.semd',
			'semf' => 'application/vnd.semf',
			'ser' => 'application/java-serialized-object',
			'setpay' => 'application/set-payment-initiation',
			'setreg' => 'application/set-registration-initiation',
			'sfd-hdstx' => 'application/vnd.hydrostatix.sof-data',
			'sfs' => 'application/vnd.spotfire.sfs',
			'sfv' => 'text/x-sfv',
			'sgi' => 'image/sgi',
			'sgl' => 'application/vnd.stardivision.writer-global',
			'sgm' => 'text/sgml',
			'sgml' => 'text/sgml',
			'sh' => 'application/x-sh',
			'shar' => 'application/x-shar',
			'shf' => 'application/shf+xml',
			'sid' => 'image/x-mrsid-image',
			'sig' => 'application/pgp-signature',
			'sil' => 'audio/silk',
			'silo' => 'model/mesh',
			'sis' => 'application/vnd.symbian.install',
			'sisx' => 'application/vnd.symbian.install',
			'sit' => 'application/x-stuffit',
			'sitx' => 'application/x-stuffitx',
			'skd' => 'application/vnd.koan',
			'skm' => 'application/vnd.koan',
			'skp' => 'application/vnd.koan',
			'skt' => 'application/vnd.koan',
			'sldm' => 'application/vnd.ms-powerpoint.slide.macroenabled.12',
			'sldx' => 'application/vnd.openxmlformats-officedocument.presentationml.slide',
			'slt' => 'application/vnd.epson.salt',
			'sm' => 'application/vnd.stepmania.stepchart',
			'smf' => 'application/vnd.stardivision.math',
			'smi' => 'application/smil+xml',
			'smil' => 'application/smil+xml',
			'smv' => 'video/x-smv',
			'smzip' => 'application/vnd.stepmania.package',
			'snd' => 'audio/basic',
			'snf' => 'application/x-font-snf',
			'so' => 'application/octet-stream',
			'spc' => 'application/x-pkcs7-certificates',
			'spf' => 'application/vnd.yamaha.smaf-phrase',
			'spl' => 'application/x-futuresplash',
			'spot' => 'text/vnd.in3d.spot',
			'spp' => 'application/scvp-vp-response',
			'spq' => 'application/scvp-vp-request',
			'spx' => 'audio/ogg',
			'sql' => 'application/x-sql',
			'src' => 'application/x-wais-source',
			'srt' => 'application/x-subrip',
			'sru' => 'application/sru+xml',
			'srx' => 'application/sparql-results+xml',
			'ssdl' => 'application/ssdl+xml',
			'sse' => 'application/vnd.kodak-descriptor',
			'ssf' => 'application/vnd.epson.ssf',
			'ssml' => 'application/ssml+xml',
			'st' => 'application/vnd.sailingtracker.track',
			'stc' => 'application/vnd.sun.xml.calc.template',
			'std' => 'application/vnd.sun.xml.draw.template',
			'stf' => 'application/vnd.wt.stf',
			'sti' => 'application/vnd.sun.xml.impress.template',
			'stk' => 'application/hyperstudio',
			'stl' => 'application/vnd.ms-pki.stl',
			'str' => 'application/vnd.pg.format',
			'stw' => 'application/vnd.sun.xml.writer.template',
			'sub' => 'image/vnd.dvb.subtitle',
			'sub' => 'text/vnd.dvb.subtitle',
			'sus' => 'application/vnd.sus-calendar',
			'susp' => 'application/vnd.sus-calendar',
			'sv4cpio' => 'application/x-sv4cpio',
			'sv4crc' => 'application/x-sv4crc',
			'svc' => 'application/vnd.dvb.service',
			'svd' => 'application/vnd.svd',
			'svg' => 'image/svg+xml',
			'svgz' => 'image/svg+xml',
			'swa' => 'application/x-director',
			'swf' => 'application/x-shockwave-flash',
			'swi' => 'application/vnd.aristanetworks.swi',
			'sxc' => 'application/vnd.sun.xml.calc',
			'sxd' => 'application/vnd.sun.xml.draw',
			'sxg' => 'application/vnd.sun.xml.writer.global',
			'sxi' => 'application/vnd.sun.xml.impress',
			'sxm' => 'application/vnd.sun.xml.math',
			'sxw' => 'application/vnd.sun.xml.writer',
			't' => 'text/troff',
			't3' => 'application/x-t3vm-image',
			'taglet' => 'application/vnd.mynfc',
			'tao' => 'application/vnd.tao.intent-module-archive',
			'tar' => 'application/x-tar',
			'tcap' => 'application/vnd.3gpp2.tcap',
			'tcl' => 'application/x-tcl',
			'teacher' => 'application/vnd.smart.teacher',
			'tei' => 'application/tei+xml',
			'teicorpus' => 'application/tei+xml',
			'tex' => 'application/x-tex',
			'texi' => 'application/x-texinfo',
			'texinfo' => 'application/x-texinfo',
			'text' => 'text/plain',
			'tfi' => 'application/thraud+xml',
			'tfm' => 'application/x-tex-tfm',
			'tga' => 'image/x-tga',
			'thmx' => 'application/vnd.ms-officetheme',
			'tif' => 'image/tiff',
			'tiff' => 'image/tiff',
			'tmo' => 'application/vnd.tmobile-livetv',
			'torrent' => 'application/x-bittorrent',
			'tpl' => 'application/vnd.groove-tool-template',
			'tpt' => 'application/vnd.trid.tpt',
			'tr' => 'text/troff',
			'tra' => 'application/vnd.trueapp',
			'trm' => 'application/x-msterminal',
			'tsd' => 'application/timestamped-data',
			'tsv' => 'text/tab-separated-values',
			'ttc' => 'font/collection',
			'ttf' => 'font/ttf',
			'ttl' => 'text/turtle',
			'twd' => 'application/vnd.simtech-mindmapper',
			'twds' => 'application/vnd.simtech-mindmapper',
			'txd' => 'application/vnd.genomatix.tuxedo',
			'txf' => 'application/vnd.mobius.txf',
			'txt' => 'text/plain',
			'u32' => 'application/x-authorware-bin',
			'udeb' => 'application/x-debian-package',
			'ufd' => 'application/vnd.ufdl',
			'ufdl' => 'application/vnd.ufdl',
			'ulx' => 'application/x-glulx',
			'umj' => 'application/vnd.umajin',
			'unityweb' => 'application/vnd.unity',
			'uoml' => 'application/vnd.uoml+xml',
			'uri' => 'text/uri-list',
			'uris' => 'text/uri-list',
			'urls' => 'text/uri-list',
			'ustar' => 'application/x-ustar',
			'utz' => 'application/vnd.uiq.theme',
			'uu' => 'text/x-uuencode',
			'uva' => 'audio/vnd.dece.audio',
			'uvd' => 'application/vnd.dece.data',
			'uvf' => 'application/vnd.dece.data',
			'uvg' => 'image/vnd.dece.graphic',
			'uvh' => 'video/vnd.dece.hd',
			'uvi' => 'image/vnd.dece.graphic',
			'uvm' => 'video/vnd.dece.mobile',
			'uvp' => 'video/vnd.dece.pd',
			'uvs' => 'video/vnd.dece.sd',
			'uvt' => 'application/vnd.dece.ttml+xml',
			'uvu' => 'video/vnd.uvvu.mp4',
			'uvv' => 'video/vnd.dece.video',
			'uvva' => 'audio/vnd.dece.audio',
			'uvvd' => 'application/vnd.dece.data',
			'uvvf' => 'application/vnd.dece.data',
			'uvvg' => 'image/vnd.dece.graphic',
			'uvvh' => 'video/vnd.dece.hd',
			'uvvi' => 'image/vnd.dece.graphic',
			'uvvm' => 'video/vnd.dece.mobile',
			'uvvp' => 'video/vnd.dece.pd',
			'uvvs' => 'video/vnd.dece.sd',
			'uvvt' => 'application/vnd.dece.ttml+xml',
			'uvvu' => 'video/vnd.uvvu.mp4',
			'uvvv' => 'video/vnd.dece.video',
			'uvvx' => 'application/vnd.dece.unspecified',
			'uvvz' => 'application/vnd.dece.zip',
			'uvx' => 'application/vnd.dece.unspecified',
			'uvz' => 'application/vnd.dece.zip',
			'vcard' => 'text/vcard',
			'vcd' => 'application/x-cdlink',
			'vcf' => 'text/x-vcard',
			'vcg' => 'application/vnd.groove-vcard',
			'vcs' => 'text/x-vcalendar',
			'vcx' => 'application/vnd.vcx',
			'vis' => 'application/vnd.visionary',
			'viv' => 'video/vnd.vivo',
			'vob' => 'video/x-ms-vob',
			'vor' => 'application/vnd.stardivision.writer',
			'vox' => 'application/x-authorware-bin',
			'vrml' => 'model/vrml',
			'vsd' => 'application/vnd.visio',
			'vsf' => 'application/vnd.vsf',
			'vss' => 'application/vnd.visio',
			'vst' => 'application/vnd.visio',
			'vsw' => 'application/vnd.visio',
			'vtu' => 'model/vnd.vtu',
			'vxml' => 'application/voicexml+xml',
			'w3d' => 'application/x-director',
			'wad' => 'application/x-doom',
			'wav' => 'audio/x-wav',
			'wax' => 'audio/x-ms-wax',
			'wbmp' => 'image/vnd.wap.wbmp',
			'wbs' => 'application/vnd.criticaltools.wbs+xml',
			'wbxml' => 'application/vnd.wap.wbxml',
			'wcm' => 'application/vnd.ms-works',
			'wdb' => 'application/vnd.ms-works',
			'wdp' => 'image/vnd.ms-photo',
			'weba' => 'audio/webm',
			'webm' => 'video/webm',
			'webp' => 'image/webp',
			'wg' => 'application/vnd.pmi.widget',
			'wgt' => 'application/widget',
			'wks' => 'application/vnd.ms-works',
			'wm' => 'video/x-ms-wm',
			'wma' => 'audio/x-ms-wma',
			'wmd' => 'application/x-ms-wmd',
			'wmf' => 'application/x-msmetafile',
			'wml' => 'text/vnd.wap.wml',
			'wmlc' => 'application/vnd.wap.wmlc',
			'wmls' => 'text/vnd.wap.wmlscript',
			'wmlsc' => 'application/vnd.wap.wmlscriptc',
			'wmv' => 'video/x-ms-wmv',
			'wmx' => 'video/x-ms-wmx',
			'wmz' => 'application/x-ms-wmz',
			'wmz' => 'application/x-msmetafile',
			'woff' => 'font/woff',
			'woff2' => 'font/woff2',
			'wpd' => 'application/vnd.wordperfect',
			'wpl' => 'application/vnd.ms-wpl',
			'wps' => 'application/vnd.ms-works',
			'wqd' => 'application/vnd.wqd',
			'wri' => 'application/x-mswrite',
			'wrl' => 'model/vrml',
			'wsdl' => 'application/wsdl+xml',
			'wspolicy' => 'application/wspolicy+xml',
			'wtb' => 'application/vnd.webturbo',
			'wvx' => 'video/x-ms-wvx',
			'x32' => 'application/x-authorware-bin',
			'x3d' => 'model/x3d+xml',
			'x3db' => 'model/x3d+binary',
			'x3dbz' => 'model/x3d+binary',
			'x3dv' => 'model/x3d+vrml',
			'x3dvz' => 'model/x3d+vrml',
			'x3dz' => 'model/x3d+xml',
			'xaml' => 'application/xaml+xml',
			'xap' => 'application/x-silverlight-app',
			'xar' => 'application/vnd.xara',
			'xbap' => 'application/x-ms-xbap',
			'xbd' => 'application/vnd.fujixerox.docuworks.binder',
			'xbm' => 'image/x-xbitmap',
			'xdf' => 'application/xcap-diff+xml',
			'xdm' => 'application/vnd.syncml.dm+xml',
			'xdp' => 'application/vnd.adobe.xdp+xml',
			'xdssc' => 'application/dssc+xml',
			'xdw' => 'application/vnd.fujixerox.docuworks',
			'xenc' => 'application/xenc+xml',
			'xer' => 'application/patch-ops-error+xml',
			'xfdf' => 'application/vnd.adobe.xfdf',
			'xfdl' => 'application/vnd.xfdl',
			'xht' => 'application/xhtml+xml',
			'xhtml' => 'application/xhtml+xml',
			'xhvml' => 'application/xv+xml',
			'xif' => 'image/vnd.xiff',
			'xla' => 'application/vnd.ms-excel',
			'xlam' => 'application/vnd.ms-excel.addin.macroenabled.12',
			'xlc' => 'application/vnd.ms-excel',
			'xlf' => 'application/x-xliff+xml',
			'xlm' => 'application/vnd.ms-excel',
			'xls' => 'application/vnd.ms-excel',
			'xlsb' => 'application/vnd.ms-excel.sheet.binary.macroenabled.12',
			'xlsm' => 'application/vnd.ms-excel.sheet.macroenabled.12',
			'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
			'xlt' => 'application/vnd.ms-excel',
			'xltm' => 'application/vnd.ms-excel.template.macroenabled.12',
			'xltx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
			'xlw' => 'application/vnd.ms-excel',
			'xm' => 'audio/xm',
			'xml' => 'application/xml',
			'xo' => 'application/vnd.olpc-sugar',
			'xop' => 'application/xop+xml',
			'xpi' => 'application/x-xpinstall',
			'xpl' => 'application/xproc+xml',
			'xpm' => 'image/x-xpixmap',
			'xpr' => 'application/vnd.is-xpr',
			'xps' => 'application/vnd.ms-xpsdocument',
			'xpw' => 'application/vnd.intercon.formnet',
			'xpx' => 'application/vnd.intercon.formnet',
			'xsl' => 'application/xml',
			'xslt' => 'application/xslt+xml',
			'xsm' => 'application/vnd.syncml+xml',
			'xspf' => 'application/xspf+xml',
			'xul' => 'application/vnd.mozilla.xul+xml',
			'xvm' => 'application/xv+xml',
			'xvml' => 'application/xv+xml',
			'xwd' => 'image/x-xwindowdump',
			'xyz' => 'chemical/x-xyz',
			'xz' => 'application/x-xz',
			'yang' => 'application/yang',
			'yin' => 'application/yin+xml',
			'z1' => 'application/x-zmachine',
			'z2' => 'application/x-zmachine',
			'z3' => 'application/x-zmachine',
			'z4' => 'application/x-zmachine',
			'z5' => 'application/x-zmachine',
			'z6' => 'application/x-zmachine',
			'z7' => 'application/x-zmachine',
			'z8' => 'application/x-zmachine',
			'zaz' => 'application/vnd.zzazz.deck+xml',
			'zip' => 'application/zip',
			'zir' => 'application/vnd.zul',
			'zirz' => 'application/vnd.zul',
			'zmm' => 'application/vnd.handheld-entertainment+xml'
		);
		if (isset( $mimeTypes[$extension] )) {
			return $mimeTypes[$extension];
		} else {
			return 'application/octet-stream';
		}
	}

	// TODO - Josiah check this .. want it to return the file_id of a language folder if it exists and is available
	public function mediaGetHomepageLanguageFolderFileIdForBrowserLanguage($iso639_1){
		try {
			$this->openDb();
			$stmt = $this->dbHandle->prepare("SELECT file_id FROM media2 WHERE parent_path_rel = '' AND mime_part1 = 'directory' AND language_iso = (SELECT Id FROM languages_iso639_3 WHERE Part1 = :part_1 LIMIT 1) AND mime_part2 = :language_directory_mime AND wifi_status > 0 AND is_deleted = 0 LIMIT 1");
			$stmt->bindValue(':language_directory_mime', self::MEDIA_MIME_LANGUAGE_DIRECTORY);
			$stmt->bindValue(':part_1', $iso639_1);
			$retVal = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if (!empty($retVal)) {
				return $retVal[0]['file_id'];
			}
			return FALSE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Return formatted arclight json file, which is empty if there is no jf content
	 * @return Associative Array with json data
	 */
	public function mediaGetArclightJsonFile() {
		try {
			$this->openDb();
			$stmt = $this->dbHandle->prepare("SELECT * FROM media2 WHERE downloader_source = '" . self::DOWNLOADER_SOURCE_JF . "' AND is_password_protected = 0 AND wifi_status > 0 AND is_deleted = 0");
			$jfMediaItemArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			$arclightJsonFile = array(
				'mediaAssets' => array(),
			);
			foreach($jfMediaItemArray as $mediaItem) {
				$downloaderJson = json_decode($mediaItem['downloader_json'], TRUE);
				$arclightJsonFile['mediaAssets'][] = array(
					'mediaComponentId' => $downloaderJson['media_component_id'],
					'mediaLanguageId' => $downloaderJson['media_language_id'],
					'urlPath' => "/file/{$mediaItem['file_id']}", // self::MediaGetPathRelWithFile($mediaItem),
					'fileSizeInBytes' => $mediaItem['file_size_bytes']
				);
			}
			return $arclightJsonFile;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get count of all media items
	 * @param $search optionally, get count of media matching a search term
	 */
	public function mediaGetCount($search = FALSE) {
		try {
			$this->openDb();
			$appendString = '';
			if ($search !== FALSE) {
				$search = \SQLite3::escapeString($search);
				$appendString .= " AND (title LIKE '%{$search}%' OR file_name LIKE '%{$search}%')";
			}
			$countStmtStr = "SELECT COUNT(*) as media_count FROM media2 WHERE is_deleted = 0{$appendString}";
			$stmt = $this->dbHandle->prepare($countStmtStr);
			$sqlite3Result = $stmt->execute();
			$languagesCount = $sqlite3Result->fetchArray(SQLITE3_ASSOC)['media_count'];
			return $languagesCount;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Search media and receive paginated results, used in <select> tags
	 * @param $search the search term
	 * @param $page the paginated page (offset = page * limit)
	 * @param $limit how many items to have on each page
	*/
	public function mediaSearch($search = FALSE, $page = 0, $limit = self::LIMIT_SEARCH_DEFAULT, $onlyVideo = FALSE) {
		try {
			$this->openDb();
			$appendString = '';
			if ($search !== FALSE) {
				$search = \SQLite3::escapeString($search);
				$appendString .= " AND (title LIKE '%{$search}%' OR file_name LIKE '%{$search}%')";
			}
			if ($onlyVideo === TRUE) {
				$appendString .= " AND mime_part1 = '" . self::MEDIA_MIME_PART1_VIDEO . "'";
			}
			$offset = $page * $limit;
			$stmtStr = "SELECT file_id, title, file_name, file_extension FROM media2 WHERE is_deleted = 0{$appendString} ORDER BY title ASC LIMIT :lim OFFSET :off";
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':lim', $limit);
			$stmt->bindValue(':off', $offset);
			$searchResults = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			$mediaCount = $this->mediaGetCount($search);
			if ($mediaCount === FALSE) {
				error_log('Err: problem getting media count');
				return FALSE;
			}
			$answer = ArrayHelper::GetPaginatedResponse($searchResults, $page, $limit, $mediaCount);
			return $answer;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function languagesGetForSelect($search = FALSE, $page = 0, $limit = self::LIMIT_SEARCH_DEFAULT) {
		return $this->languagesGetHelper($search, NULL, $page, $limit);
	}

	public function languagesGetWithIso($languageIso) {
		return $this->languagesGetHelper(FALSE, $languageIso);
	}

	/**
	 * Get count of languages in the lanaguages db. Useful for pagination.
	 * @param $search option to only count languages that match the search term
	 */
	public function languagesGetCount($search = FALSE) {
		try {
			$appendString = '';
			if ($search !== FALSE) {
				$search = \SQLite3::escapeString($search);
				$appendString .= "AND languages_iso639_3.Ref_Name LIKE '%{$search}%'";
			}
			$countStmtStr = "SELECT COUNT(*) as languages_count FROM languages_iso639_3 JOIN languages_jf_arclight ON languages_jf_arclight.iso639_3 = languages_iso639_3.Id WHERE languages_iso639_3.Type = 'L' {$appendString}";
			$stmt = $this->dbHandle->prepare($countStmtStr);
			$sqlite3Result = $stmt->execute();
			$languagesCount = $sqlite3Result->fetchArray(SQLITE3_ASSOC)['languages_count'];
			return $languagesCount;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Gets languages from the languages_iso639_3 database, while getting localized
	* names from the languages_jf_arclight database.
	*
	* @param $search string to search languages for
	* @param $languageIso if set, only that language will be returned as an
	* associative array
	* @return Mixed: Array of Associative Arrays, or Paginated AssociativeArray, or Associative Array (if languageIso)
	*/
	private function languagesGetHelper($search = FALSE, $languageIso = NULL, $page = NULL, $limit = self::LIMIT_SEARCH_DEFAULT) {
		try {
			$this->openDb();
			$appendString = '';
			$hasLanguageIso = ($languageIso !== NULL);
			$hasPage = $page !== NULL;
			if ($search !== FALSE && !$hasLanguageIso) {
				$search = \SQLite3::escapeString($search);
				$appendString .= "AND languages_iso639_3.Ref_Name LIKE '%{$search}%'";
			}
			if ($hasLanguageIso) {
				$appendString .= ' AND languages_iso639_3.Id = :languageIso LIMIT 1';
			} else {
				$appendString .= ' ORDER BY languages_iso639_3.Ref_Name ASC';
				if ($hasPage) {
					$offset = \SQLite3::escapeString($page * $limit);
					$limit = \SQLite3::escapeString($limit);
					$appendString.= " LIMIT {$limit} OFFSET {$offset}" ;
				}
			}
			$stmtStr = "SELECT languages_iso639_3.Id AS iso, languages_iso639_3.Ref_Name AS 'name_english', languages_jf_arclight.name_localized as 'name_localized' FROM languages_iso639_3 JOIN languages_jf_arclight ON languages_jf_arclight.iso639_3 = languages_iso639_3.Id WHERE languages_iso639_3.Type = 'L' {$appendString}";			
			$stmt = $this->dbHandle->prepare($stmtStr);

			// Return single associative array if getting from language iso
			$answer = FALSE;
			if ($hasLanguageIso) {
				$stmt->bindValue(':languageIso', $languageIso);
				$answer = $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
			} else {
				$answer = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			}
			if ($hasPage) {
				$languagesCount = $this->languagesGetCount($search);
				if ($languagesCount === FALSE) {
					error_log('Err: problem getting languages count');
					return FALSE;
				}
				$answer = ArrayHelper::GetPaginatedResponse($answer, $page, $limit, $languagesCount);
			}
			return $answer;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}
}
