<?php

namespace Renew;

class LogManager
{
    /**
     * The path to the log file to manage
     *
     * @var string
     * @access private
     **/
    private $logPath;
    /**
     * The handle for the opened log file (A File Object)
     *
     * @var object
     * @access private
     **/
    private $logHandle;
    /**
     * The byte size recorded for the log file
     *
     * @var integer
     * @access private
     **/
    public $byteSize = 0;
    /**
     * Initialize the class
     *
     * @param string $filePath the path to the log file to manage
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($filePath)
    {
      try {
        if (!file_exists($filePath)) {
            throw new \InvalidArgumentException("The file does not exist: " . $filePath);
        }
        $this->logPath = $filePath;
        $this->setByteSize();
      } catch (\Exception $e) {
        error_log($e);
      }
    }
    /**
     * return the current byte size of the file
     *
     * @return integer
     * @access public
     * @author JPul
     **/
    public function getByteSize()
    {
        return $this->byteSize;
    }
    /**
     * reads the contents of the log file
     *
     * @param integer $lines the toal lines to return
     * @param boolean $adaptive do you want the buffer size to adapt (optional default:true)
     * @param boolean $reverse do you want the contents returned in reverse order (optional default:false)
     * @return string the content of the file
     * @access public
     * @author JPul
     * @link https://gist.github.com/lorenzos/1711e81a9162320fde20
     **/
    public function read($lines = 1, $adaptive = true, $reverse = false)
    {
        $this->setByteSize();
        $output = '';
        $chunk = '';
        $this->logHandle = fopen($this->logPath, 'r');
        if ($adaptive === true) {
            if ($lines < 2) {
                $buffer = 64;
            } elseif ($lines < 10) {
                $buffer = 512;
            } else {
                $buffer = 4096;
            }
        } else {
            $buffer = 4096;
        }
        /**
         * Jump to the last character
         */
        fseek($this->logHandle, -1, SEEK_END);
        /**
         * Read it and adjust line number if necessary
         * (Otherwise the result would be wrong if file doesn't end with a blank line)
         */
        if (fread($this->logHandle, 1) != "\n") {
            $lines -= 1;
        }
        while (ftell($this->logHandle) > 0 && $lines >= 0) {

            /**
             * Figure out how far back we should jump
             */
            $seek = min(ftell($this->logHandle), $buffer);

            /**
             * Do the jump (backwards, relative to where we are)
             */
            fseek($this->logHandle, -$seek, SEEK_CUR);

            /**
             * Read a chunk and prepend it to our output
             */
            $chunk = fread($this->logHandle, $seek);
            $output = $chunk . $output;

            /**
             * Jump back to where we started reading
             */
            fseek($this->logHandle, -mb_strlen($chunk, '8bit'), SEEK_CUR);

            /**
             * Decrease our line counter
             */
            $lines -= substr_count($chunk, "\n");
        }
        /**
         * While we have too many lines
         * (Because of buffer size we might have read too many)
         */
        while ($lines++ < 0) {
            /**
             * Find first newline and remove all text before that
             */
            $output = substr($output, strpos($output, "\n") + 1);
        }
        if ($reverse === true) {
            $output = $this->reverseContents($output);
        }
        fclose($this->logHandle);
        return trim($output);
    }
    /**
     * Read the log starting at the given offset in bytes
     *
     * @param integer $bytes the amount of bytes to offset the read with
     * @return string the file content
     * @access public
     * @author JPul
     **/
    public function readWithOffset($bytes)
    {
        $this->setByteSize();
        $this->logHandle = fopen($this->logPath, 'r');
        /**
         * Jump to the last character
         */
        fseek($this->logHandle, 0, SEEK_END);
        /**
         * Get the current byte size of the file
         */
        $size = ftell($this->logHandle);
        $remainingBytesNeeded = $size-$bytes;
        if ($remainingBytesNeeded <= 0) {
            /**
             * No new content to return
             */
            return "";
        }
        /**
         * Do the jump (backwards, relative to where we are)
         */
        fseek($this->logHandle, -$remainingBytesNeeded, SEEK_CUR);
        $output = fread($this->logHandle, $remainingBytesNeeded);

        fclose($this->logHandle);
        return trim($output);
    }
    /**
     * Reverse the order of content returned by the log file
     *
     * @param string $contents the contents of the file
     * @return string
     * @access private
     * @author JPul
     **/
    private function reverseContents($contents)
    {
        $contentPieces = explode("\n", $contents);
        $reversedContentPieces = array_reverse($contentPieces);
        return implode("\n", $reversedContentPieces);
    }
    /**
     * sets the files current byte size
     *
     * @return void
     * @access private
     * @author JPul
     **/
    private function setByteSize()
    {
        $this->byteSize = filesize($this->logPath);
    }
}
