<?php

namespace Renew;

/**
 * This class handles the management for the pages of the system.
 *
 * @package default
 * @author JPul
 **/
class PageManager
{
    /**
     * The SQLite3 Database object of the stats database
     *
     * @var object
     * @access private
     **/
    private $db;
    /**
     * Initialize the class
     *
     * @param object $db the SQLite3 Database object
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($db)
    {
        $this->db = $db;
    }
    /**
     * save() the pages information.  It will either create (no id provided), or update (id provided) based
     * on passed params
     *
     * @param array $data the data that you would like to save
     * @param integer $id the id of the page to edit (optional)
     * @return booolean Saved correctly
     * @access public
     * @author JPul
     **/
    public function save($data, $id = null)
    {
      try {
        $data['title'] =  strip_tags($data['title']);
        $data['media_title'] =  strip_tags($data['media_title']);
        if (is_null($id)) {
            return $this->saveRecord($data);
        } else {
            if ($this->recordExists($id) === false) {
                throw new \InvalidArgumentException("The id does not exist in the database.");
            }
            return $this->updateRecord($data, $id);
        }
      } catch (\Exception $e) {
        error_log($e);
        return FALSE;
      }
    }
    /**
     * returns the page information that belongs to the given id
     *
     * @param integer $id the id for the given page
     * @return array
     * @access public
     * @author JPul
     **/
    public function findById($id)
    {
        $stmt = $this->db->prepare("SELECT * FROM pages WHERE id = :id");
        $stmt->bindValue(':id', intval($id));
        $results = $stmt->execute();
        return $results->fetchArray(SQLITE3_ASSOC);
    }
    /**
     * Find the page based on it's slug
     *
     * @param string $slug the slug to use in the find
     * @return array
     * @access public
     * @author JPul
     **/
    public function findBySlug($slug)
    {
        $stmt = $this->db->prepare("SELECT * FROM pages WHERE slug = :slug");
        $stmt->bindValue(':slug', $slug);
        $results = $stmt->execute();
        return $results->fetchArray(SQLITE3_ASSOC);
    }
    /**
     * Save the record to the pages table
     *
     * @param array $data the data to save to the database
     * @return boolean did it save?
     * @access private
     * @author JPul
     **/
    private function saveRecord($data)
    {
        $stmt = $this->db->prepare(
            "INSERT INTO pages " .
            "(title, summary_content, content, slug, media_title) " .
            "VALUES(:title, :summary_content, :content, :slug, :media_title)"
        );
        $stmt->bindValue(':title', $data['title']);
        $stmt->bindValue(':summary_content', $data['summary_content']);
        $stmt->bindValue(':content', $data['content']);
        $stmt->bindValue(':slug', $data['slug']);
        $stmt->bindValue(':media_title', $data['media_title']);
        return $stmt->execute();
    }
    /**
     * Update the given record in the pages table
     *
     * @param array $data the data that you would like to save
     * @param integer $id the id of the page to edit (optional)
     * @return boolean did it save?
     * @access private
     * @author JPul
     **/
    private function updateRecord($data, $id)
    {
        $currentRecord = $this->findById($id);
        $newData = array_merge($currentRecord, $data);
        $stmt = $this->db->prepare(
            "UPDATE pages SET " .
            "title = :title, " .
            "summary_content = :summary_content, " .
            "content = :content, " .
            "slug = :slug, " .
            "media_title = :media_title " .
            "WHERE id = :id"
        );
        $stmt->bindValue(':id', $id);
        $stmt->bindValue(':title', $newData['title']);
        $stmt->bindValue(':summary_content', $newData['summary_content']);
        $stmt->bindValue(':content', $newData['content']);
        $stmt->bindValue(':slug', $newData['slug']);
        $stmt->bindValue(':media_title', $newData['media_title']);
        return $stmt->execute();
    }
    /**
     * checks to see if the record exists
     *
     * @param integer $id the id of the page
     * @return boolean does it exist?
     * @access private
     * @author JPul
     **/
    private function recordExists($id)
    {
        $stmt = $this->db->prepare("SELECT Count(*) as count FROM pages WHERE id = :id");
        $stmt->bindValue(':id', intval($id));
        $results = $stmt->execute();
        $row = $results->fetchArray();
        if ($row['count'] == 1) {
            return true;
        } else {
            return false;
        }
    }
}
