<?php

namespace Renew;

/**
 * This class handles the management for the settings of the system.
 *
 * @package default
 * @author JPul
 **/
class SiteSettingsManager
{
    /**
     * The SQLite3 Database object of the stats database
     *
     * @var object
     * @access private
     **/
    private $db;
    /**
     * Initialize the class
     *
     * @param object $db the SQLite3 Database object
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($db)
    {
        $this->db = $db;
    }

    public function getTitleActionOption(){
            $value = $this->findBySlug("title-action-option");
            if (!isset($value['slug'])){
                    $this->saveBySlug("title-action-option", "Stream", "the action taken when the title of a file is clicked on the home page");
                    $value = $this->findBySlug("title-action-option");
            }
            return $value;
    }

    public function getStreamButtonOption(){
            $value = $this->findBySlug("stream-button-option");
            if (!isset($value['slug'])){
                    $this->saveBySlug("stream-button-option", "Show", "the option for the Stream button on the homepage");
                    $value = $this->findBySlug("stream-button-option");
            }
            return $value;
    }

    public function getDownloadDirButtonOption(){
            $value = $this->findBySlug("download-dir-button-option");
            if (!isset($value['slug'])){
                    $this->saveBySlug("download-dir-button-option", "Show", "the option for the Download Dir button on the homepage");
                    $value = $this->findBySlug("download-dir-button-option");
            }
            return $value;
    }

    public function setTitleActionOptionSetting($setting){
            $value = $this->getTitleActionOption();
            $value['setting'] = $setting;
            $this->save($value);
    }

    public function setStreamButtonOptionSetting($setting){
            $value = $this->getStreamButtonOption();
            $value['setting'] = $setting;
            $this->save($value);
    }

    public function setDownloadDirButtonOptionSetting($setting){
            $value = $this->getDownloadDirButtonOption();
            $value['setting'] = $setting;
            $this->save($value);
    }

    public function saveBySlug($slug, $setting, $description){
            $setting = array(
                'slug'          =>  $slug,
                'setting'       =>  $setting,
                'description'   =>  $description
            );
            $this->save($setting);
    }
    /**
     * save the given site settings
     *
     * @param array $data the site settings to save
     * @return boolean Did it save?
     * @access public
     * @author JPul
     **/
    public function save($data)
    {
      try {
        if ((!isset($data['slug'])) || (empty($data['slug']))) {
            throw new \InvalidArgumentException("You must pass a valid slug param to save the site setting.");
        }
        $currentRecord = $this->findBySlug($data['slug']);
        if (isset($currentRecord['slug'])) {
            $newData = array_merge($currentRecord, $data);
            return $this->updateRecord($newData);
        } else {
            return $this->saveRecord($data);
        }
      } catch (\Exception $e) {
        error_log($e);
        return FALSE;
      }
    }
    /**
     * Find the site settings based on the slug
     *
     * @param string $slug the slug for the site settings
     * @return array The site Settings data
     * @access public
     * @author JPul
     **/
    public function findBySlug($slug)
    {
            try{
                    $stmt = $this->db->prepare("SELECT * FROM site_settings WHERE slug = :slug");
                    $stmt->bindValue(':slug', $slug);
                    $results = $stmt->execute();
                    return $results->fetchArray(SQLITE3_ASSOC);
            }catch (Exception $e){
                    return false;
            }
    }
    /**
     * Inserts the new settings
     *
     * @param array $data the settings to save
     * @return boolean Did it save?
     * @access private
     * @author JPul
     **/
    private function saveRecord($data)
    {
        $stmt = $this->db->prepare(
            "INSERT INTO site_settings " .
            "(slug, setting, description) " .
            "VALUES(:slug, :setting, :description)"
        );
        $stmt->bindValue(':slug', $data['slug']);
        $stmt->bindValue(':setting', $data['setting']);
        $stmt->bindValue(':description', $data['description']);
        return $stmt->execute();
    }
    /**
     * Update the given record in the pages table
     *
     * @param array $data the data that you would like to save
     * @return boolean did it save?
     * @access private
     * @author JPul
     **/
    private function updateRecord($data)
    {
        $stmt = $this->db->prepare(
            "UPDATE site_settings SET " .
            "setting = :setting, description = :description " .
            "WHERE slug = :slug"
        );
        $stmt->bindValue(':slug', $data['slug']);
        $stmt->bindValue(':setting', $data['setting']);
        $stmt->bindValue(':description', $data['description']);
        return $stmt->execute();
    }
}
