<?php
namespace Renew;
require_once('StringHelper.php');

class SystemHelper {

	/**
	* Get the name of the scsi disk in the slot
	*
	* @return mixed string matching sd[a-z] if exists or false
	* @access private
	**/
	private static function UsbDriveGetScsiDiskName() {
		try {
			// TODO: this is the path for the LightStream-Pocket hardware ... do we support other hardwares?
			$link = @readlink('/dev/disk/by-path/platform-musb-hdrc.1.auto-usb-0:1:1.0-scsi-0:0:0:0');
			if ($link === FALSE) {
				return FALSE;
			}
			$array = explode('/', $link);
			$scsiDisk = end($array);

			if (preg_match('/^sd[a-z]$/', $scsiDisk) === FALSE) {
				return FALSE;
			}

			return $scsiDisk;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}



	/**
	* Check to see if there is a usb drive in the slot .. return TRUE or FALSE
	*
	* @return boolean
	* @access private
	**/
	private static function UsbDriveRestartSerice() {
		try {
			$scsiDisk = self::UsbDriveGetScsiDiskName();
			if ($scsiDisk === FALSE) {
				return FALSE;
			}
			$success = self::RunCmd("sudo /usr/bin/systemctl restart rwo_deviceLauncher@{$scsiDisk}");
			return $success;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/**
	* Check to see if there is a usb drive in the slot .. return TRUE or FALSE
	*
	* @return boolean
	* @access public
	**/
	public static function UsbDriveIsInSlot() {
		try {
			if (self::UsbDriveGetScsiDiskName() === FALSE) {
				return FALSE;
			}
			return TRUE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Check to see if there is an sd card in the slot .. return TRUE or FALSE
	*
	* @return boolean
	* @access public
	**/
	public static function SdCardIsInSlot() {
		try {
			// you must check both for a rare case, becaise in some cases they can be swapped
			if (file_exists('/dev/mmcblk0') && file_exists('/dev/mmcblk1')) {
				return TRUE;
			}
			return FALSE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Restart the device launcher service for the SD card
	*
	* @return boolean
	* @access private
	**/
	private static function SdCardRestartService() {
		try {
			self::RunCmd('sudo /usr/bin/systemctl restart rwo_deviceLauncher@mmcblk1');
			self::RunCmd('sudo /usr/bin/systemctl restart rwo_deviceLauncher@mmcblk0');
			return TRUE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Stop the device launcher service for the SD card
	*
	* @return boolean
	* @access private
	**/
	public static function SdCardStopService() {
		try {
			self::RunCmd('sudo /usr/bin/systemctl stop rwo_deviceLauncher@mmcblk1');
			self::RunCmd('sudo /usr/bin/systemctl stop rwo_deviceLauncher@mmcblk0');
			return TRUE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Check to see if there is an sd card in the slot, and it's a mirror sd card
	*
	* @return boolean
	* @access public
	**/
	public static function SdCardIsInSlotAndIsMirrorCard() {
		try {
			if (is_link('/dev/disk/by-partuuid/aa1212cd-01')) {
				return TRUE;
			}
			return FALSE;
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public static function IsPidRunning($pid) {
		try {
			if (strlen($pid) === 0){
				return FALSE;
			}
			// support for mac
			if (function_exists('posix_getpgid') === TRUE) {
				$pgid_int = posix_getpgid($pid);
				return ($pgid_int != FALSE);
			}
			// support for linux
			else {
				exec('ps '.$pid, $output);
				if(count($output) < 2){
					return FALSE;
				}
				return TRUE;
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Run a given string on the command line
	*
	* @return Mixed returns an associative array containing keys 'stdout' and 'return_var' on success, and FALSE on failure
	* @access public
	**/
	public static function RunCmd($cmd, $prependSudo = FALSE){
		try {
			// TODO DEV ENVIRONMENT - This is a lot of dirnames. Remove this for the hardware, non-dev version?
			$basePath = dirname(dirname(dirname(dirname(dirname(__FILE__))))); // for dev environment
			$pathVar = "PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin";
			if (strlen($basePath) > 1) {
				$pathVar = "{$pathVar}:{$basePath}/usr/bin";
			}
			putenv($pathVar);
			if ($prependSudo === TRUE) {
				$cmd = "/usr/bin/sudo $cmd";
			}
			exec($cmd, $output, $return);
			$retArray = array();
			$retArray['stdout'] = $output;
			$retArray['return_var'] = $return;
			// Log bash errors
			if (count($output) > 0) {
				foreach ($output as $out) {
					error_log(print_r($out, TRUE));
				}
			}
			return $retArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Run a given string on the command line
	*
	* @return boolean a string, and FALSE if there was an exception
	* @access public
	**/
	public static function RunCmdReturnSingleLine($cmdStr){
		try {
			$returnVar = self::RunCmd($cmdStr);
			if ($returnVar === FALSE) {
				$returnVar = FALSE;
			}elseif (count($returnVar) > 0 && array_key_exists('stdout', $returnVar) && count($returnVar['stdout']) > 0) {
				$returnVar = $returnVar['stdout'][0];
			} else {
				$returnVar = '';
			}
			return $returnVar;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Run a given string on the command line
	*
	* @return boolean TRUE success, and FALSE if fail ...
	* @access public
	**/
	public static function RunCmdReturnTrueOrFalse($cmdStr) {
		try {
			return self::RunCmd($cmdStr)['return_var'] === 0;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Restore all system defaults including deleting all user media and images
	*
	* @return boolean TRUE success, and FALSE if fail ...
	* @access public
	**/
	public static function SystemResetDefaults() {
		try {
			return self::RunCmdReturnTrueOrFalse('sudo rwo_system.sh systemReset');
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Scan for nearby wifi networks and return an associative array of associative arrays
	*
	* @return Array of AssociativeArrays on success
	* @return ... and the inner has keys 'ssid' and 'quality', and FALSE if fail
	* @access public
	**/
	public static function Wlan1ScanForWifiNetworksReturnSSIDArray($showOpenNetworks = FALSE) {
		try {
			// TODO, handle new array with ssids and signal
			$outputArray = self::RunCmd('sudo /usr/bin/rwo_wifi.sh wlan1Scan')['stdout'];
			$returnArray = array();
			/*
			wifiNetworkNameWithWpa2
			51
			wpa2
			0938284789718497184972194
			anotherWifiNetworkNameThatIsOpen
			60
			none
			0938284789718497184972194
			*/
			$ssid='';
			$index = 0;
			foreach ($outputArray as $value) {
				// there are 3 lines per network
				$modVal = $index % 4;
				if ($modVal === 0) {
					$ssid = $value;
				} else if ($modVal === 1) {
					$quality = $value;
				} elseif ($modVal === 2) {
					$security = $value;
				}else if ($modVal === 3) {
					$ssidHex = $value;
					$arrayKey = $ssid . ' | ' . $security;
					$quality = intval($quality);
					if (array_key_exists($arrayKey, $returnArray)) {
						// check if the one you are about to put in has a higher Quality
						if ($returnArray[$arrayKey]['quality'] < $quality) {
							$returnArray[$arrayKey]['quality'] = $quality;
						}
					} else {
						$isPasswordProtected = $security !== 'none';
						if ($isPasswordProtected || $showOpenNetworks === TRUE) {
							$ssidArray = array();
							$ssidArray['ssid'] = $ssid;
							$ssidArray['ssid_hex'] = $ssidHex;
							$ssidArray['quality'] = $quality;
							$ssidArray['security'] = $security;
							$ssidArray['password_protected'] = $isPasswordProtected;
							$returnArray[$arrayKey] = $ssidArray;
						}
					}
				}
				++$index;
			}
			$returnArraySorted = array_values($returnArray);
			usort($returnArraySorted, function($a, $b) {
				return (int) $b['quality'] -(int) $a['quality'];
			});
			return $returnArraySorted;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Connect to a wifi network, if you are connecting to one with a password, make sure the password is set in the database
	*
	* @return boolean TRUE success if it connected, and FALSE if fail it did not connect
	* @access public
	**/
	public static function Wlan1ConnectToSsid($ssidHex, $password = '', $wait = FALSE) {
		try {
			self::RunCmd("sudo chmod -R 777 /tmp/rwo");
			$cmd = "sudo bash -c \"\$(/usr/bin/echo " . escapeshellarg($ssidHex) . " > '/tmp/rwo/rwo_wifi@wlan1Connect.ssid')\"";
			self::RunCmd($cmd);
			$cmd = "sudo bash -c \"\$(/usr/bin/echo " . escapeshellarg($password) . " > '/tmp/rwo/rwo_wifi@wlan1Connect.password')\"";
			self::RunCmd($cmd);
			self::RunCmdReturnSingleLine("sudo /usr/bin/systemctl --no-block restart rwo_wifi@wlan1Connect");
			if ($wait) {
				for ($i=0; $i < 15; $i++) {
					sleep(1);
					$status = self::Wlan1ConnectGetStatus();
					if ($status === "connected") {
						return TRUE;
					} elseif ($status === "failed") {
						return FALSE;
					}
				}
				return FALSE;
			}
			// case where you are not waiting
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get the status of the connect event
	*
	* @return AssociativeArray ssd: string, status: string connecting, connected, failed, obtaining ip address
	* @access public
	**/
	public static function Wlan1ConnectGetStatus() {
		try {
			$ssidFile = '/tmp/rwo/rwo_wifi@wlan1Connect.ssid';
			$ssidHex = '';
			if (file_exists($ssidFile)) {
				$ssidHex = trim(file_get_contents($ssidFile));
			}
			$status = self::RunCmdReturnSingleLine("sudo /usr/bin/rwo_wifi.sh wlan1GetConnectStatus");
			$ssid = \Renew\StringHelper::ConvertHexToText($ssidHex);
			return array(
				'status' => $status,
				'ssid_hex' => $ssidHex,
				'ssid' => $ssid,
			);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* TODO REMOVE? Is this being used? Status seems to have taken its place.
	* Get the currently connected wifi network to wlan1
	*
	* @return mixed string if success, and blank string if disconnected, and FALSE if fail
	* @access public
	**/
	public static function Wlan1GetConnectedSsid() {
		try {
			return self::RunCmdReturnSingleLine("sudo /usr/bin/rwo_wifi.sh wlan1GetConnectedSsid");
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Disconnect wlan1 from the currently connected wireless network
	*
	* @return boolean TRUE if success, and FALSE if fail
	* @access public
	**/
	public static function Wlan1Disconnect() {
		try {
			self::RunCmdReturnTrueOrFalse("sudo /usr/bin/rwo_wifi.sh wlan1Disconnect");
			return self::RunCmdReturnTrueOrFalse("sudo /usr/bin/systemctl --no-block stop rwo_wifi@wlan1Connect");
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Set the flag to create the mirror, the flag expries every X seconds
	*
	* @return boolean TRUE if success, and FALSE if fail
	* @access public
	**/
	public static function DeviceLauncherMirrorCreateSetFlag() {
		try {

			self::RunCmd("sudo chmod -R 777 /tmp/rwo");
			$cmd = "sudo /usr/bin/touch /tmp/rwo/rwo_deviceLauncher.mirrorCreate";
			self::RunCmd($cmd);
			self::SdCardRestartService();
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Set the flag to run the mirror drive, the flag expries every X seconds
	*
	* @return boolean TRUE if success, and FALSE if fail
	* @access public
	**/
	public static function DeviceLauncherMirrorRunSetFlag() {
		try {

			self::RunCmd("sudo chmod -R 777 /tmp/rwo");
			$cmd = "sudo /usr/bin/touch /tmp/rwo/rwo_deviceLauncher.mirrorRun";
			self::RunCmd($cmd);
			@unlink('/tmp/rwo/rwo_deviceLauncher.mirrorCreate.status');
			self::SdCardRestartService();
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get the status of the mirror drive creation process
	*
	* @return mixed boolean FALSE if the mirror creation has not yet started, a string otherwise
	* @return ... some possible values for the string are: STATUS_IN_PROGRESS, STATUS_SUCCESS, STATUS_ERROR_TOO_SMALL, STATUS_ERROR_FAILED
	* @access public
	**/
	public static function DeviceLauncherMirrorCreateGetStatus() {
		try {
			$statusFile='/tmp/rwo/rwo_deviceLauncher.mirrorCreate.status';
			return self::GetStatusFromFile($statusFile);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get the percentage of the mirror creation
	 *
	 * @return mixed FALSE if the mirror drive is not in the slot, or an interger 0-100 represeting the progress otherwise.
	 * @access public
	 **/
	public static function DeviceLauncherMirrorCreatePercentage() {
		try {
			$statusFile='/tmp/rwo/rwo_deviceLauncher.mirrorCreate.percentage';
			$contents = self::GetStatusFromFile($statusFile);
			// if it fails, it will return an argument that double equals false
			if ($contents == FALSE) {
				return FALSE;
			}
			return intval($contents);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Set the flag to create the update drive, the flag expries every X seconds
	*
	* @return boolean TRUE if success, and FALSE if fail
	* @access public
	**/
	public static function DeviceLauncherUpdateDriveCreateSetFlag() {
		try {

			self::RunCmd("sudo chmod -R 777 /tmp/rwo");
			$cmd = "sudo /usr/bin/touch /tmp/rwo/rwo_deviceLauncher.updateDriveCreate";
			self::RunCmd($cmd);
			@unlink('/tmp/rwo/rwo_deviceLauncher.updateDriveCreate.status');
			return self::UsbDriveRestartSerice();
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Set the flag to create a update drive, the flag expries every X seconds
	*
	* @return mixed boolean FALSE if the update drive creation has not yet started, a string otherwise
	* @return ... some possible values for the string are: STATUS_SUCCESS, STATUS_ERROR_FAILED, STATUS_ERROR_TIMEOUT
	* @access public
	**/
	public static function DeviceLauncherUpdateDriveCreateGetStatus() {
		try {
			$statusFile='/tmp/rwo/rwo_deviceLauncher.updateDriveCreate.status';
			return self::GetStatusFromFile($statusFile);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get the status (content) from a file or return FALSE
	*
	* @return mixed boolean FALSE if the file is not there, otherwise a string that is the content of the file
	* @access private
	**/
	private static function GetStatusFromFile($statusFile) {
		try {
			if (!file_exists($statusFile)) {
				return FALSE;
			}
			else {
				// progress file
				$contents = file_get_contents($statusFile);
				$contents = rtrim($contents);
				return $contents;
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}
}
