<?php

namespace Renew\Utilities;

/**
 * This class handles the validation and uploading of PHP files
 *
 * @package default
 * @author JPul
 **/
class Uploader
{
    /**
     * The maximum file size for the upload (in MB)
     *
     * @var integer
     * @access public
     **/
    public $maxFileSize = null;
    /**
     * An array of errors with the file
     *
     * @var array
     **/
    public $errors = array();
    /**
     * The destination for the newly created file. The absolute Server path.
     *
     * @var string
     * @access private
     **/
    private $destinationPath = '';
    /**
     * The permissions to set on the folder
     *
     * @var integer
     * @access private
     **/
    private $directoryPermissions = 0775;
    /**
     * The file object to upload
     *
     * @var array
     * @access private
     **/
    private $file = array();
    /**
     * Initialize the class
     *
     * @param string $destinationPath the absolute path to where the file will be saved.
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($destinationPath)
    {
        if (!$this->setDestination($destinationPath)) {
            throw new \Exception("Can't create the upload destination folder.");
        }
    }
    /**
     * Do the upload of the file.  If successful, it returns the new filename
     *
     * @param array $file the file array information
     * @return string
     * @access public
     * @author JPul
     **/
    public function doUpload($file)
    {
        $this->setFileData($file);
        if ($this->isValid()) {
            $status = move_uploaded_file(
                $this->file["tmp_name"],
                $this->file["full_path"]
            );

            if (!$status) {
                $this->errors[] = _('The file could not be uploaded');
                throw new \Exception("Can't upload the passed file");
            } else {
                return basename($this->file['full_path']);
            }
        }
        return "";
    }
    /**
     * checks if the upload is a valid file for uploading
     *
     * @return boolean
     * @access private
     * @author JPul
     **/
    private function isValid()
    {
      try {
        if (!is_null($this->maxFileSize)) {
            if ($this->file['size_in_MB'] > $this->maxFileSize) {
                $this->errors[] = sprintf(_('File is too large. Upload a file that is %s or less.'), $this->maxFileSize . ' MB');
            }
        }
        if (!empty($this->errors)) {
            throw new \InvalidArgumentException("The file did not pass the validation requirements.");
            return FALSE;
        }
        return TRUE;
      } catch (\Exception $e) {
        error_log($e);
        return FALSE;
      }
    }
    /**
     * Sets the $file variable of this class to the file data
     *
     * @param array $file the file array information
     * @return void
     * @access private
     * @author JPul
     **/
    private function setFileData($file)
    {
        $mimeType = $this->getMimeType($file['tmp_name']);
        $extension = $this->getExtension($file['name']);
        $fileSize = $this->getFileSize($file['tmp_name']);
        $fileName = $this->getFileName($file['name']);
        $fullPath = $this->destinationPath . DIRECTORY_SEPARATOR . $fileName;
        $this->file = array(
            'error'                 =>  $file['error'],
            'full_path'             =>  $fullPath,
            'original_mime_type'    =>  $file['type'],
            'original_name'         =>  $file['name'],
            'tmp_name'              =>  $file['tmp_name'],
            'server_mime_type'      =>  $mimeType,
            'size_in_bytes'         =>  $fileSize,
            'size_in_MB'            =>  $this->bytestoMB($fileSize)
        );
    }
    /**
     * gets the mime type of the file provided
     *
     * @param $file the absolute path to file to check
     * @return string the mime type
     * @access private
     * @author JPul
     **/
    private function getMimeType($file)
    {
        $finfo = finfo_open(FILEINFO_MIME_TYPE);
        return finfo_file($finfo, "$file");
    }
    /**
     * get the file extension
     *
     * @param $file the absolute path to file to check
     * @return string (extension with no preceding period)
     * @access public
     * @author JPul
     **/
    private function getExtension($file)
    {
        $pathParts = pathinfo($file);
        return $pathParts['extension'];
    }
    /**
     * get the bit size of the file
     *
     * @param $file the absolute path to the file to calculate the size of
     * @return integer
     * @access private
     * @author JPul
     **/
    private function getFileSize($file)
    {
        return filesize($file);
    }
    /**
     * Sets the destination path for the file
     *
     * @param string $destinationPath Absolute path where the file is going
     * @return boolean Was it able to set the path?
     * @access private
     * @author JPul
     **/
    private function setDestination($destinationPath)
    {
        /**
         * Ensure we have a DIRECTORY_SEPERATOR at the end
         */
        $this->destinationPath = rtrim($destinationPath, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR;
        return $this->destinationPathExists() ?: $this->createDestinationPath();
    }
    /**
     * Checks if the folder exists and is writable
     *
     * @return boolean
     * @access private
     * @author JPul
     **/
    private function destinationPathExists()
    {
        return is_writable($this->destinationPath);
    }
    /**
     * If possible it will create a writable folder for the given destination path
     *
     * @return void
     * @author JPul
     **/
    private function createDestinationPath()
    {
        return mkdir($this->destinationPath, $this->directoryPermissions, true);
    }
    /**
     * Converts bytes into megabytes
     *
     * @param integer $bytes the bytes to convert
     * @return integer
     * @access private
     * @author JPul
     **/
    private function bytestoMB($bytes)
    {
        return round(($bytes / 1048576), 2);
    }
    /**
     * Prepares the filename to be in a specific format
     *
     * @param string $filename the current file name with extension (no path)
     * @return string
     * @access private
     * @author JPul
     **/
    private function getFileName($filename)
    {
        $fullFilePath = $this->destinationPath . DIRECTORY_SEPARATOR . $filename;
        $timestamp = date('YmdHis');
        if (file_exists($fullFilePath)) {
            return $timestamp . "-" . $filename;
        } else {
            return $filename;
        }
    }
}
