<?php
namespace Renew;
require_once('Renew' . DIRECTORY_SEPARATOR . 'DiskInfo.php');

/**
* Class used to validate data
*
* @author JP
*/
class Validator {

  // Class constants
  // Image constants
  const IMAGE_MAX_SIZE_MB = 20;
  const IMAGE_MAX_SIZE_BYTES = self::IMAGE_MAX_SIZE_MB * 1000000;
  const IMAGE_ALLOWED_MIME_TYPES = array(
      'image/png',
      'image/gif',
      'image/jpeg',
      'image/pjpeg'
  );
  // NOTE: if these have a period in front it breaks fineuploader
  const IMAGE_ALLOWED_EXTENSIONS = array(
      'png',
      'gif',
      'jpeg',
      'jpg'
  );
  const TEXT_INPUT_CHARACTER_LIMIT = 500;
  const TEXT_FIELD_CHARACTER_LIMIT = 15000;

  // General constants
  const IMAGE_SELECT_DEFAULT_STRING = 'default';
  const IMAGE_SELECT_CURRENT_STRING = 'current';
  const IMAGE_SELECT_CURRENT_THUMBNAIL_STRING = 'current_thumbnail';
  const STRING_TRUE = 'true';
  const STRING_FALSE = 'false';
  const EMAIL_MAX_STRING_LENGTH = 250;
  const URL_MAX_STRING_LENGTH = 250;
  const WIFI_PASSWORD_MIN_LENGTH = 8;
  const WIFI_PASSWORD_MAX_LENGTH = 63;

  // Message index constants
  const ERR_IMAGE_TOO_LARGE = 'ERR_IMAGE_TOO_LARGE';
  const ERR_IMAGE_NOT_ALLOWED = 'ERR_IMAGE_NOT_ALLOWED';
  const ERR_INPUT_BLANK = 'ERR_INPUT_BLANK';
  const ERR_INPUT_TOO_LONG = 'ERR_INPUT_TOO_LONG';
  const ERR_INPUT_NOT_BETWEEN = 'ERR_INPUT_NOT_BETWEEN';
  const ERR_NOT_IMAGE = 'ERR_NOT_IMAGE';
  const ERR_EMPTY = 'ERR_EMPTY';
  const ERR_NOT_HEX = 'ERR_NOT_HEX';
  const ERR_SAVE = 'ERR_SAVE';
  const ERR_NOT_EMAIL = 'ERR_NOT_EMAIL';
  const ERR_NOT_URL = 'ERR_NOT_URL';
  const ERR_ARRAY_NOT_UNIQUE = 'ERR_ARRAY_NOT_UNIQUE';
  const ERR_THUMBNAIL_FAILED = 'ERR_THUMBNAIL_FAILED';
  const ERR_PASSWORDS_DO_NOT_MATCH = 'ERR_PASSWORDS_DO_NOT_MATCH';
  const ERR_WIFI_PASSWORD_INVALID = 'ERR_WIFI_PASSWORD_INVALID';

  // Instance Variables
  private $formErrors;
  private $errorMessageArray;

  public function __construct($errorMessageOverrideArray = array()) {
    // Array of strings
    $formErrors = array();

    // Translated String Error Messages
    $this->errorMessageArray = self::getErrorMessageArray();

    // Optionally override error messages
		foreach ($errorMessageOverrideArray as $key => $value) {
      $this->errorMessageArray[$key] = $value;
    }
  }

  public static function getErrorMessageArray() {
    return array(
      self::ERR_IMAGE_TOO_LARGE => sprintf(_('File is too large. Upload a file that is %s or less.'), \Renew\DiskInfo::BytesToHumanReadableSize(self::IMAGE_MAX_SIZE_BYTES)),
      self::ERR_IMAGE_NOT_ALLOWED => sprintf(_('File type not allowed. Upload one of these file types: %s'), 'jpeg, png, gif'),
      self::ERR_NOT_IMAGE => _('Image upload failed or selection is not an image'),
      self::ERR_EMPTY => _('Selection or input is empty'),
      self::ERR_NOT_HEX => _('The color is not formatted correctly'),
      self::ERR_INPUT_BLANK => _('Fields cannot be left blank'),
      self::ERR_INPUT_TOO_LONG => _('Text input too long (cannot be longer than %s characters)'),
      self::ERR_INPUT_NOT_BETWEEN => _('Text input must have a length between %s and %s characters'),
      self::ERR_SAVE => _('Save failed'),
      self::ERR_NOT_EMAIL => _('Email format incorrect'),
      self::ERR_NOT_URL => _('URL format incorrect'),
      self::ERR_ARRAY_NOT_UNIQUE => _('Duplicate values are not allowed'),
      self::ERR_THUMBNAIL_FAILED => _('Create New Thumbnail Failed'),
      self::ERR_PASSWORDS_DO_NOT_MATCH => _('Passwords do not match'),
      self::ERR_WIFI_PASSWORD_INVALID => _('WiFi password must contain only numbers and letters')
    );
  }

  /**
  * Check if input is an int, and if it is between two integer values
  *
  * @author JP
  * @param val The input integer to check
  * @param lowInt
  * @param highInt
  */
  public static function intBetween($val, $lowInt, $highInt) {
    if (!self::isInt($val)) {
      return FALSE;
    }
    if ($val < $lowInt || $val > $highInt) {
      return FALSE;
    }
    return TRUE;
  }

  /*
    @return bool TRUE if is integer, FALSE if not
  */
  public static function isInt($val) {
    if (!ctype_digit($val)) {
      return FALSE;
    }
    return TRUE;
  }

  /*
    @param $string
    @param $regexExpression Regex that string must match
    @return bool TRUE if string matches a regex expression
  */
  public static function stringMatchesRegex($string, $regexExpression) {
    if (!preg_match($regexExpression, $string)){
      return FALSE;
    }
    return TRUE;
  }

  /**
  * Check if input is an int, and if it is between two integer values
  *
  * @param val The input string to check
  * @param length
  */
  public static function stringLengthLessOrEqual($val, $length) {
    return self::stringLengthBetweenInclusive($val, 0, $length);
  }

  /*
    TODO Comment
  */
  public static function stringLengthBetweenInclusive($val, $minLength, $maxLength) {
    if (gettype($val) !== 'string') {
      return FALSE;
    }
    $stringLength = strlen($val);
    if ($stringLength > $maxLength) {
      return FALSE;
    } else if ($stringLength < $minLength) {
      return FALSE;
    }
    return TRUE;
  }

  /*
    TODO Comment
  */
  public static function isHexString($string) {
    if ($string[0] !== '#') {
      return FALSE;
    }
    $string = str_replace('#', '', $string);
    if (strlen($string) === 6 && ctype_xdigit($string)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Check if array doesn't contain any FALSE boolean values
  *
  * @author JP
  * @param array The array of boolean values
  */
  public static function booleanArrayAllTrue($array) {
    if (gettype($array) !== 'array') {
      return FALSE;
    }
    foreach ($array as $item) {
      if ($item === FALSE) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /*
    TODO Comment
  */
  public static function arrayUnique($array) {
    return count($array) === count(array_unique($array));
  }

  /**
  * Take a standard checkbox input of "on" or empty and turn into 'true' or 'false'
  *
  * @author JP
  * @param checkboxString The POST data string from the checkbox
  * @return 'true' or 'false'
  */
  public static function formatCheckbox($checkboxString) {
    return self::FormatCheckboxHelper($checkboxString, 'true', 'false');
  }

  /*
    Makes sure that a radio with 'true' and 'false' options isn't empty

    @param $radioString a 'true' or 'false' string
  */
  public static function formatTrueFalseRadio($radioString) {
    if ($radioString === 'true' || $radioString === 'false') {
      return $radioString;
    }
    return 'false';
  }

  /*
  TODO Comment
  */
  public static function formatCheckboxBoolean($checkboxString) {
    return self::FormatCheckboxHelper($checkboxString, TRUE, FALSE);
  }

  /*
  TODO Comment
  */
  public static function formatCheckboxOneZero($checkboxString) {
    return self::FormatCheckboxHelper($checkboxString, 1, 0);
  }

  /*
    TODO Comment
  */
  public static function formatCheckboxAvailability($checkboxString) {
    return self::FormatCheckboxHelper($checkboxString, 2, 0);
  }

  /*
    TODO Comment
  */
  public static function FormatCheckboxShowFirst($checkboxString) {
    return self::FormatCheckboxHelper($checkboxString, 1, NULL);
  }

  /*
    TODO Comment
  */
  public static function FormatCheckboxHelper($checkboxString, $onValue, $offValue) {
    if ($checkboxString === 'on') {
      return $onValue;
    } else {
      return $offValue;
    }
  }

  /*
    TODO Comment
  */
  public static function formatURLRemoveHTTPStrings($url) {
    $url = str_replace('http://','', $url);
    $url = str_replace('https://','', $url);
    return $url;
  }

  /**
  * @param string $string
  * @return boolean
  */
  public static function StringToBoolean($string) {
    if ($string === 'true') {
      return TRUE;
    } else if ($string === 'false') {
      return FALSE;
    } else {
      return $string;
    }
  }

  /*
    TODO Comment
  */
  public function validateTextInput($text, $formErrorPrefix) {
    return $this->validateTextInputMaxLengthNotBlank($text, $formErrorPrefix, self::TEXT_INPUT_CHARACTER_LIMIT);
  }

  /*
    TODO Comment
  */
  public function validateTextField($text, $formErrorPrefix) {
    return $this->validateTextInputMaxLengthNotBlank($text, $formErrorPrefix, self::TEXT_FIELD_CHARACTER_LIMIT);
  }

  /*
    TODO Comment
  */
  public function validateTextLengthBetween($text, $formErrorPrefix, $minLength, $maxLength) {
    $success = TRUE;
    $success = $this->validateNotBlank($text, $formErrorPrefix);
    if (self::stringLengthBetweenInclusive($text, $minLength, $maxLength) === FALSE) {
      $this->addFormErrorString($formErrorPrefix . ' - ' . sprintf($this->errorMessageArray[self::ERR_INPUT_NOT_BETWEEN], $minLength, $maxLength));
      $success = FALSE;
    }
    return $success;
  }

  /**
  * TODO Comment
  */
  public function validateTextInputMaxLengthNotBlank($text, $formErrorPrefix, $maxLength) {
    $success = TRUE;
    $success = $this->validateNotBlank($text, $formErrorPrefix);
    if (self::stringLengthLessOrEqual($text, $maxLength) === FALSE) {
      $this->addFormErrorString($formErrorPrefix . ' - ' . sprintf($this->errorMessageArray[self::ERR_INPUT_TOO_LONG], $maxLength));
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateIntInputBetween($intInput, $formErrorPrefix, $min, $max) {
    $success = TRUE;
    $success = $this->validateNotBlank($intInput, $formErrorPrefix);

    if (self::intBetween($intInput, $min, $max) === FALSE) {
      $this->addFormErrorString(
        $formErrorPrefix . ' - '
        . sprintf(_('Must be a number between %s and %s'), $min, $max));
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateInt($intInput, $formErrorPrefix) {
    $success = TRUE;
    $success = $this->validateNotBlank($intInput, $formErrorPrefix);

    if (self::isInt($intInput) === FALSE) {
      $this->addFormErrorString($formErrorPrefix . ' - ' . _('Must be a number'));
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateTextInputLength($text, $formErrorPrefix) {
    return $this->validateTextInputMaxLength($text, $formErrorPrefix, self::TEXT_INPUT_CHARACTER_LIMIT);
  }

  /*
    TODO Comment
  */
  public function validateTextFieldLength($text, $formErrorPrefix) {
    return $this->validateTextInputMaxLength($text, $formErrorPrefix, self::TEXT_FIELD_CHARACTER_LIMIT);
  }

  /*
    TODO Comment
  */
  public function validateTextInputMaxLength($text, $formErrorPrefix, $maxLength) {
    $success = TRUE;
    if (self::stringLengthLessOrEqual($text, $maxLength) === FALSE) {
      $this->addFormErrorString($formErrorPrefix . ' - ' . sprintf($this->errorMessageArray[self::ERR_INPUT_TOO_LONG], $maxLength));
      $success = FALSE;
    }
    return $success;
  }

  /* TODO Comment */
  public function validateInputEmail($text, $formErrorPrefix, $allowEmpty) {
    $success = TRUE;

    if ($allowEmpty && empty($text)) {
      return TRUE;
    }

    // Validate not empty, length
    $success = $this->validateTextInput($text, $formErrorPrefix, self::EMAIL_MAX_STRING_LENGTH);

    // Validate email
    if(!filter_var($text, FILTER_VALIDATE_EMAIL)) {
      $this->addFormError($formErrorPrefix, self::ERR_NOT_EMAIL);
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateWifiPassword($wifiPassword, $formErrorPrefix) {
    $this->validateTextLengthBetween($wifiPassword, $formErrorPrefix, self::WIFI_PASSWORD_MIN_LENGTH, self::WIFI_PASSWORD_MAX_LENGTH);
    if (!self::stringMatchesRegex($wifiPassword, "/^[a-zA-Z0-9]*$/")) {
      $this->addFormError($formErrorPrefix, self::ERR_WIFI_PASSWORD_INVALID);
    }
  }

  
  public function validateLegalUrl($url, $formErrorPrefix) {
    $url = self::formatURLRemoveHTTPStrings(strtolower($url));
    $url = str_replace('www.', '', $url);
    $success = strpos($url, 'facebook.') === false
      && strpos($url, 'google.') === false
      && strpos($url, 'amazon.') === false
      && strpos($url, 'youtube.') === false
      && strpos($url, 'baidu.') === false;
    if (!$success) {
      $this->addFormError($formErrorPrefix, self::ERR_NOT_URL);
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateSimpleUrl($url, $formErrorPrefix, $maxLength) {
    try {
      $success = TRUE;
      if (!$maxLength) {
        $maxLength = self::URL_MAX_STRING_LENGTH;
      }
      if(!$this->validateTextInputMaxLengthNotBlank($url, $formErrorPrefix, $maxLength)) {
        $success = FALSE;
      }
      $regex="/^[a-zA-Z0-9]+(\.[a-zA-Z0-9]+)$/";
      if (!self::stringMatchesRegex($url, $regex)) {
        $this->addFormError($formErrorPrefix, self::ERR_NOT_URL);
        $success = FALSE;
      }
      return $success;
    } catch (\Exception $e) {
      error_log($e);
      return FALSE;
    }
  }

  /*
    TODO Comment
  */
  public function validateNotEmpty($input, $formErrorPrefix) {
    return $this->validateNotEmptyBlankHelper($input, $formErrorPrefix, self::ERR_EMPTY);
  }

  /*
    TODO Comment
  */
  public function validateNotBlank($input, $formErrorPrefix) {
    return $this->validateNotEmptyBlankHelper($input, $formErrorPrefix, self::ERR_INPUT_BLANK);
  }

  /*
    TODO Comment
  */
  private function validateNotEmptyBlankHelper($input, $formErrorPrefix, $errorCode) {
    $success = TRUE;
    if ($input === '' || $input === NULL || $input === array()) {
      $this->addFormError($formErrorPrefix, $errorCode);
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function validateHexString($string, $formErrorPrefix) {
    $success = TRUE;
    if (!self::isHexString($string)) {
      $this->addFormError($formErrorPrefix, self::ERR_NOT_HEX);
      $success = FALSE;
    }
    return $success;
  }

  /**
  * TODO Comment
  */
  public function validateImage($imagePathAbs, $formErrorPrefix) {
    $success = TRUE;
    if (!isset($imagePathAbs) || empty($imagePathAbs)) {
      $this->addFormError($formErrorPrefix, self::ERR_NOT_IMAGE);
      return FALSE;
    }

    $mimeType = mime_content_type($imagePathAbs);
    //$imageSize = filesize($imagePathAbs);

    // Validate file type
    if (!in_array($mimeType, self::IMAGE_ALLOWED_MIME_TYPES)) {
        $this->addFormError($formErrorPrefix, self::ERR_IMAGE_NOT_ALLOWED);
        $success = FALSE;
    }

    // // Validate size
    // if ($imageSize > self::IMAGE_MAX_SIZE_BYTES) {
    //   $this->addFormError($formErrorPrefix, self::ERR_IMAGE_TOO_LARGE);
    //   $success = FALSE;
    // }

    return $success;
  }

  /*
    TODO Comment
  */
  public function validateArrayOfArraysNoDuplicateParams($arrayOfArrays, $formErrorPrefix, $param) {
    $success = TRUE;
    $paramValues = array();
    foreach($arrayOfArrays as $array) {
      $paramValues[] = $array[$param];
    }
    if (!(self::arrayUnique($paramValues))) {
      $this->addFormError($formErrorPrefix, self::ERR_ARRAY_NOT_UNIQUE);
      $success = FALSE;
    }
    return $success;
  }

  /*
    TODO Comment
  */
  public function getFormErrors() {
    return $this->formErrors;
  }

  /*
    TODO Comment
  */
  public function formErrorsIsEmpty() {
    if (empty($this->formErrors)) {
      return TRUE;
    }
    return FALSE;
  }

  /*
    TODO Comment
  */
  public function addFormError($formErrorPrefix, $errorCode) {
    $errorMessage = $this->errorMessageArray[$errorCode];
    if (!$errorMessage) {
      $errorMessage =  $errorCode;
      error_log('Error: Message for error message "' . $errorCode . '" not defined.');
    }
    $prefix = '';
    if (!empty($formErrorPrefix)) {
      $prefix = $formErrorPrefix . ' - ';
    }
    $this->formErrors[] = $prefix . $errorMessage;
  }

  /*
    TODO Comment
  */
  public function addFormErrorString($formErrorString) {
    $this->formErrors[] = $formErrorString;
  }

  /*
    TODO Comment
  */
  public function addFormErrorStringArray($formErrorStringArray) {
    foreach($formErrorStringArray as $formErrorString) {
      $this->addFormErrorString($formErrorString);
    }
  }

  /*
    TODO Comment
  */
  public function addFormErrorSaveFailed() {
    $this->addFormError('', self::ERR_SAVE);
  }

  /*
    TODO Comment
  */
  public function ajaxFormResponse($successArgument = FALSE, $failArgument = FALSE) {
    if ($this->formErrorsIsEmpty()) {
      $answer['success'] = TRUE;
      $answer['response'] = '';
      $answer['argument'] = FALSE;
      if ($successArgument !== FALSE) {
        $answer['argument'] = $successArgument;
      }
    } else {
      $answer['success'] = FALSE;
      $answer['response'] = implode("<br>", $this->formErrors);
      $answer['argument'] = FALSE;
      if ($failArgument !== FALSE) {
        $answer['argument'] = $failArgument;
      }
    }
    return json_encode($answer);
  }
}
