#!/bin/bash

# includes ...
. "/usr/bin/rwo_includeLogging.sh"
. "/usr/bin/rwo_includeLeds.sh"
. "/usr/bin/rwo_includeHelpers.sh"

##### BEGIN FUNCTIONS #####

function lockButtonSetup {
	LOCK_BTN_GPIO=192
	LOCK_BTN_GPIO_DIR="/sys/class/gpio/gpio${LOCK_BTN_GPIO}"
	if [[ ! -d "${LOCK_BTN_GPIO_DIR}" ]]; then
		echo "${LOCK_BTN_GPIO}" > "/sys/class/gpio/export"
		echo "in" > "${LOCK_BTN_GPIO_DIR}/direction"
		echo "both" > "${LOCK_BTN_GPIO_DIR}/edge"
	fi
}

function lockButtonWaitForLow {
	lockButtonSetup
	while true; do
		interrupt=$(rwo_gpioWaitForInterrupt ${LOCK_BTN_GPIO})
		if [[ "${interrupt}" == "low" ]]; then
			return
		fi
	done
}

function _waitForeverAndToggleLock {
	sdLockDisabled=$(helperSqlite3 "${LSSD_SQLITE3}" "SELECT value FROM settings WHERE key = 'sd_lock_disabled'")
	if [[ "${HWPLATFORM}" == 'pocket-v1' ]] && [[ "${BASE_DEVICE}" =~ ^mmcblk ]] && [[ "$sdLockDisabled" != "true" ]]; then

		action="lock"
		while [[ $WAIT_FOREVER_LOOP == 1 ]] ; do
			lockButtonWaitForLow
			if [[ "${action}" == "lock" ]]; then
				sdtool "/dev/${BASE_DEVICE}" lock
				ledsSdSetLockLed 1
				action="unlock"
			else
				sdtool "/dev/${BASE_DEVICE}" unlock
				ledsSdSetLockLed 0
				action="lock"
			fi
		done
		# not a pocket SD card
	else
		_waitForever
	fi

	exit 0
}

function _waitForever {
	while [[ $WAIT_FOREVER_LOOP == 1 ]]; do
		sleep 0.5
	done
	exit 0
}

function checkIfBootDeviceExitIfYes {
	if [[ "${ROOTFS_DEVICE_PARTITION}" =~ ^${BASE_DEVICE} ]]; then
		# it is the boot device, do not copy media to it
		rwoLog "detected rootfs device:${BASE_DEVICE} ... exit"
		exit 0
	fi
}

function checkIfShouldDoMirrorCreateDoIfYes {
	# check for the mirrorCreate flag, but only for an mmc
	if [[ "${BASE_DEVICE}" =~ ^mmcblk[0-9]$ ]]; then
		if [[ -f "${MIRROR_CREATE_FLAG_FILE}" ]]; then
			# check if the flag has timed out (the flag was created too long ago)
			secondsAgo=$(($(date +%s) - $(date +%s -r "${MIRROR_CREATE_FLAG_FILE}")))
			# delete the flag
			rm "${MIRROR_CREATE_FLAG_FILE}" > "${MIRROR_CREATE_STATUS_FILE}"
			if [[ ${secondsAgo} -gt ${FLAG_TIMEOUT} ]] || [[ ${secondsAgo} -lt 0 ]]; then
				echo "${STATUS_ERROR_TIMEOUT}" > "${MIRROR_CREATE_STATUS_FILE}"
			else
				# it's time to create the mirror with this device .. overwrite the function
				doFunctionMirrorCreate
			fi
		fi
	fi
}

function checkIfShouldCreateUpdateDrive {
	# check for the updateDriveCreate flag, but only for an mmc
	if [[ "${BASE_DEVICE}" =~ ^sd[a-z]$ ]]; then
		if [[ -f "${UPDATE_DRIVE_CREATE_FLAG_FILE}" ]]; then
			# check if the flag has timed out (the flag was created too long ago)
			secondsAgo=$(($(date +%s) - $(date +%s -r "${UPDATE_DRIVE_CREATE_FLAG_FILE}")))
			# delete the flag
			rm "${UPDATE_DRIVE_CREATE_FLAG_FILE}"
			if [[ ${secondsAgo} -gt ${FLAG_TIMEOUT} ]] || [[ ${secondsAgo} -lt 0 ]]; then
				echo "${STATUS_ERROR_TIMEOUT}" > "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
			else
				# it's time to create
				doFunctionUpdateDriveCreate
			fi
		fi
	fi
}

function checkIfShouldRunMirrorAndRunIfYes {
	# must be an mmc device
	if [[ "${BASE_DEVICE}" =~ ^mmcblk[0-9]$ ]]; then
		FIRST_PARTITION="${BASE_DEVICE}p1"
		SECOND_PARTITION="${BASE_DEVICE}p2"

		# check to see if it's likely the first boot after a mirror success
		if [[ -e "${MIRROR_DONE_FILE}" ]]; then
			rm "${MIRROR_DONE_FILE}"
			rm "${MIRROR_RUN_BOOT_MIRROR_FILE}"
			exit 1
		fi

		if [[ ! -e "/dev/${SECOND_PARTITION}" ]]; then
			# second partition does not exist, so it's not a mirror drive
			return
		fi
		partuuid=$(helperGetPartUuidForDevice "${SECOND_PARTITION}")
		rootFsPartUuid=$(helperGetPartUuidForDevice "${ROOTFS_DEVICE_PARTITION}")
		# the case where you have inserted a mirror card but not yet booted from a mirror
		if [[ $partuuid == 'aa1212cd-02' ]]; then
			doFunctionMirrorRunConfirm
		# this is the case where you have booted from a mirror card ...
		elif [[ "${rootFsPartUuid}" == 'aa1212cd-02' ]] && [[ "${partuuid}" == 'b0075dcd-02' ]] && [[ -e "${MIRROR_RUN_BOOT_MIRROR_FILE}" ]]; then
			doFunctionMirrorRun
		# you have inserted a bootsdcard
		elif [[ "${partuuid}" == 'b0075dcd-02' ]]; then
			rwoLog "detected boot sd card partuuid"
			_deviceWriteMirrorSdCardPartUuidToDevice "${BASE_DEVICE}"
			devicePartitionMountExitAndFlashIfFail "${FIRST_PARTITION}"
			# touch the mirror boot file so when you put the card in later it can be run
			touch "/mnt/${FIRST_PARTITION}/mirrorBoot.opt"
			doFunctionMirrorRunConfirm
		fi
	fi
}

function checkIfShouldWriteImageDoFunctionWriteImageIfYes {
	if [[ -f "${WRITE_IMAGE_FILE}" ]] && [[ "${BASE_DEVICE}" =~ ^mmcblk[0-9]$ ]]; then
		doFunctionWriteImage
	fi
}

# checks if the device you just mounted is an upload Device
# if it is an upload device, it will unmount it, call the upload service, and exit
function checkForUploadDeviceWithDevicePartition {
	local devicePartition="${1}"
	local isUploadDevice=0
	local arclightJsonPath="/mnt/${devicePartition}/arclight.json"

	# check for our special LightStream serial number pattern
	serialMatch=$(udevadm info -a "/dev/${devicePartition}" | grep "ATTRS{serial}==\"3F094508FF8291.*\"" | wc -l)
	if [[ $serialMatch == 1 ]]; then
		rwoLog "serial numbers match for upload device ... start upload"
		isUploadDevice=1
	else
		# check for a text file that matches the regex
		for txtFile in $( find "/mnt/${devicePartition}/" -maxdepth 1 -type f -name *.txt ); do
			hasPassphrase=$(head -n 1 "${txtFile}" | grep -i "^SHIBBOLETH$" | wc -l)
			if [[ $hasPassphrase == 1 ]]; then
					rwoLog "found upload device with special passphrase:${devicePartition}"
					isUploadDevice=1
					break
			fi
		done
	fi

	if [[ -f "$arclightJsonPath" ]]; then
		isUploadDevice=1
	else
		arclightJsonPath=""
	fi

	if [[ $isUploadDevice == 1 ]]; then
		if [[ -d "/mnt/${devicePartition}/_testHardware" ]]; then
			doFunctionTestHardwareWithDevicePartition "${devicePartition}"
		elif [[ -d "/mnt/${devicePartition}/_qcRuntimeTest" ]]; then
			doFunctionQcRuntimeTestWithDevicePartition "${devicePartition}"
		else
			doFunctionUploadWithDevicePartition "${devicePartition}" "${arclightJsonPath}"
		fi
	fi
}

function devicePartitionMountExitAndFlashIfFail {
	local devicePartition="${1}"
	local options="${2}"
	# mount the device partition
	devicePartitionMount "${devicePartition}" "${options}"
	devicePartitionIsMounted
	mountSuccess=$?
	if [[ ${mountSuccess} != 0 ]]; then
		rwoLog "failed to mount device partition:${devicePartition} ... exiting"
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi
}

# Mount download device to fresh mount directory.
function devicePartitionMount {
	local devicePartition="${1}"
	local options="${2}"
	local mountPoint="/mnt/${1}"
	devicePartitionIsMounted "${devicePartition}"
	if [[ ${retVal} != 0 ]]; then
		rwoLog "Creating fresh mount directory for ${devicePartition} ..."
		mkdir  "${mountPoint}"
		rwoLog "Mounting ${devicePartition} to mount directory..."
		if [[ "${devicePartition}" == "$MTP_DEVICE" ]]; then
			simple-mtpfs "${mountPoint}"
			while [[ ! -d "${mountPoint}/Phone" ]]; do
				sleep 0.5
			done
		else
			if [[ "${options}" == "utf8" ]]; then
				# try not to specify the type, insert the options here
				mount -o iocharset=utf8 "/dev/${devicePartition}" "${mountPoint}"
			else
				# try not to specify the type, insert the options here
				mount "/dev/${devicePartition}" "${mountPoint}"
			fi
		fi
		devicePartitionIsMounted "${devicePartition}"
	fi
}


# Safely prepare to remove download device from system.
function devicePartitionUnmount {
	local devicePartition="${1}"
	local mountPoint="/mnt/${1}"
	# Unmount the partitions and boot volume of the USB SCSI disk device.
	checkIfBootDeviceExitIfYes
	rwoLog "Unmounting ${devicePartition} from system..."
	devicePartitionIsMounted "${devicePartition}"
	#count=0
	if [[ ${retVal} == 0 ]]; then
		umount "/dev/${devicePartition}"
		sleep 0.1
		umount "${mountPoint}"
		sleep 0.1
		devicePartitionIsMounted "${devicePartition}"
		sleep 0.1
		#count=$(( $count + 1 ))
	fi

	# check if the device partition is finally unmounted
	devicePartitionIsMounted "${devicePartition}"
	if [[ ${retVal} == 1 ]]; then
		rm -R "${mountPoint}"
	else
		rwoLog "ERROR: there was a problem unmounting the device"
	fi

	rmdir "${mountPoint}"
}

# set retVal to 0 if mounted and 1 if unmounted
function devicePartitionIsMounted {
	local devicePartition="${1}"
	retVal=0
	if [ ! -n "$(mount | grep "/mnt/${devicePartition}")" ]; then
		retVal=1
		return 1
	else
		return 0
	fi
}


function _deviceWriteMirrorSdCardPartUuidToDevice {
	# write 440B into the SD card the PTUUID will be a 4 byte value in reverse order
	# the 4 byte value below comes out to be aa1212cd (i.e. "MRR cd" .. ie "Mirror Card" )
	# ... it can be any value but did not seem to like all 0's
	# this value should match what is in the uboot script image in order to look for the sd card
	# you want to sign the card at the end with this so that you know it was a success
	local baseDevice="${1}"
	echo -e "\xcd\x12\x12\xaa" | dd bs=1 count=4 of="/dev/${baseDevice}" seek=440 conv=fdatasync
}


function doFunctionMirrorCreate {

	# calcualte how large our mirror file needs to be
	# do this by getting space used for boot and root and then add a buffer
	FUNCTION="mirrorCreate"
	echo "0" > "${MIRROR_CREATE_PERCENTAGE_FILE}"
	echo "$STATUS_IN_PROGRESS" > "${MIRROR_CREATE_STATUS_FILE}"
	# set this variable so that the progress from the LEDs gets written here
	LEDSPROGRESSFILE="${MIRROR_CREATE_PERCENTAGE_FILE}"

	ledsFlashLoopStart ledsSdSetLedOn 1 ledsSdSetLedOff 1 0.1

	# keep in mind, this should be an mmcblk device
	FIRST_PARTITION="${BASE_DEVICE}p1"
	SECOND_PARTITION="${BASE_DEVICE}p2"
	MIRROR_MOUNT_POINT_BOOT="/mnt/${FIRST_PARTITION}"
	MIRROR_MOUNT_POINT_ROOT="/mnt/${SECOND_PARTITION}"

	sizeRootFs=$(helperGetSizeUsedForMountPoint '/')
	sizeBootFs=$(helperGetSizeUsedForMountPoint '/boot')
	sizeBuffer=100000000
	sizeMirror=$(( $sizeRootFs + $sizeBootFs ))
	sizeMirrorAndBuffer=$(( $sizeMirror + $sizeBuffer ))

	# check if you have enough space for the mirror
	# you do not have enough space blink the error light and exit
	if [[ "${sizeMirrorAndBuffer}" -gt "${BASE_DEVICE_SIZE}" ]]; then
		rwoLog "the size of the mirror creation drive is too small ... estimated size needed plus buffer:${sizeMirrorAndBuffer} size of drive:${BASE_DEVICE_SIZE}"
		echo "$STATUS_ERROR_TOO_SMALL" > "${MIRROR_CREATE_STATUS_FILE}"
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi

	# Unlock the card in case it is locked
	sdtool "/dev/${BASE_DEVICE}" unlock
	sleep 0.25

	# clear out all of the partiton table and spl file
	dd if=/dev/zero of="/dev/${BASE_DEVICE}" bs=1M count=1 conv=fdatasync
	#echoY "will write spl and u-boot to card at proper location"
	#dd if="${ubootsunxisplfile}" of="${writedev}" bs=1024 seek=8
	#echoY "will partiton card with boot and root partitions"

# write the partitons
blockdev --rereadpt /dev/${BASE_DEVICE}
cat <<EOT | sfdisk /dev/${BASE_DEVICE}
1M,100M,c
,,L
EOT

	# make the filesystems for the partitions
	mkfs.vfat "/dev/${FIRST_PARTITION}"
	mkfs.ext4 "/dev/${SECOND_PARTITION}"
	sync

	# allocate space for the mirror file
	# use seek in dd for quick allocation
	# then make the ext4 filesystem in the file
	#timestamp=$(date +"%Y%m%dT%H%M%S")
	#mirrorFile="${MOUNT_POINT}/${timestamp}.mirror"
	#dd if=/dev/zero of="${mirrorFile}" bs=1 count=0 seek=$sizeMirror
	#mkfs.ext4 "${mirrorFile}"
	# mount the mirror file]
	devicePartitionMount "${FIRST_PARTITION}"
	devicePartitionMount "${SECOND_PARTITION}"

	# run the blinking leds and update progress in the background
	ledsFlashLoopStop
	ledsSdDoProgressTransferBgStart "${MIRROR_MOUNT_POINT_ROOT}" $sizeMirror 4 0.1

	sleep 1
	sync

	# now copy the root and boot filesystems over to the mirror file
	rwoLog "begin rsync of root filesystem"
	rsyncRootSuccess=$(rsyncNumTimes "/" "${MIRROR_MOUNT_POINT_ROOT}/" 3)
	rwoLog "begin rsync of boot filesystem"
	rsyncBootSuccess=$(rsyncNumTimes "/boot/" "${MIRROR_MOUNT_POINT_BOOT}/" 3)

	# now it's time to check if it's an autorun mirror

	sync "${MIRROR_MOUNT_POINT_BOOT}"
	sync "${MIRROR_MOUNT_POINT_ROOT}"
	sleep 1

	# get the new sizes of your mirror mount points
	sizeBootFsMirror=$(helperGetSizeUsedForMountPoint "${MIRROR_MOUNT_POINT_BOOT}")
	sizeRootFsMirror=$(helperGetSizeUsedForMountPoint "${MIRROR_MOUNT_POINT_ROOT}")
	sizeMirrorFinal=$(( $sizeBootFsMirror + $sizeRootFsMirror ))

	devicePartitionUnmount "${FIRST_PARTITION}"
	devicePartitionUnmount "${SECOND_PARTITION}"

	# check if not enough bytes have been copied
	if [[ $rsyncRootSuccess != 'success' ]] || [[ $rsyncBootSuccess != 'success' ]]; then
		rwoLog "ERROR - there was a problem, the new mirror did not seem to get all of the bytes"
		echo "$STATUS_ERROR_FAILED" > "${MIRROR_CREATE_STATUS_FILE}"
		# stop the leds from blinking ...
		ledsSdDoProgressTransferBgStop
		ledsSdSetLockLed 1
		ledsSdSetUnlockLed 0
		ledsSdShowFinalProgress $sizeMirrorFinal $sizeMirror
	else
		_deviceWriteMirrorSdCardPartUuidToDevice "${BASE_DEVICE}"
		sleep 1
		# Don't lock the card, because you will have to boot off of the card
		#sdtool "/dev/${BASE_DEVICE}" lock

		rwoLog "SUCCESS! Finished the mirror drive"
		echo "$STATUS_SUCCESS" > "${MIRROR_CREATE_STATUS_FILE}"
		echo "100" > "${MIRROR_CREATE_PERCENTAGE_FILE}"

		# stop the leds from blinking ...
		ledsSdDoProgressTransferBgStop
		ledsSdSetLockLed 0
		ledsSdSetUnlockLed 1
		ledsSdShowFinalProgress 100 100
	fi


	_waitForever
}


function doFunctionQcRuntimeTestWithDevicePartition {
	# TODO ... add columns for seconds since test began, mAh and Wh ...
	local devicePartition="$1"
	local serialNumber=$(rwo_system.sh getSerialNumber)
	local runtimeTestFile="/mnt/${devicePartition}/_qcRuntimeTest/${serialNumber}_1.csv"
	local count=0
	local pollingInterval=10
	local runtimeTestStart=$(date +%s)
	local batterymAh=0
	local batterymWh=0
	FUNCTION="qcRuntimeTest"
	while [[ -f "${runtimeTestFile}" ]]; do
		count=$(( $count + 1 ))
		runtimeTestFile="/mnt/${devicePartition}/_qcRuntimeTest/${serialNumber}_${count}.csv"
	done
	touch "${runtimeTestFile}"
	echo "\"Date(YYYY-MM-DDTHH:MM:SS.NNNNNNN)\",\"Time Since Start\",\"Serial Number\",\"Battery Voltage (mV)\",\"Battery Charge Current (mA)\",\"Battery Discharge Current (mA)\",\"Battery Percentage\",\"Battery mAh\",\"Battery mWh\"" >> "${runtimeTestFile}"
	local lastPoll=$(date +%s)
	while true; do
		ledsBatterySetSixOclockLed 0
		local date=$(date +'%Y-%m-%dT%H:%M:%S.%N')
		local batVoltage=$(rwo_battery.sh getBatVoltage)
		local batChargeCurrent=$(rwo_battery.sh getBatChargeCurrent)
		local batDischargeCurrent=$(rwo_battery.sh getBatDischargeCurrent)
		local batPct=$(rwo_battery.sh getBatPct)
		local secondsAgo=$(($(date +%s) - $runtimeTestStart))
		local secondsSinceLastPoll=$(($(date +%s) - $lastPoll))
		local lastPoll=$(date +%s)
		local millionthHours=$(( $secondsSinceLastPoll * 1000000 / 3600 ))
		local thisuAh=$(( $batDischargeCurrent * $millionthHours / 1000 ))
		local thisuWh=$(( $thisuAh * $batVoltage / 1000 ))
		local batteryuAh=$(( $batteryuAh + $thisuAh ))
		local batteryuWh=$(( $batteryuWh + $thisuWh ))
		batterymAh=$(( $batteryuAh / 1000 ))
		batterymWh=$(( $batteryuWh / 1000 ))
		echo "\"${date}\",\"${secondsAgo}\",\"${serialNumber}\",\"${batVoltage}\",\"${batChargeCurrent}\",\"${batDischargeCurrent}\",\"${batPct}\",\"${batterymAh}\",\"${batterymWh}\"" >> "${runtimeTestFile}"
		sync "${runtimeTestFile}"
		ledsBatterySetSixOclockLed 1
		sleep "${pollingInterval}"
	done
}

function doFunctionMirrorRunConfirm {
	FUNCTION="mirrorRunConfirm"
	devicePartitionMountExitAndFlashIfFail "${SECOND_PARTITION}"
	rwoLog "running function to confirm if you should run a mirror"

	local sqlQueryStr="SELECT user_name, password FROM users WHERE id = 1"
	local sqlDatabaseFile="/etc/lightstream/data/lswifi.sqlite3"
	local mirrorUsernamePassword=$(helperSqlite3 "/mnt/${SECOND_PARTITION}${sqlDatabaseFile}" "${sqlQueryStr}")
	local localUsernamePassword=$(helperSqlite3 "${sqlDatabaseFile}" "${sqlQueryStr}")
	local mirrorDeviceSize=$(helperGetSizeInBytesForDevice "${BASE_DEVICE}")
	local rootDeviceSize=$(helperGetSizeInBytesForDevice "${ROOTFS_DEVICE_PARTITION}")
	local mirrorDeviceSpaceUsed=$(helperGetSizeUsedForMountPoint "/mnt/${SECOND_PARTITION}")

	# check to see if the mirror device is too small for
	if [[ $mirrorDeviceSpaceUsed -gt $rootDeviceSize ]]; then
		rwoLog "ERROR: the mirror drive is too large to run on this sd card"
		ledsFlashLoopStart ledsSdSetLockLed 1 ledsSdSetLockLed 1 0.5
		_waitForever
	fi

	ledsFlashLoopStart ledsAllBlueOn 0 ledsAllBlueOff 0 0.5
	while true; do
		# first thing ... see if there is a mirrorRun file already there
		if [[ -e $MIRROR_RUN_FLAG_FILE ]]; then
			# check if the flag has timed out (the flag was created too long ago)
			secondsAgo=$(($(date +%s) - $(date +%s -r "${MIRROR_RUN_FLAG_FILE}")))
			if [[ ${secondsAgo} -gt ${FLAG_TIMEOUT} ]] || [[ ${secondsAgo} -lt 0 ]]; then
				rwoLog "The mirror run flag is too old ... deleting the old flag file"
				rm "${MIRROR_RUN_FLAG_FILE}"
				continue
			else
				rwoLog "Initiating the run mirror because of the flag file ..."
				break
			fi
		fi

		# if the mirrorRun file is not there, or too old, now we can wait for manual confirmation
		lockButtonWaitForLow
		adminAdminLoginTrue=$(php -f /srv/http/lib/Renew/Utilities/CmdLineCheckLogin.php admin admin)
		if [[ "$mirrorUsernamePassword" == "$localUsernamePassword" ]]; then
			rwoLog "Initiating the run mirror because the passwords match ..."
			break
		elif [[ "$adminAdminLoginTrue" == "true" ]]; then
			rwoLog "Initiating the run mirror because the login on this device is admin/admin ..."
			break
		else
			# tell the user that it is locked
			rwoLog "Admin logins do not match. Must give manual confirmation!"
			ledsFlashLockLed 3
		fi
	done

	STATUS="$STATUS_CONFIRMED"
	ledsFlashLoopStop
	touch "${MIRROR_RUN_BOOT_MIRROR_FILE}"
	_ledsDoMirrorRunConfirm
	rwo_system.sh systemRebootNow
	_ledsDoMirrorRunConfirm
	exit 0
}

function doFunctionMirrorRun {

	FUNCTION="mirrorRun"
	rwoLog "Run mirror!"

	rwoLog "Deleting all stats before mirroring ..."
	php -f "/srv/http/lib/Renew/Utilities/CmdLineDeleteAllStats.php"

	# turn on the right leds
	# stop services
	systemctl stop rwo_wifi@wlan0Ap
	systemctl stop rwo_bluetooth

	ledsFlashLoopStart ledsSetRingsAllLedOn 1 ledsSetRingsAllLedOff 1 0.1

	# now the rootfs should be the mirror card
	sizeRootFs=$(helperGetSizeUsedForMountPoint '/')
	# the bootfs should still be the
	sizeBootFs=$(helperGetSizeUsedForMountPoint '/boot')
	# get the dev for the boot device
	deviceToMirrorTo=$(realpath "/dev/disk/by-partuuid/b0075dcd-02" | cut -d/ -f3)
	bootDeviceToMirrorFrom=$(realpath "/dev/disk/by-partuuid/aa1212cd-01" | cut -d/ -f3)

	if [[ $deviceToMirrorTo == "" ]] || [[ $bootDeviceToMirrorFrom == "" ]]; then
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi

	# mount this partition
	devicePartitionMount "${deviceToMirrorTo}"
	# check if the mount has failed
	if [[ $retVal != 0 ]]; then
		rwoLog "ERROR could not mount device to mirror to:${deviceToMirrorTo}"
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi

	# check if the mount has failed
	devicePartitionMount "${bootDeviceToMirrorFrom}"
	if [[ $retVal != 0 ]]; then
		rwoLog "ERROR could not mount boot device to mirror from:${deviceToMirrorTo}"
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi

	# remove the files from the device to mirror to
	#rwoLog "begin remove files ..."
	#rm -Rf "/mnt/${deviceToMirrorTo}/"*
	#rm -Rf "/boot/"*
	#rwoLog "end remove files ..."

	# begin the progress leds
	ledsFlashLoopStop
	ledsSdDoProgressTransferBgStart "/mnt/${deviceToMirrorTo}" "$sizeRootFs" 3 0.1 'all'

	# sync the filesystems first
	sync
	sleep 1

	rwoLog "begin transfer files to root partition ..."
	rsyncRootSuccess=$(rsyncNumTimes "/" "/mnt/${deviceToMirrorTo}/" 3)
	# add the delete flag to remove extraneous files (eg secure boot)
	rwoLog "begin transfer files to boot partition ..."
	rsyncBootSuccess=$(rsyncNumTimes "/mnt/${bootDeviceToMirrorFrom}/" "/boot/" 3)

	sync
	sleep 1

	sizeRootFsMirrorTo=$(helperGetSizeUsedForMountPoint '/')
	devicePartitionUnmount "${deviceToMirrorTo}"
	devicePartitionUnmount "${bootDeviceToMirrorFrom}"
	ledsSdDoProgressTransferBgStop

	# check if not enough bytes have been copied
	if [[ $rsyncRootSuccess != 'success' ]] || [[ $rsyncBootSuccess != 'success' ]]; then
		rwoLog "ERROR - There was an error applying the mirror"
		ledsSdSetLockLed 1
		ledsSdSetUnlockLed 0
		ledsSdShowFinalProgress $sizeRootFsMirrorTo $sizeRootFs
	else
		rwoLog "SUCCESS! The mirror was applied successfully!"
		ledsSdSetLockLed 0
		ledsSdSetUnlockLed 1
		ledsSdShowFinalProgress 100 100
		ledsAllBlueOn
		# remove the boot flag now, you won't need it now ...
		rm "${MIRROR_RUN_BOOT_MIRROR_FILE}"
		# add the flag for mirror done
		touch "${MIRROR_DONE_FILE}"
	fi

	# go ahead and reboot the LightStream now
	sleep 5
	rwo_system.sh systemRebootNow
	exit 1
}

function rsyncNumTimes {
	local rsSrc="$1"
	local rsDst="$2"
	local rsMax="$3"
	local rsNum=0
	local rsExt=1
	while [[ $rsNum -lt $rsMax ]]; do
		rwoLog "begin rsync attempt:${rsNum} src:${rsSrc} dst:${rsDst}"
		rsync --delete-before -axHAX "${rsSrc}" "${rsDst}"
		rsExt=$?
		if [[ $rsExt == 0 ]]; then
			echo "success"
			return
		fi
		rsNum=$(( $rsNum + 1 ))
	done
	echo "fail"
}

function doFunctionUpdateDriveCreate {
	FUNCTION="updateDriveCreate"
	echo "${STATUS_IN_PROGRESS}" > "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
	ledsFlashLoopStart ledsSdSetLedOn 1 ledsSdSetLedOff 1 0.1

	# write the partitons
	dd if=/dev/zero of="/dev/${BASE_DEVICE}" bs=1M count=1 conv=fdatasync
blockdev --rereadpt /dev/${BASE_DEVICE}
cat <<EOT | sfdisk /dev/${BASE_DEVICE}
,,c
EOT

	# check for the first partition
	FIRST_PARTITION="${BASE_DEVICE}1"
	if [[ ! -e "/dev/${FIRST_PARTITION}" ]]; then
		ledsFlashLoopStop
		rwoLog "ERROR : there was a problem with partitioning the update drive"
		_ledsFlashLoopStartSdLockOnError
		echo "${STATUS_ERROR_FAILED}" > "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
		_waitForever
	fi

	# make the filesystems for the partitions
	mkfs.vfat "/dev/${FIRST_PARTITION}"

	devicePartitionMountExitAndFlashIfFail "${FIRST_PARTITION}"
	_doFunctionUpdateDriveCreateFilesIfNeededWithDevicePartition "${FIRST_PARTITION}"
	local identifierFile="/mnt/${FIRST_PARTITION}/DO_NOT_DELETE.txt"
	ledsFlashLoopStop

	if [[ ! -e "${identifierFile}" ]]; then
		rwoLog "ERROR : there was a problem with the update drive creation"
		echo "${STATUS_ERROR_FAILED}" > "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
		_ledsFlashLoopStartSdLockOnError
	else
		devicePartitionUnmount "${FIRST_PARTITION}"
		ledsSdSetUnlockLed 1
		ledsSdShowFinalProgress 100 100
		echo "${STATUS_SUCCESS}" > "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
	fi
	_waitForever
}

function _ledsFlashLoopStartSdLockOnError {
	ledsFlashLoopStart "ledsSdSetLockLed" "1" "ledsSdSetLockLed" "0" "0.25"
}

function _doFunctionUpdateDriveCreateFilesIfNeededWithDevicePartition {
	local devicePartition="$1"
	local identifierFile="/mnt/${devicePartition}/DO_NOT_DELETE.txt"
	echo "SHIBBOLETH" > "${identifierFile}"
	echo " " >> "${identifierFile}"
	mkdir "/mnt/${devicePartition}/media"
	mkdir "/mnt/${devicePartition}/firmware"
}


# Copy Media To Download Device
function doFunctionDownloadWithDevicePartition {
	# pass it the device partiton as the first argument
	# make sure the device partition has already been mounted
	local devicePartition=$1
	FUNCTION="download"
	local mountPoint="/mnt/${devicePartition}"
	local uploadPoint="${mountPoint}"
	if [[ "${devicePartition}" == "${MTP_DEVICE}" ]]; then
		local uploadPoint="${mountPoint}/Phone/Download"
	fi

	# get all of the media from the database
	local query="SELECT SUM(file_size_bytes) FROM media2 WHERE usb_status > 0 AND mime_part1 != 'directory' AND is_deleted = 0"
	local totalBytesAttempted=$(helperSqlite3 "${MEDIA_SQLITE3}" "${query}")
	if [[ $totalBytesAttempted == '' ]]; then
		local totalBytesAttempted=0
	fi
	local totalBytesCopied=0
	# Copy all the usb enabled files over to the download device
	local query="SELECT file_id, parent_path_rel, file_name, file_extension, file_size_bytes FROM media2"
	local query="${query} WHERE usb_status > 0 AND mime_part1 != 'directory' AND is_deleted = 0 ORDER BY parent_path_rel, sort_order IS NULL, sort_order, title ASC"
	local count=0

	if [[ "${HWPLATFORM}" == 'pocket-v1' ]]; then
		# always start by unlocking the card because you want to add media
		if [[ "${BASE_DEVICE}" =~ ^mmcblk ]]; then
			sleep 0.5
			sdtool "/dev/${BASE_DEVICE}" unlock
		fi
		# run the blinking leds and update progress in the background
		ledsSdDoProgressTransferBgStart "${mountPoint}" $totalBytesAttempted 3 0.1
	fi

	# perform the sql query with -line option does a col per line, and a blank line a the end
	local files=$(helperSqlite3 -line "${MEDIA_SQLITE3}" "${query}")
	while read line; do
		# if it's a zero length string, then reset the count
		if [[ -z "${line}" ]]; then
			local count=0
			continue
		fi
		# increment the count
		local count=$(( $count + 1 ))
		# the lines are of the format "      some_colname = some_value"
		# this ensures that the values should be able to contain the string ' = ' provided that the colnames do not contain an = sign
		local value=$(echo "${line}" | cut -d'=' -f2- | cut -d' ' -f2-)
		if [[ count -eq 1 ]]; then
			local fileId="${value}"
		elif [[ count -eq 2 ]]; then
			local parentPathRel="${value}"
		elif [[ count -eq 3 ]]; then
			local fileName="${value}"
		elif [[ count -eq 4 ]]; then
			if [[ -n ${value} ]]; then
				# do the extension and add the dot
				local fileExtension=".${value}"
			else
				local fileExtension=""
			fi
		elif [[ count -eq 5 ]]; then
			local fileSizeBytes=$(( ${value} ))
			# reset the count, and transfer the file
			local filePathRel="${parentPathRel}${fileName}${fileExtension}"
			local filePathAbs="${MEDIA_DIR}/${filePathRel}"
			# make the name and path windows safe for the destination
			local safeFilePathRel="${filePathRel//[\\:*?\"\<\>\|]/�}"
			local fileDestPathAbs="${uploadPoint}/${safeFilePathRel}"
			# check for same file on download device
			if [ -f  "${fileDestPathAbs}" ]; then
				#we do not copy this one, so subtract it from the total
				local totalBytesAttempted=$(( $totalBytesAttempted - $fileSizeBytes ))
				rwoLog "The file \"${filePathRel}\" already exists on ${devicePartition}."
				continue
			fi
			# check for free disk space on download device
			local freeDiskSpaceBytes=$(helperGetSizeFreeForMountPoint "${mountPoint}")
			if [ $fileSizeBytes -ge $freeDiskSpaceBytes ]; then
				rwoLog "Skipping '${filePathRel}'. Not enough space on ${devicePartition}."
				continue
			fi
			# Copy each file, one at a time, from LST to download device.
			rwoLog "Copying ${filePathRel} to ${devicePartition} ..."
			local fileDestDir=$(dirname "${fileDestPathAbs}")
			local secondsStart=${SECONDS}
			mkdir -p "${fileDestDir}"
			cp "${filePathAbs}" "${fileDestPathAbs}"
			local cpExitCode=$?
			local push_durration=$((${SECONDS}-${secondsStart}))
			local status="success"
			if [[ $cpExitCode != 0 ]]; then
				local status="fail"
				rwoLog "Encountered error trying to copy ${filePathRel} to ${devicePartition}. Device may have been uplugged or has bad flash."
				continue
			else
				local totalBytesCopied=$(( $totalBytesCopied + $fileSizeBytes ))
				local query="INSERT INTO sd_interaction_stats (session_id,session_start_datetime,file_path,date_time,push_durration,file_size_bytes,download_device,file_id,status)"
				local query="${query} VALUES('${SESSION_ID}','${SESSION_START}','${filePathRel}',STRFTIME('%Y-%m-%d %H:%M:%f', 'NOW'),"
				local query="${query}'${transferDurration}','${fileSizeBytes}','${devicePartition}','${fileId}','${status}')"
				helperSqlite3 "${LSSD_SQLITE3}" "${query}"
			fi

		else
			rwoLog "ERROR! Unknown count: ${count}"
			local count=0
		fi
	done <<< "${files}"
	rwoLog "copied total bytes to device:\"${devicePartition}\" bytes copied:\"${totalBytesCopied}\" bytes attempted:${totalBytesAttempted}"

	sync "${mountPoint}"
	if [[ $HWPLATFORM == 'pocket-v1' ]]; then
		ledsSdDoProgressTransferBgStop
	fi
	devicePartitionUnmount "${devicePartition}"
	ledsSdShowFinalProgress ${totalBytesCopied} ${totalBytesAttempted}
	_waitForeverAndToggleLock
}

function doFunctionWriteImage {
	#set a flag saying you are still writing the image
	STATUS=''
	FUNCTION="writeImage"
	rwoLog "detected lightstream write image at:${WRITE_IMAGE_FILE}"

	# unmount the device
	devicePartitionUnmount "${DEVICE_PARTITON}"

	ledsSdSetLedsOn 0
	ledsSdSetSixOclockLed 0
	ledsSdSetLedsOn 2
	ledsSdSetLockLed 0

	rwoLog "write image at:${WRITE_IMAGE_FILE} to base device:${BASE_DEVICE}"
	dd if="${WRITE_IMAGE_FILE}" of="/dev/${BASE_DEVICE}" bs=10M conv=fdatasync
	sync
	ledsSdSetLedsOn 5

	# do a byte-for-byte comparison of the image write
	rwoLog "compare image at:${WRITE_IMAGE_FILE} to base device:${BASE_DEVICE}"
	cmpOut=$( cmp "${WRITE_IMAGE_FILE}" "/dev/${BASE_DEVICE}" 2>&1 )
	cmpExitCode=$?
	ledsSdSetLedsOn 7

		# indicate success or fail
	if [[ $( echo "$cmpOut" | grep "EOF" | wc -c ) -gt 0 ]]; then
		ledsSdSetSixOclockLed 1
	else
		ledsSdSetLockLed 1
	fi

	#set a flag saying you are done writing the image
	STATUS="$STATUS_SUCCESS"
	_waitForever
}

# Upload media from upload drive with devicePartition and optional arclightJsonPath (path to arclight.json data file)
function doFunctionUploadWithDevicePartition {
	# set the function
	FUNCTION="upload"
	local devicePartition="$1"
	local arclightJsonPath="$2"
	local mountPoint="/mnt/${devicePartition}"
	local reserveSizeBytes=536870912 # 512MiB
	local firmwareDir="${mountPoint}/firmware"
	local scriptFile="${mountPoint}/_script.sh"
	local mediaDestDir="/srv/http/htdocs/media/master"

	# remount the device partition with utf8 iocharset
	devicePartitionUnmount "${devicePartition}"
	devicePartitionMount "${devicePartition}" "utf8"

	_doFunctionUpdateDriveCreateFilesIfNeededWithDevicePartition "${devicePartition}"

	# SCRIPT FILE
	# a failsafe in case the firmware update gets broken...
	if [[ -f "${scriptFile}" ]]; then
		/bin/bash "${scriptFile}"
	fi

	# FIRMWARE
	local firmwareTarballPath=$(find "${firmwareDir}/" -regex .*\.tar\.gz$ | head -n 1)
	if [ "$(ls "${firmwareDir}/"*.tar.gz | wc -l)" -gt 1 ]; then
		rwoLog "More than one potential firmware update file found. Please make sure only one is present."
	elif [[ $(echo "${firmwareTarballPath}" | wc -w) -gt 0 ]]; then
		rwoLog "Found potential firmware update file. ... will attempt to run it"
		rwo_updateFirmware.sh runUpdateFileWithPath "${firmwareTarballPath}"
	fi

	ledsFlashLoopStart ledsSdSetLedOn 1 ledsSdSetLedOff 1 0.1

	# IMAGES
	imagesDir="${mountPoint}/images"
	tmpfsImagesDir="/tmp/rwo/images"
	mkdir -p "${tmpfsImagesDir}"
	if [[ -d "${imagesDir}" ]]; then
		cp -R "${imagesDir}"/* "${tmpfsImagesDir}"
		cd "${tmpfsImagesDir}"
		cdSuccess=$?
		if [[ $cdSuccess == 0 ]]; then
			# find for proper images
			images=$(find * -name '*.jpg' -o -name '*.png' -o -name '*.jpeg' | sort)
			while read imagePath; do
				php /srv/http/lib/Renew/Utilities/CmdLineAddImageWithPathToDb.php "$imagePath"
			done <<< "${images}"
		fi
	fi

	# MEDIA
	# Change working directory to 'media' directory."
	usbMediaDir="${mountPoint}/media"
	mkdir -p "${usbMediaDir}"
	cd "${usbMediaDir}"
	cdSuccess=$?
	if [[ $cdSuccess != 0 ]]; then
		rwoLog "ERROR cd failed to change into directory:${cdSuccess} ..."
		_ledsFlashLoopStartSdLockOnError
		_waitForever
	fi

	local mediaSize=$(helperGetDirectorySize "${usbMediaDir}")
	local freeDiskSpaceBytes=$(helperGetSizeFreeForMountPoint '/')
	local freeDiskSpaceBytes=$(( $freeDiskSpaceBytes - $reserveSizeBytes ))
	local completedSizeBytes=0
	local attemptedSizeBytes=0
	local fileUploadsFailed=0
	local fileUploadsSuccess=0


	ledsFlashLoopStop
	ledsSdDoProgressTransferBgStart "/" $mediaSize 4 0.1

	files=$(find * | sort | grep -v '/\.')

	while read fileToUploadPath; do

		# catch for empty string
		if [[ $fileToUploadPath == '' ]]; then
			continue
		fi

		local sizeOfFileBytes=$(stat --printf="%s" "${fileToUploadPath}")
		local hypotheticalCompleteSizeBytes=$(( $completedSizeBytes + $sizeOfFileBytes ))
		local fileToUploadPathInMediaDestDir="${mediaDestDir}/${fileToUploadPath}"
		
		# if the file already exists, consider it done
		if [[ -e "${fileToUploadPathInMediaDestDir}" ]]; then
			rwoLog "File arleady exists:${fileToUploadPath} ... skipping"
			local fileUploadsSuccess=$(( $fileUploadsSuccess + 1 ))
			local completedSizeBytes=$hypotheticalCompleteSizeBytes
			continue
		fi

		# once you are sure the file is not already there, now we will consider it an "attempt"
		local attemptedSizeBytes=$(( $attemptedSizeBytes + $sizeOfFileBytes ))
		#check for enough space
		if [[ $hypotheticalCompleteSizeBytes -ge $freeDiskSpaceBytes ]]; then
			rwoLog "not enough space for file:${fileToUploadPath}"
			local fileUploadsFailed=$(( $fileUploadsFailed + 1 ))
			continue
		fi

		rwoLog "upload file:${fileToUploadPath}"
		# create the directory or copy the file
		if [[ -d "${fileToUploadPath}" ]]; then
			mkdir -p "${fileToUploadPathInMediaDestDir}"
			success=$?
		else
			local newFileDirname=$(dirname "${fileToUploadPathInMediaDestDir}")
			[ -d "${newFileDirname}" ] || mkdir -p "${newFileDirname}"
			cp "${fileToUploadPath}" "${fileToUploadPathInMediaDestDir}"
			success=$?
		fi

		# check for add success
		if [ $success -ne 0 ]; then
			rwoLog "Failed to add file to LightStream. Removing '${fileToUploadPath}' from LightStream..."
			rm -Rf "${fileToUploadPathInMediaDestDir}"
			local fileUploadsFailed=$(( $fileUploadsFailed + 1 ))
		else
			rwoLog "Success!"
			sync "${fileToUploadPathInMediaDestDir}"
			if [[ "$fileToUploadPathInMediaDestDir" =~ ^.*\.(3gp|3GP)$ ]]; then
				php /srv/http/lib/Renew/Utilities/CmdLineAddMediaWithPathToDb.php -u "$fileToUploadPathInMediaDestDir" "$arclightJsonPath" "/${fileToUploadPath}"
			else
				php /srv/http/lib/Renew/Utilities/CmdLineAddMediaWithPathToDb.php -wu "$fileToUploadPathInMediaDestDir" "$arclightJsonPath" "/${fileToUploadPath}"
			fi
			local fileUploadsSuccess=$(( $fileUploadsSuccess + 1 ))
			local completedSizeBytes=$hypotheticalCompleteSizeBytes
		fi
	done <<< "${files}"

	rwoLog "Setting owner and group to http:http for all media files and directories..."
	chown -R http:http "${mediaDestDir}"
	rwoLog "Setting file mode to 777 for all media files and directories..."
	chmod -R 777 "${mediaDestDir}"

	# clear the smarty cache so the new files show
	php /srv/http/lib/Renew/Utilities/CmdLineClearSmartyCache.php

	# LOGS : copy log files to the device
	_compressLogsToFile "${mountPoint}/logs.tar.gz"

	sync

	# FINISHING UP ...
	devicePartitionUnmount "$devicePartition"
	# stop the LEDs
	ledsSdDoProgressTransferBgStop
	# now do the final progress
	ledsSdShowFinalProgress $completedSizeBytes $attemptedSizeBytes
	_waitForever
}

function _compressLogsToFile {
	local logsFile="$1"
	rwoLog "copy logs to file:$1"
	cd /var/log
	[[ -e "${logsFile}" ]] && rm "${logsFile}"
	tar -czf "${logsFile}" .
	cd /
}


function doFunctionTestHardwareWithDevicePartition {
	# get setup and ready
	FUNCTION='testHardware'
	local devicePartition="$1"
	systemctl stop rwo_wifi@wlan0Ap
	ip link set dev wlan1 up

	# flash all leds
	ledsFlashLoopStart ledsAllOn 1 ledsAllOff 1 0.2
	# make a file for the test of this pocket
	local serialNumber=$(rwo_system.sh getSerialNumber)
	local date=$(date +'%Y%m%d%H%M%S')
	local hwPlatform=$(rwo_system.sh getHwPlatform)
	local wlan0Mac=$(ip link show wlan0 | grep ether | awk '{print $2}')
	local wlan1Mac=$(ip link show wlan1 | grep ether | awk '{print $2}')
	local batVoltage=$(rwo_battery.sh getBatVoltage)
	local batChargeCurrent=$(rwo_battery.sh getBatChargeCurrent)
	local batDischargeCurrent=$(rwo_battery.sh getBatDischargeCurrent)
	local rootFsBCapacity=$(df -B1 | grep "^/dev/root" | awk '{print $2}')
	local rootFsBUsed=$(df -B1 | grep "^/dev/root" | awk '{print $3}')
	local bootFsBCapacity=$(df -B1 | grep "/boot$" | awk '{print $2}')
	local bootFsBUsed=$(df -B1 | grep "/boot$" | awk '{print $3}')4
	local lightStreamVersion=$(rwo_system.sh getVersion)
	local networksPrettyStr=''
	local bootSdCardDev=$(readlink -f /dev/disk/by-partuuid/b0075dcd-02 | cut -d'p' -f1)
	local bootSdCardCsdRegister=$(sdtool ${bootSdCardDev} status | grep "CSD" | awk '{print $4}' | tr -d '.')


	local testHwDir="/mnt/${devicePartition}/_testHardware"
	local testFilePath="${testHwDir}/testHardware.csv"
	local testHwLogsDir="${testHwDir}/logs"

	if [[ ! -f "${testFilePath}" ]]; then
		touch "${testFilePath}"
		echo "\"Serial Num\",\"Date\",\"Hw Platform\",\"wlan0 Mac\",\"wlan1 Mac\",\"Bat. Voltage (mV)\",\"Bat Charge Current (mV)\",\
\"Bat Discharge Current (mA)\",\"rootfs capacity (B)\",\"rootfs used (B)\",\"bootfs capacity (B)\",\"bootfs used (B)\",\
\"LightStream Version\",\"Wifi Networks Nearby\",\"Boot SD Card Dev\",\"Boot SD Card CSD Register\"" >> "${testFilePath}"
	fi

	if [[ ! -d "${testHwLogsDir}" ]]; then
		mkdir -p "${testHwLogsDir}"
	fi

	# check to see if you can find some nearby wifi networks
	local ssidQualityStr=$(rwo_wifi.sh wlan1Scan)
	local count=0
	local quality=''
	while read line; do
		count=$(( $count + 1 ))
		if [[ "$count" == "1" ]]; then
			ssid="$line"
		elif [[ "$count" == "2" ]]; then
			networksPrettyStr="${networksPrettyStr} || ${ssid} @ ${line}"
			count=0
		fi
	done <<< "${ssidQualityStr}"

	# write it all to the csv file
	echo "\"${serialNumber}\",\"${date}\",\"${HWPLATFORM}\",\"${wlan0Mac}\",\"${wlan1Mac}\",\"${batVoltage}\",\"${batChargeCurrent}\",\
\"${batDischargeCurrent}\",\"${rootFsBCapacity}\",\"${rootFsBUsed}\",\"${bootFsBCapacity}\",\"${bootFsBUsed}\",\
\"${lightStreamVersion}\",\"${networksPrettyStr}\",\"${bootSdCardDev}\",\"${bootSdCardCsdRegister}\"" >> "${testFilePath}"

	#rwoLog "PRINTOUT SYSTEM INFORMATION ..."
	#systemctl status
	#df -hH
	#free -h
	#lsusb
	#cat /proc/cpuinfo
	#cat /proc/meminfo

	 #_compressLogsToFile "${testHwLogsDir}/${serialNumber}.logs.tar.gz"
	 #cp "/var/log/lightstream/lightstream.log" "${testHwLogsDir}/${serialNumber}.lightstream.log"

	# now wait for the secondary button interrupt
	ledsFlashLoopStop
	ledsAllOn
	lockButtonWaitForLow
	STATUS="${STATUS_SUCCESS}"
	ledsAllOff
	ledsBatterySetSixOclockLed 1
	devicePartitionUnmount "${BASE_DEVICE_PARTITION}1"
	_waitForever
}

function _ledsDoDefaultOnExit {
	ledsSdSetSixOclockLed 0
	ledsSdSetLockLed 0
	ledsSdSetUnlockLed 0
	ledsSdSetLedsOn 0
}

function ledsDoMirrorRunConfirm {
	ledsSdSetLedsOn 1
	ledsWifiSetLedsOn 1
	ledsBatterySetLedsOn 1
}

# function gets called ehwn this process exits
function _onExit {
	ledsSdDoProgressTransferBgStop
	ledsFlashLoopStop
	sleep 0.1

	if [[ $FUNCTION == 'mirrorCreate' ]]; then
		_ledsDoDefaultOnExit
		rm "${MIRROR_CREATE_PERCENTAGE_FILE}"
		rm "${MIRROR_CREATE_STATUS_FILE}"
	elif [[ $FUNCTION == 'mirrorRunConfirm' ]]; then
		if [[ $STATUS == "${STATUS_CONFIRMED}" ]]; then
			_ledsDoMirrorRunConfirm
		else
			# the mirror was not confirmed
			# all blues were flashing now turn them all off
			ledsAllBlueOff
		fi
	elif [[ $FUNCTION == 'writeImage' ]]; then
		_ledsDoDefaultOnExit
		if [[ $STATUS != "${STATUS_SUCCESS}" ]]; then
			# turn on the light to indicate error
			ledsSdSetLockLed 1
		else
			ledsSdSetLockLed 0
			ledsSdSetUnlockLed 0
		fi
	elif [[ $FUNCTION == 'updateDriveCreate' ]]; then
		_ledsDoDefaultOnExit
		ledsSdSetUnlockLed 0
		rm "${UPDATE_DRIVE_CREATE_STATUS_FILE}"
	elif [[ $FUNCTION == 'testHardware' ]]; then
		ledsAllOff
		# indicate still on
		ledsBatterySetSixOclockLed 1
		_ledsDoDefaultOnExit
	else
		_ledsDoDefaultOnExit
	fi

	devicePartitionUnmount "${BASE_DEVICE_PARTITION}1"
	devicePartitionUnmount "${BASE_DEVICE_PARTITION}2"
	WAIT_FOREVER_LOOP=0
}



############### END FUNCTIONS ###############
############### --- MAIN --- ###############
# set newline as IFS
IFS=$'\n'
# this should be the base device (not a partition ... eg sda, sdb, etc)
BASE_DEVICE="$1"
# the action (eg add or remove) (optional)
ACTION="$2"

# function gets set based on what function is performed
# satus gets set sometimes as a secondary check
FUNCTION=""

STATUS=""
STATUS_SUCCESS='STATUS_SUCCESS'
STATUS_ERROR_FAILED='STATUS_ERROR_FAILED'
STATUS_ERROR_TOO_SMALL='STATUS_ERROR_TOO_SMALL'
STATUS_IN_PROGRESS='STATUS_IN_PROGRESS'
STATUS_ERROR_TIMEOUT='STATUS_ERROR_TIMEOUT'
STATUS_CONFIRMED='STATUS_CONFIRMED'

# get the size of the device you are inserting in bytes
BASE_DEVICE_SIZE=$(helperGetSizeInBytesForDevice "${BASE_DEVICE}")
ROOTFS_DEVICE_PARTITION=$(helperGetRootFsDevice)
# mirror creation related things
MIRROR_CREATE_FLAG_FILE="/tmp/rwo/rwo_deviceLauncher.mirrorCreate"
MIRROR_RUN_FLAG_FILE="/tmp/rwo/rwo_deviceLauncher.mirrorRun"
MIRROR_CREATE_PERCENTAGE_FILE="/tmp/rwo/rwo_deviceLauncher.mirrorCreate.percentage"
MIRROR_CREATE_STATUS_FILE="/tmp/rwo/rwo_deviceLauncher.mirrorCreate.status"

MIRROR_RUN_BOOT_MIRROR_FILE='/boot/mirrorBoot.opt'
MIRROR_DONE_FILE='/boot/mirrorDone.opt'

UPDATE_DRIVE_CREATE_FLAG_FILE="/tmp/rwo/rwo_deviceLauncher.updateDriveCreate"
UPDATE_DRIVE_CREATE_STATUS_FILE="/tmp/rwo/rwo_deviceLauncher.updateDriveCreate.status"

# number of seconds to wait inbetween flags
FLAG_TIMEOUT=10

# download things
LSSD_SQLITE3="/etc/lightstream/data/lssd.sqlite3"
SESSION_START=$(date +'%Y-%m-%d %H:%M:%S.%N' | cut -c 1-23)
SESSION_ID=$(date +'%Y%m%d%H%M%S%N')-$RANDOM
MEDIA_SQLITE3="/etc/lightstream/data/media.sqlite3"
MEDIA_DIR="/srv/http/htdocs/media/master"

HWPLATFORM=$(rwo_system.sh getHwPlatform)
TOTAL_BYTES_COPIED=0
TOTAL_BYTES_ATTEMPTED=0
WRITE_IMAGE_FILE="${MEDIA_DIR}/LightStreamWriteImage.img"

WAIT_FOREVER_LOOP=1


MTP_DEVICE='libmtp'

# systemd sends a SIGTERM before exiting
# ... trap the signals and call _onExit when the signal is given
# ... the signal should be sent when the device is removed
trap '_onExit' EXIT INT HUP TERM

# log this ...
rwoLog "device:${BASE_DEVICE} action:${ACTION}"

# handle mtp devices
if [[ "${BASE_DEVICE}" == "${MTP_DEVICE}" ]]; then
	DEVICE_PARTITON="${BASE_DEVICE}"
	devicePartitionMountExitAndFlashIfFail "${DEVICE_PARTITON}" "utf8"
	doFunctionDownloadWithDevicePartition "${DEVICE_PARTITON}"
	# handle a scsi disk (usb), or handle mmc device
	# examples: mmcblk1 , and and sda / sdb
elif ([[ "${BASE_DEVICE}" =~ ^sd[a-z]$ ]] || [[ "${BASE_DEVICE}" =~ ^mmcblk[0-9]$ ]]); then
	# now dealing with the first partition ...
	# identify the first partition
	BASE_DEVICE_PARTITION="${BASE_DEVICE}"
	[[ "${BASE_DEVICE}" =~ ^mmcblk[0-9]$ ]] && BASE_DEVICE_PARTITION="${BASE_DEVICE}p"
	DEVICE_PARTITON="${BASE_DEVICE_PARTITION}1"
	TESTHW_DIR="/mnt/${DEVICE_PARTITON}/_testHardware"

	# Dealing with the base device ...
	checkIfBootDeviceExitIfYes
	checkIfShouldDoMirrorCreateDoIfYes
	checkIfShouldWriteImageDoFunctionWriteImageIfYes
	checkIfShouldRunMirrorAndRunIfYes
	checkIfShouldCreateUpdateDrive


	if [[ ! -e "/dev/${DEVICE_PARTITON}" ]]; then
		# if the first partition does not exist, try mounting base device
		DEVICE_PARTITON="${BASE_DEVICE}"
	fi

	# try to mount the partition, but if you fail, exit
	devicePartitionMountExitAndFlashIfFail "${DEVICE_PARTITON}" "utf8"
	# check if it is an upload device by the special file, or if it has the serial number
	checkForUploadDeviceWithDevicePartition "${DEVICE_PARTITON}"
	doFunctionDownloadWithDevicePartition "${DEVICE_PARTITON}"

# bluetooth dongles ...
elif [[ "${BASE_DEVICE}" =~ ^hci[0-9]$ ]]; then
	if [[ "${ACTION}" == "add" ]]; then
		rwoLog "hci device added:${BASE_DEVICE}"
		ledsFlashWifiSixOclockLed 3
		systemctl --no-block start rwo_bluetooth
	elif [[ "${ACTION}" == "remove" ]]; then
		rwoLog "hci device removed:${BASE_DEVICE}"
		ledsFlashWifiSixOclockLed 3
		systemctl --no-block stop rwo_bluetooth
	fi
# this kind of device is formatted similar to hci0:10
# TODO: I think this means a device that it is trying to pari with, or send a file to
elif [[ "${BASE_DEVICE}" =~ ^hci[0-9]:[0-9]+$ ]]; then
	if [[ "${ACTION}" == "add" ]]; then
		ledsFlashWifiSixOclockLed 2
	fi
fi
