#!/bin/bash

# this script should run on each boot at the multi-user.target
# it is launched by the systemd service /etc/systemd/system/rwo_onBoot.service
# this script should not hang for a long time, but please try to let it exit within 10 seconds or so...
# use this script to perform startup tasks, initialize hardware and software.. etc

# if you need to start a longer running process,
# I recommend you make a systemd service and you can either enable it,
# or start it from this script

# if you want something to only run once at bootup,
# eg, at the first boot of the device only
# I recommend you write a file to /var/log/lightstream/ to indicate that the process has been already run, and check for that file to see

# includes
. /usr/bin/rwo_includeLogging.sh
. /usr/bin/rwo_includeHelpers.sh
. /usr/bin/rwo_includeLeds.sh

sqlite3DbFileLsWifi="/etc/lightstream/data/lswifi.sqlite3"

function removeTemporaryFiles {
	# clear out all of the temporary directories
	rm -Rf /srv/http/downloads/*
	rm -Rf /srv/http/reports/*
	rm -Rf /srv/http/htdocs/media/downloads/*
	rm -Rf /srv/http/htdocs/uploads/*
	rm -Rf /srv/http/htdocs/uploads/tmp/*
	rm -Rf /srv/http/htdocs/images/user_images/tmp/
	rm -Rf /srv/http/htdocs/admin/utilities/fine_uploader/chunks/*
	rm -Rf /srv/http/htdocs/admin/utilities/fine_uploader/files/*
	rm -Rf /srv/http/templates_c/*
	rm -f /srv/http/data/cdo.json
	# If upload_tmp_dir is NOT commented out in php.ini, then clear it
	if [ $(grep 'upload_tmp_dir' /etc/php/php.ini | tr -d ' ' | cut -b 1) != ";" ]; then
		# Get path to temporary directory from php.ini file (sans whitespace).
		phpUploadTmpDir=$(grep 'upload_tmp_dir' /etc/php/php.ini | tr -d ' ' | cut -d = -f 2)
		rm -Rf "${phpUploadTmpDir}/"*
		mkdir -p "${phpUploadTmpDir}"
		chmod 777 "${phpUploadTmpDir}"
		chown http:http "${phpUploadTmpDir}"
	fi
}

################## MAIN #######################
systemMode=$(helperSqlite3 "${sqlite3DbFileLsWifi}" "SELECT value FROM settings WHERE key = 'system_mode'")
rootFsDevice=$(helperGetRootFsDevice)
partuuidRootFs=$(helperGetPartUuidForDevice "$rootFsDevice")
internalSlotDevice=$(helperGetInternalSdSlotDevice)

# check for which hardware you are running on
hwPlatform=$(/usr/bin/rwo_system.sh getHwPlatform)
rwoLog "boot script started, platform detected:${hwPlatform}"

rwoLog "will set lo interface ip address to ${loIpAddr}"
ifconfig lo:0 "192.168.211.11"


rwoLog "configure axp209 ... "
rwo_system.sh axp209Configure

# create system tmp dir
helperCreateSystemTmpDir

# check if the internal card is permanently locked
if [[ $(sdtool /dev/${internalSlotDevice} status | grep "state: Permanent" | wc -l) -eq 1 ]]; then
	ledsSetRingsAllLedsOn 7
	ledsSdSetLockLed 1
fi

# check to see if you just booted from a mirror drive by checking for the presence of the boot partition
if [[ "$partuuidRootFs" == "aa1212cd-02" ]] && [[ ! -e '/dev/disk/by-partuuid/b0075dcd-01' ]]; then
	rwoLog "detected a mirror in the internal sd slot ... "
	# delete the mirror boot file in case
	mount '/dev/disk/by-partuuid/aa1212cd-01' '/boot'
	rm '/boot/mirrorBoot.opt'
	# change the partuuid back to b0075dcd
	echo -e "\xcd\x5d\x07\xb0" | dd bs=1 count=4 of="/dev/${internalSlotDevice}" seek=440 conv=fdatasync
	sync
	rwo_system.sh systemRebootNow
elif [[ "$partuuidRootFs" == "aa1212cd-02" ]]; then
	rwoLog "detected mirror apply ... "
	# don't startup wifi, and don't startup button listener
	ledsSetRingsAllLedsOn 1
	# start up the systemd service
	udevadm control -S
else
	rwoLog "booting into normal mode"
	# disable ntp so that it will not try to get internet time
	# set the local rtc to 0, and timezone to UTC
	# this is for setting the timestamp in the php script
	timedatectl set-ntp 0
	timedatectl set-local-rtc 0
	timedatectl set-timezone UTC

	rwoLog "will start systemd timers"
	systemctl start rwo_on15MinClock.timer
	systemctl start rwo_onMidnightClock.timer

	rwoLog "will start eth0 state listener"
	systemctl restart rwo_eth0StateListener-d
	# check hd space at each boot to see if it needs expanding
	# note, this may also restart the system ...
	systemctl start rwo_checkHdSpace
	removeTemporaryFiles
	systemctl start rwo_powerButtonListener-d

	# Clock Things!
	# get the last wifi interaction id so you can know to set the clock
	maxWifiInteractionAtBoot=$(helperSqlite3 "${sqlite3DbFileLsWifi}" "SELECT MAX(interaction_id) FROM wifi_interaction_stats")
	if [[ "$maxWifiInteractionAtBoot" == '' ]]; then
		maxWifiInteractionAtBoot=0
	fi
	helperSqlite3 "${sqlite3DbFileLsWifi}" "INSERT OR REPLACE INTO system_info (key, value_int) VALUES('wifi_interaction_id_at_boot', ${maxWifiInteractionAtBoot})"
	# clear out the ip_addrs table, this is where client_date_offsets are also stored
	helperSqlite3 "${sqlite3DbFileLsWifi}" "DELETE FROM ip_addrs"

	# check if you should scan for media, always scan unless its false
	shouldScanMedia=$(helperSqlite3 "${sqlite3DbFileLsWifi}" "SELECT value FROM settings WHERE key = 'should_scan_media'")
	if [[ "${shouldScanMedia}" != 'false' ]]; then
		php -f "/srv/http/lib/Renew/Utilities/MediaDbScanUpdate.php"
	fi

	# startup essential services
	systemctl start nginx
	systemctl start php-fpm
	systemctl start php7.0-fpm
	systemctl start incrond
	systemctl start rwo_wifi@wlan0Ap
	# stop the blinking wifi led that was started in rwo_onSysinitTarget
	systemctl stop rwo_ledBlink@wifiSixOclock
	systemctl start rwo_bluetooth
	# check to see if the test file exists
	systemctl start rwo_test@currentAndVoltage
fi
