#!/bin/bash
# Include logging to log stderr and stdout to specific file
LOG_SILENT_STDERR=0
LOG_SILENT_STDOUT=0
LOG_SILENT_PRINTOUT=0
thisDir=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
. "${thisDir}/rwo_includeLogging.sh"
. "${thisDir}/rwo_includeHelpers.sh"
. "${thisDir}/rwo_includeLeds.sh"

function systemUpdateRootDirectory {
	ROOT_DIR="$1"
	BOOT_DIR="${ROOT_DIR}/boot"

	MEDIA_MASTER_DIR="${ROOT_DIR}/srv/http/htdocs/media/master"
	USER_IMAGES_DIR="${ROOT_DIR}/srv/http/htdocs/images/user_images"
	DATA_DIR="${ROOT_DIR}/etc/lightstream/data"
	LOGS_DIR="${ROOT_DIR}/var/log/lightstream"

	# we do it this way so that during the update this function can be called
	VARSION_FILE_1="/etc/lightstream/version"
	VERSION_FILE_2="/var/log/lightstream/version.txt"
	VERSION_FILE_3="/var/log/lightstream/version.info"

	MEDIA_DB_FILE="${DATA_DIR}/media.sqlite3"
	WIFI_DB_FILE="${DATA_DIR}/lswifi.sqlite3"
	SD_DB_FILE="${DATA_DIR}/lssd.sqlite3"
	BT_DB_FILE="${DATA_DIR}/lsbt.sqlite3"
	SATELLITE_DB_FILE="${DATA_DIR}/satellite.sqlite3"

	# rename the old files that end in .db to make them end in .sqlite3
	OLD_USB_DB_FILE="${DATA_DIR}/lsusb.db"
	OLD_USB_DB_FILE_2="${DATA_DIR}/lsusb.sqlite3"
	OLD_BT_DB_FILE="${DATA_DIR}/lsbt.db"
	OLD_SATELLITE_DB_FILE="${DATA_DIR}/satellite.db"
}

# setup all the root directory variables
systemUpdateRootDirectory ''

######################### SERIAL NUMBER #######################
function _getSerialNumber {
	snFile1="/proc/device-tree/serial-number"
	if [[ -e "${snFile1}" ]]; then
		serialNumber=$( cat "${snFile1}" | tr '\0' '\n' )
	else
		serialNumber="0000000000000000"
	fi
}

function getSerialNumber {
	_getSerialNumber
	rwoPrintout "${serialNumber}"
}

##################### VERSION ############################
function _getVersion {
	local verFile3='/etc/lightstream/version'
	local verFile2='/var/log/lightstream/version.txt'
	local verFile2='/var/log/lightstream/version.info'
	if [[ -f "${verFile3}" ]]; then
		version=$(cat "${verFile3}")
	elif [[ -f "${verFile2}" ]]; then
		version=$(cat "${verFile2}" | cut -d' ' -f3 | cut -d'-' -f1)
	elif [[ -f "${verFile1}" ]]; then
		version=$(cat "${verFile1}" | cut -d' ' -f3 | cut -d'-' -f1)
	fi
}

function getVersion {
	_getVersion
	rwoPrintout "${version}"
}

# for comparison operators reurn 1 for TRUE and 0 for FALSE
function versionLt {
	versionCompare $1 $2
	[[ $? == 199 ]] && return 0
	return 1
}

function versionLe {
	versionCompare $1 $2
	[[ $? != 201 ]] && return 0
	return 1
}

function versionEq {
	versionCompare $1 $2
	[[ $? == 200 ]] && return 0
	return 1
}

function versionGe {
	versionCompare $1 $2
	[[ $? != 199 ]] && return 0
	return 1
}

function versionGt {
	versionCompare $1 $2
	[[ $? == 201 ]] && return 0
	return 1
}

# compares the first argument to the second argument
# if no second argument is supplied compares the current version to the first argument
function versionCompare {
	if [[ "$#" == 1 ]]; then
		_getVersion
		leftVer="${version}"
		rightVer="$1"
	elif [[ "$#" -ge 2 ]]; then
		leftVer="$1"
		rightVer="$2"
	else
		return 200 # no arguments are supplied, return equal
	fi
	leftCount=$(echo "${leftVer}" | tr -cd "." | wc -c)
	rightCount=$(echo "${rightVer}" | tr -cd "." | wc -c)
	maxCount=$leftCount
	[ $rightCount -gt $leftCount ] && maxCount=$rightCount
	maxFields=$(( $maxCount + 1 ))
	for (( i = 1; i <= $maxFields; i++ )); do
		leftThisPlace=$(echo "${leftVer}" | cut -d'.' -f "$i")
		rightThisPlace=$(echo "${rightVer}" | cut -d'.' -f "$i")
		[ -z "$leftThisPlace" ] && leftThisPlace=0
		[ -z "$rightThisPlace" ] && rightThisPlace=0
		[ $leftThisPlace -gt $rightThisPlace ] && rwoPrintout "greater" && return 201
		[ $leftThisPlace -lt $rightThisPlace ] && rwoPrintout "less" && return 199
	done
	rwoPrintout "equal"
	return 200
}


######################
function _getHwPlatformAndHwVendor {
	# TODO, test on rpi1, rpi2, and rpi3
	# prints out the hardware version, rpi1, rpi2, rpi3, pocket, or unknown
	# in the proc/cpuinfo file get the "Hardware" line
	# split the line by delimiter ': ' and get right side
	hadrdware=$(cat /proc/cpuinfo | grep Hardware | awk -F ': ' '{print $2}')
	hwPlatform='unknown'
	hwVendor='unknown'
	if [[ "${hadrdware}" == 'Allwinner sun4i/sun5i Families' ]]; then
		 hwPlatform='pocket-v1'
		 hwVendor='allwinner'
		 return 0
	elif [[ "${hadrdware}" == 'Allwinner sun8i Family' ]]; then
		 hwPlatform='pocket-v2'
		 hwVendor='allwinner'
		 return 0
	# RPi2 and RPi3 use the same BCM2709 hardware
	elif [[ "${hadrdware}" == 'BCM2709' ]]; then
		#check the revision to determine if rpi2 or rpi3 model
		revision=$(cat /proc/cpuinfo | grep Revision | awk -F ': ' '{print $2}')
		hwVendor='broadcom'
		if [[ "${revision}" =~ a[02]2082 ]]; then
			hwPlatform='rpi3'
			return 0
		else
			hwPlatform='rpi2'
			return 0
		fi
	fi
	# some unknown hardware
	return 1
}

function getHwPlatform {
	_getHwPlatformAndHwVendor
	rwoPrintout "${hwPlatform}"
}

function getHwVendor {
	_getHwPlatformAndHwVendor
	rwoPrintout "${hwVendor}"
}

############# AXP209 ###############################
function axp209Configure {
	# configure the settings on the AXP209
	_axp209SetReg 0x12 0x5F # turn on/off proper outputs
	_axp209SetReg 0x23 0x16 # set dc-dc2 to 1.25V
	_axp209SetReg 0x25 0x00 # configure dc-dc2
	_axp209SetReg 0x27 0x16 # set dc-dc3 to 1.25V
	_axp209SetReg 0x28 0xCF # set LDO2 to 3.0V and LDO4 to 3.3V
	_axp209SetReg 0x29 0x68 # LDO3 configure
	_axp209SetReg 0x30 0x03 # VBUS-IPSOUT configure
	_axp209SetReg 0x31 0x00 # hard shutdown at 2.9V batt voltage
	_axp209SetReg 0x32 0x46
	_axp209SetReg 0x33 0xC9 # enable charging, target 4.2V, target I > 10%, 1800mA
	_axp209SetReg 0x34 0x45 # more charging configuration
	_axp209SetReg 0x35 0x22 # disable backup charger
	_axp209SetReg 0x36 0x3F # PEK settings
	_axp209SetReg 0x37 0x0A
	_axp209SetReg 0x38 0xA5
	_axp209SetReg 0x39 0x1F
	_axp209SetReg 0x3A 0x00
	_axp209SetReg 0x3B 0x00
	_axp209SetReg 0x3C 0xFC
	_axp209SetReg 0x3D 0x16
	_axp209SetReg 0x40 0x0C # IRQ enable 1
	_axp209SetReg 0x41 0x00 # IRQ enable 2
	_axp209SetReg 0x42 0x00 # IRQ enable 3
	_axp209SetReg 0x43 0x00 # IRQ enable 4
	_axp209SetReg 0x44 0x00 # IRQ enable 5

	_axp209SetReg 0x80 0x00
	_axp209SetReg 0x82 0xFF # enable the adcs for current and voltage of battery
	_axp209SetReg 0x83 0x00
	_axp209SetReg 0x84 0x32 # TS pin settings, changing these may result in flashing CHGLED
	_axp209SetReg 0x85 0x00
	_axp209SetReg 0x86 0xFF
	_axp209SetReg 0x87 0x00
	_axp209SetReg 0x8A 0x00
	_axp209SetReg 0x8B 0x08 # vbus settings
	_axp209SetReg 0x8F 0x21
	_axp209SetReg 0x90 0x07
	_axp209SetReg 0x91 0x05
	_axp209SetReg 0x94 0x00
	_axp209SetReg 0x95 0x02

	_axp209SetReg 0xB8 0x00 # Coulomb counter disable
	_axp209SetReg 0xB9 0x00 # Fuel Gague enable normal

}

function _axp209SetReg {
	i2cset -y -f 0 0x34 $@
}


############### SYSTEM ###################
function systemReset {
	local rootDir="$1"
	systemUpdateRootDirectory "${rootDir}"
	rm -Rf "$MEDIA_MASTER_DIR"/*
	rm -Rf "$DATA_DIR"/*
	rm -Rf "$USER_IMAGES_DIR"/*
	# delete all the options from the boot dir
	rm -Rf "$BOOT_DIR"/*.opt
	#rm -Rf "$LOGS_DIR"/*.log
	mkdir "$MEDIA_MASTER_DIR"
	mkdir "$DATA_DIR"
	mkdir "$USER_IMAGES_DIR"
	mkdir "$LOGS_DIR"
	lsDatabasesUpdate "${rootDir}"
	exit 0
}
function systemModeWifiStart {
	systemctl start rwo_wifi@wlan0Ap
}

function systemModeWifiStop {
	systemctl stop rwo_wifi@wlan0Ap
}

function systemModeSecureStart {
	systemctl stop rwo_wifi@wlan0Ap
	systemctl stop nginx
	systemctl stop php7.0-fpm
	systemctl stop rwo_bluetooth
}

function systemModeNormalStart {
	systemModeWifiStart
	systemctl start rwo_bluetooth
}

function systemShutdownNow {
	_systemShutdownNow "shutdown"
}

function systemRebootNow {
	_systemShutdownNow "reboot"
}

function _systemShutdownNow {
	#systemctl start rwo_ledBlink@batterySixOclock
	ledsAllBlueOnOnly
	systemctl stop rwo_wifi@wlan0Ap
	ledsAllBlueOnOnly
	sleep 0.5
	ledsAllBlueOnOnly
	if [[ "$1" == "reboot" ]]; then
		shutdown -r now
	else
		shutdown now
	fi

}

function _getLightStreamDefaultName {
	_getSerialNumber
	rwoLog "Assign a wifi ssid"
	serialLastFour=$( echo "${serialNumber}" | tail -c 5 )
	echo "GoWifi.org-${serialLastFour}"
}

###################### WiFi #################

function setWiFiToNewDefaultsIfNeeded {
	ssid=$(helperSqlite3 "${WIFI_DB_FILE}" "SELECT value FROM settings WHERE key = 'ssid'")
	if [[ "${ssid}" == "" ]] || [[ "${ssid}" =~ ^GoWifi.org-[0-9a-fA-F]{4}$ ]]; then
		defaultWiFiSsid=$(_getLightStreamDefaultName)
		if [[ "${ssid}" != "${defaultWiFiSsid}" ]]; then
			setWifiSsidToDefault
			setWifiChannelToRandom
		fi
	fi
}

function setWifiSsidToDefault {
	lightStreamDefaultName=$(_getLightStreamDefaultName)
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT or REPLACE into settings (key, value) VALUES ('ssid', '${lightStreamDefaultName}')"
}

function setWifiChannelToRandom {
	# channel of 0 is random
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT or REPLACE into settings (key, value) VALUES ('channel', '0')"
}

################## BLUETOOTH ###############
function setBluetoothDeviceNameToDefault {
	lightStreamDefaultName=$(_getLightStreamDefaultName)
	helperSqlite3 "${BT_DB_FILE}" "INSERT or REPLACE into settings (key, value) VALUES ('device_name', '${lightStreamDefaultName}')"
}

################ DATABASES ##################
function lsDatabasesUpdate {
	_lsDatabasesInitialize "$1" "INSERT OR IGNORE"
}

function _lsDatabasesInitialize {
	# if left blank this will put them under the root dir
	# use this for setting up dev environment
	local rootDir="$1"
	systemUpdateRootDirectory "$rootDir"
	local insertStmt="$2"
	local intoSettingsStmt='INTO settings (key, value) VALUES'
	if [[ $insertStmt == '' ]]; then
		insertStmt="INSERT OR IGNORE"
	fi
	rwoLog "running _lsDatabasesInitialize with rootDir:${rootDir} insertStmt:${insertStmt}"

	# LSBT (LightStream Bluetooth) settings
	local maxPushRetry="5"
	local reject_delay="58"
	local engine_log="/var/log/lightstream/lsbt.log"
	local engine_log_level="ERROR"
	local scan_delay="15"
	local device_name="1"
	local scan_for_devices="1"
	# migrate over the lsbt.conf settings into the new settings database if the lsbt.conf file exists
	local lsbtConfFile="/etc/lsbt/lsbt.conf"

	# move /srv/http/data to /etc/lightstream
	local dataDirHttp="${ROOT_DIR}/srv/http/data"
	[[ -d "${DATA_DIR}" ]] || mkdir "${DATA_DIR}"
	if [[ ! -L "${dataDirHttp}" ]]; then
		cp -Rf "${dataDirHttp}/"* "${dataDirEtc}/"
		chmod -R 777 "${dataDirEtc}"
		rm -Rf "${dataDirHttp}"
	fi
	# create the symbolic link to http data dir
	rm -f ${dataDirHttp}
	#ln -s "${DATA_DIR}" "${dataDirHttp}"

	# move the old database files to the new names
	if [ -e "${OLD_USB_DB_FILE}" ]; then
		mv "${OLD_USB_DB_FILE}" "${SD_DB_FILE}"
		helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE sd_interaction_stats RENAME TO sd_interaction_stats"
	fi

	if [ -e "${OLD_USB_DB_FILE_2}" ]; then
		mv "${OLD_USB_DB_FILE_2}" "${SD_DB_FILE}"
		helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE sd_interaction_stats RENAME TO sd_interaction_stats"
	fi

	if [ -e "${OLD_BT_DB_FILE}" ]; then
		mv "${OLD_BT_DB_FILE}" "${BT_DB_FILE}"
	fi

	if [ -e "${OLD_SATELLITE_DB_FILE}" ]; then
		mv "${OLD_SATELLITE_DB_FILE}" "${SATELLITE_DB_FILE}"
	fi

    # Check whether usb download statistics file does not exist.
	if [ ! -f "${MEDIA_DB_FILE}" ]; then
		rwoLog "No MEDIA DB_FILE ... will create"
		touch "${MEDIA_DB_FILE}"
	fi

	if [ ! -f "${WIFI_DB_FILE}" ]; then
		rwoLog "No WIFI DB_FILE ... will create"
		touch "${WIFI_DB_FILE}"
	fi

	if [ ! -f "${SD_DB_FILE}" ]; then
		rwoLog "No SD_DB_FILE ... will create"
		touch "${SD_DB_FILE}"
	fi

	if [ ! -f "${BT_DB_FILE}" ]; then
		rwoLog "No BT_DB_FILE ... will create"
		touch "${BT_DB_FILE}"
	fi

	if [ ! -f "${SATELLITE_DB_FILE}" ]; then
		rwoLog "No SATELLITE_DB_FILE ... will create"
		touch "${SATELLITE_DB_FILE}"
	fi

	# setup the users and the groups
	chown -R http "${DATA_DIR}"
	chgrp -R http "${DATA_DIR}"
	chmod -R 0777 "${DATA_DIR}"

	helperSqlite3 "${MEDIA_DB_FILE}" "CREATE TABLE IF NOT EXISTS media2 (
                            file_id INTEGER PRIMARY KEY AUTOINCREMENT ,
                            parent_path_rel TEXT,                   -- The path of the parent directory relative to the master media directory
                            hash TEXT,                              -- The hash of at least some of the actual content of the file
                            file_name TEXT,                         -- The name of the file, less then extension. What it downloads as, and what is shown if title is null
                            file_extension TEXT,                    -- The extension for the file
                            mime_part1 TEXT,                        -- This is the first part of the mime type, before the slash
                            mime_part2 TEXT,                        -- This is the second part of the mime type, after the slash
                            wifi_status INTEGER,                    -- 0 for not available, 1 for some available in a dir, 2 for available
                            bluetooth_status INTEGER,               -- 0 for not available, 1 for some available in a dir, 2 for available
                            usb_status INTEGER,                     -- 0 for not available, 1 for some available in a dir, 2 for available
                            image_id INTEGER,                       -- This is the id of the image associated with this media item
                            description TEXT,                       -- Limit the number of chars
                            title TEXT,                             -- This is what we display to the user
                            featured INTEGER,                       -- 0 if not featured
                            sort_order INTEGER,                     -- This is the sort order of the file in the directory
                            datetime_added TEXT,                    -- The timestamp at which the file was added to the database
                            datetime_edited TEXT,                   -- The timestamp at which the file was last edited
                            language_iso TEXT,                      -- The ISO 639-3 code of the langauge for this media
                            file_size_bytes INTEGER,                -- The size in bytes of the file
                            file_length_seconds INTEGER,            -- The length of the audio or video file in seconds. Default value is 0
                            is_deleted INTEGER                      -- If set to 1, the file has been deleted or moved. Default value is 0
														);"
	
	# MEDIA2 TABLE ALTERATIONS
	# promoted_media_file_id -- 0 if no language directory promoted media file
	# is_download_disabled --If set to 1, the file is stream only. Default value is 0
	# is_password_protected -- add option to password protect media
	# greeting_title -- for folders that want a custom greeting ... they can use the description for the greeting text
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN promoted_media_file_id INTEGER DEFAULT 0;"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN is_download_disabled INTEGER DEFAULT 0;"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN is_password_protected INTEGER DEFAULT 0;"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN greeting_title TEXT DEFAULT '';"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN downloader_source TEXT DEFAULT '';"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN downloader_json TEXT DEFAULT '';"
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE media2 ADD COLUMN is_global INTEGER DEFAULT 0;"

	_lsDatabasesInitializeSortOrderTriggers "${MEDIA_DB_FILE}" "media2" "parent_path_rel"

	helperSqlite3 "${MEDIA_DB_FILE}" "CREATE TABLE IF NOT EXISTS media_passwords (
                            file_id INTEGER PRIMARY KEY,
                            password TEXT                                  -- Hashed password
														);"

	helperSqlite3 "${MEDIA_DB_FILE}" "CREATE TABLE IF NOT EXISTS images (
                            image_id INTEGER PRIMARY KEY AUTOINCREMENT,    -- This is the id of the image, it is also stored in the fs with this file name
                            image_title TEXT UNIQUE NOT NULL               -- This is the title displayed to the user, should be unique
                            );"

	# add the iso-639-3 database ... data is adapted from https://iso639-3.sil.org/code_tables/download_tables#termsofuse
	helperSqlite3 "${MEDIA_DB_FILE}" ".read '${rootDir}/etc/lightstream/static_data/languages_iso639_3.sqlite3.insert'"
	# add language data from arclight API ... includes JF's codes for languages and some localized language names
	helperSqlite3 "${MEDIA_DB_FILE}" ".read '${rootDir}/etc/lightstream/static_data/languages_arclight.sqlite3.insert'"

	# the file_id for which the image is a thumbnail of
	helperSqlite3 "${MEDIA_DB_FILE}" "ALTER TABLE images ADD COLUMN from_media_file_id INTEGER DEFAULT 0;"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS wifi_interaction_stats (
                            interaction_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            file_id INTEGER,                      -- What is the id of the file for this interaction
                            path_rel TEXT,                        -- What was the parent path of the file at the time
                            file_name TEXT,                       -- What was the file's name at the time
                            file_extension TEXT,                  -- What is the file's extension
                            date_time TEXT,                       -- When did this interaction occur?
                            interaction_type TEXT                 -- This is like stream, download, ... etc
                            );"


	helperSqlite3 "${WIFI_DB_FILE}" "ALTER TABLE wifi_interaction_stats ADD COLUMN device_id TEXT DEFAULT '';"

	# this is for the form interactions ... the default value is NULL
	helperSqlite3 "${WIFI_DB_FILE}" "ALTER TABLE wifi_interaction_stats ADD COLUMN form_id INTEGER DEFAULT NULL;"

	# this is the table to store system stats like time running, total number of boots
	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS system_info (
                            key TEXT PRIMARY KEY,
                            value_txt TEXT,
                            value_int INTEGER
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS settings (
                            key TEXT PRIMARY KEY,
                            value TEXT
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS themes (
                            key TEXT PRIMARY KEY,                 -- the key that identifies this particular setting
                            value TEXT,                           -- the value associated with the
                            type TEXT,                            -- what type of setting is it, often may be used in the CSS
                            effect TEXT                           -- maps to an associative array, Eg blur
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS ip_addrs (
                            ip_addr TEXT PRIMARY KEY,            -- the ip of this device currently
                            device_id TEXT                       -- this is the unique id for the device
                            );"
	

	# the IP addresses table is a safe place to put the client_date_offset, since it can be cleared out at each boot
	# the number of seconds the client time is off from the LightStream's time
	helperSqlite3 "${WIFI_DB_FILE}" "ALTER TABLE ip_addrs ADD COLUMN client_date_offset INTEGER DEFAULT 0;"
	
	# Add in default internal device ip address
	local internalDeviceId="device-id-internal";
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT OR REPLACE INTO ip_addrs (ip_addr, device_id) VALUES ('127.0.0.1', '${internalDeviceId}')";

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS trusted_devices (
													device_num INTEGER PRIMARY KEY,
													device_id TEXT UNIQUE,
													device_type TEXT										   -- The title of the device, like 'iPhone', or 'Samsung'
													);"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS devices (
                            device_num INTEGER PRIMARY KEY AUTOINCREMENT,
                            device_id TEXT UNIQUE,               -- this is the unique id for the device
                            is_admin TEXT,                       -- has this device ever logged in as an admin
                            os TEXT,                             -- the operating system, eg. Android, iOS, Windows, Linux
                            os_version TEXT,                     -- the version of the operating system
                            make TEXT,                           -- the make of the device (eg Samsung, Apple, Asus)
                            model TEXT,                          -- the model of the device
                            type TEXT,                           -- the type of device it is (eg. phone, tablet, computer)
                            language TEXT                        -- the langauge that the bowser is set to
                            );"													
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT OR REPLACE INTO devices (device_id, device_num) VALUES ('${internalDeviceId}', 0)";
													
	# Alter devices table
	# TODO do we really want this in devices? Is that a security concern?
	helperSqlite3 "${WIFI_DB_FILE}" "ALTER TABLE devices ADD COLUMN chat_user_name TEXT DEFAULT '';"
	helperSqlite3 "${WIFI_DB_FILE}" "ALTER TABLE devices ADD COLUMN chat_is_banned INTEGER DEFAULT 0;"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS forms (
                            form_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            title TEXT NOT NULL,                  -- this is the title used on the page for the form
                            button_title TEXT,                    -- this is the title for the button associated with this form
                            icon TEXT,                            -- this is the icon associated with this
                            image_id INTEGER,                     -- the image associated with this form
                            description TEXT,                     -- this will be a forward-facing description
                            submit_success_title TEXT,            -- title shown upon form submit success
                            submit_button_title TEXT,             -- the title of the submit button for the form
                            cancel_button_title TEXT,             -- the title for the cancel button
                            is_deleted INTEGER                    -- value of 1 means this is deleted
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS form_fields (
                            field_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            form_id INTEGER NOT NULL,             -- the id of the form that this field is associated with
                            sort_order INTEGER,                   -- the sort order for the fields in this form
                            title TEXT,                           -- the title of this field, should be unique
                            icon TEXT,                            -- the icon for this field
                            type TEXT,                            -- the type of field it is (eg. short, long)
                            options TEXT,                         -- options associated with this field
                            default_value TEXT,                   -- the default value associated with this field
                            is_deleted INTEGER                    -- value of 1 means this is deleted
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS form_responses (
                            response_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            date_time TEXT,                        -- the value associated with the
                            device_id TEXT,                        -- what was the id for the user who filled out the form
                            form_id INTEGER                        -- what was the title of the form they were filling out
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS form_data (
                            response_id INTEGER NOT NULL,
                            field_id INTEGER,                     -- the value associated with the
                            data TEXT                             -- what is the data associated with this particular field
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS users (
                            id INTEGER PRIMARY KEY NOT NULL,
                            user_name TEXT,
                            password TEXT,
                            salt TEXT,
                            status INTEGER
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS wifi_networks (
                            network_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            sort_order INTEGER,                   -- the sort order for the networks
                            ssid TEXT,                            -- the the ssid for the network
                            password TEXT,                        -- the password for the network
                            is_deleted INTEGER                    -- value of 1 means this is deleted
                            );"

	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS captive_portal_languages (
														id INTEGER PRIMARY KEY AUTOINCREMENT,
														language_iso TEXT,
														message TEXT,
														language_display_name TEXT,
														sort_order INTEGER
														);"
	
	helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS captive_portal_languages_default_messages (
														language_iso TEXT PRIMARY KEY,
														message TEXT
														);"

	# helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS chat_messages (
	# 												id INTEGER PRIMARY KEY AUTOINCREMENT,
	# 												device_num INTEGER,
	# 												room_id INTEGER,
	# 												message TEXT,
	# 												datetime TEXT,
	# 												persist_seconds INTEGER
	# 												);"	

	# helperSqlite3 "${WIFI_DB_FILE}" "CREATE TABLE IF NOT EXISTS chat_room_users (
	# 												device_num INTEGER,
	# 												room_id INTEGER,
	# 												highest_message_id_last_access INTEGER,
	# 												PRIMARY KEY (device_num, room_id)
	# 												);"																										

	# Insert default captive portal language messages
	captivePortalMessagesInsertStmt="INSERT OR REPLACE INTO captive_portal_languages_default_messages (language_iso, message) VALUES"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('eng', 'To use the WiFi, open your web browser and go to [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('cmn', '要使用 Wi-Fi，请打开您的网络浏览器并前往 [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('hin', 'वाईफाई का उपयोग करने के लिए, अपना वेब ब्राउज़र खोलें और [url] पर जाएँ')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('spa', 'Para usar el WiFi, abra su navegador web y vaya a [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('fra', 'Pour utiliser le WiFi, ouvrez votre navigateur et allez sur [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('arb', 'لاستخدام WiFi، افتح متصفح الويب واذهب إلى [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('ben', 'ওয়াই-ফাই ব্যবহার করতে, আপনার ওয়েব ব্রাউজারে যান এবং [url] এ যান')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('rus', 'Чтобы использовать Wi-Fi, откройте веб-браузер и перейдите на [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('por', 'Para usar o Wi-fi, abra o seu navegador e acesse [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('ind', 'Untuk menggunakan Wi-Fi, buka peramban web Anda dan kunjungi [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('urd', 'وائی فائی کو استعمال کرنے کیلئے، اپنا ویب براؤزر کھولیں اور [url] پر جائیں')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('deu', 'Um das WLAN zu nutzen, öffnen Sie Ihren Webbrowser und gehen Sie zu [url]')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('jpn', 'WiFiを使用するには、ウェブブラウザを起動して[url]を開いてください。')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('swh', 'Fungua mtandao wa web kwenye simu yako kisha ufungue kurasa la [url] ili kutumia mtambo wa WiFi')"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('tur', \"Wi-Fi'yi kullanmak için web tarayıcınızı açın ve [url] adresine gidin\")"
	helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('kor', 'WiFi를 사용하려면 웹 브라우저를 열고 [url]로 이동하세요')"
  helperSqlite3 "${WIFI_DB_FILE}" "${captivePortalMessagesInsertStmt} ('pes', 'برای استفاده از وای-فای، مرورگر وب را باز کنید و به [url] بروید')"

	# Insert language messages if the database is empty
	php -f "${rootDir}/srv/http/lib/Renew/Utilities/CmdLineSetUpCaptivePortalLanguages.php"

	helperSqlite3 "${SD_DB_FILE}" "CREATE TABLE IF NOT EXISTS sd_interaction_stats (
                            interaction_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            session_id TEXT,                      -- the identifier for this session: 'YmdHMSN-(random)'
                            session_start_datetime TEXT,          -- the datetime the session stated: 'Y-m-d H:M:S.sss' (UTC)
                            file_path TEXT,                       -- path to the file transferred
                            date_time TEXT,                       -- datetime it was completed: 'Y-m-d H:M:S.sss' (UTC)
                            push_durration INTEGER,               -- the durration of the file transfer in seconds
                            file_size_bytes INTEGER,              -- size of the file in bytes
                            download_device TEXT,                 -- the download device
                            file_id INTEGER,                      -- the file_id as found in the media db
                            status TEXT                           -- the status of the transfer: 'success' -OR- 'cp exit code'
                            );"

	helperSqlite3 "${SD_DB_FILE}" "CREATE TABLE IF NOT EXISTS settings (
                            key TEXT PRIMARY KEY,
                            value TEXT
                            );"

	helperSqlite3 "${BT_DB_FILE}" "CREATE TABLE IF NOT EXISTS bluetooth_interaction_stats (
                            interaction_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            file_path TEXT,                       -- the path to the file that was attempted or transferred
                            device_id TEXT,                       -- the bluetooth mac address of the device being pushed to
                            device_name TEXT,                     -- optional, the bluetooth device name being transferred to
                            date_time TEXT NOT NULL,              -- the date and time of the transmission 'Y-m-d H:M:S.sss' (UTC)
                            push_durration INTEGER,               -- the total time of the transfer in seconds
                            interaction_type TEXT NOT NULL,       -- the interaction type (push, scan ... )
                            hci_device TEXT,                      -- the hci device on the local machine that is being used (eg hci0...)
                            file_id INTEGER                       -- the id of the file as found in the media database on the LightStream
                            );"

	helperSqlite3 "${BT_DB_FILE}" "CREATE TABLE IF NOT EXISTS settings (
                            key TEXT PRIMARY KEY,
                            value TEXT
                            );"

	helperSqlite3 "${SATELLITE_DB_FILE}" "CREATE TABLE IF NOT EXISTS satellite_interaction_stats (
                            interaction_id INTEGER PRIMARY KEY AUTOINCREMENT,
                            date_time TEXT NOT NULL,              -- the datetime that the transmission was completed 'Y-m-d H:M:S.sss'
                            msg_tx TEXT,                          -- this is the message that was transmitted hexidecima encoded
                            msg_rx TEXT,                          -- this was the response to the message sent
                            msg_type TEXT,                        -- this is the type of message (eg stats)
                            msg_description TEXT                  -- this is a description of the format key1=value1;key2=value2;
                            );"

	helperSqlite3 "${SATELLITE_DB_FILE}" "CREATE TABLE IF NOT EXISTS settings (
                            key TEXT PRIMARY KEY,
                            value TEXT
                            );"

	# the default user
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} INTO users VALUES(1,'admin','70fbc60cc06be29de4cf60465236e1a05ab73545a8145169e7db120abab1b8f3','afhDj2/lXaHsDPqE98k878QNz6j7/lDQ22/qk10Sgxc=',1);"

	# add some values for the wifi settings database to initialize it
	ssid=$(_getLightStreamDefaultName)
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('ssid', '${ssid}');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('wifi_password_enabled', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('wifi_password', '');"
	# wifi channel of 0 is random
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('channel', '0');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('wifi_max_users', '20');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('captive_portal_mode', 'default');" # default, message, disabled
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('captive_portal_mode_message_media_file_id', '0');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('captive_portal_mode_message_media_autoplay', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('reporting_email', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('password_hint', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('help_messages_highlight', 'true');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('chat_enable_user_chat', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('chat_enable_global_chat', 'true');"

	# for fixing potentially blank channel
	channel=$(helperSqlite3 "${WIFI_DB_FILE}"  "SELECT value FROM settings WHERE key = 'channel'")
	if [[ "${channel}" == "" ]]; then
		helperSqlite3 "${WIFI_DB_FILE}" "UPDATE settings SET value = '1' WHERE key = 'channel'"
	fi

	# general settings
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_disable_sliders', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_logo', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_logo_enabled', 'true');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_logo_text', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_logo_mode', 'default');" #default, text, image
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_language_redirect', 'true');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_download_mode', 'false');"

	# Remove old general settings
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM settings WHERE key = 'general_site_title'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM settings WHERE key = 'general_play_title'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM settings WHERE key = 'general_download_title'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM settings WHERE key = 'general_view_title'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM settings WHERE key = 'general_more_media_title'"

	# featured settings
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('featured_slider_disabled', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('featured_slider_autoplay_speed', '5');"

	# greeting settings
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_enabled', 'false');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_show_before_featured', 'true');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_title', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_text', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_mode', 'text');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_mode_media_file_id', '0');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_mode_media_image_src', '');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_mode_media_image_action', 'download');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('greeting_mode_image_src', '');"

	# WiFi Settings
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('site_url', 'GoWifi.org');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('wifi_max_downloads', '3');"

	# for the initial setup walk-through ... if it is TRUE, then the setup should show
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('setup_should_show', 'true');"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('statistics_include_raw_data', 'false');"

	# insert the hashes into the db
	# user and admin js and css .. get it from this current repo
	htdocsDir="${rootDir}/srv/http/htdocs"
	jsFileAdmin="${htdocsDir}/js/application.admin.min.js"
	jsFileUser="${htdocsDir}/js/application.user.min.js"
	cssFileAdmin="${htdocsDir}/css/application.admin.min.css"
	cssFileUser="${htdocsDir}/css/application.user.min.css"
	hashAdminJs=$(md5sum "${jsFileAdmin}" | cut -d' ' -f1)
	hashUserJs=$(md5sum "${jsFileUser}" | cut -d' ' -f1)
	hashAdminCss=$(md5sum "${cssFileAdmin}" | cut -d' ' -f1)
	hashUserCss=$(md5sum "${cssFileUser}" | cut -d' ' -f1)
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT OR REPLACE ${intoSettingsStmt} ('hash_admin_js', '${hashAdminJs}');"
	helperSqlite3 "${WIFI_DB_FILE}" "INSERT OR REPLACE ${intoSettingsStmt} ('hash_admin_css', '${hashAdminCss}');"
	
	# Remove old theme items from the database
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM themes WHERE key = 'background_button_2'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM themes WHERE key = 'background_button_followup'"
	helperSqlite3 "${WIFI_DB_FILE}" "DELETE FROM themes WHERE key = 'background_featured'"

	# migrate over the site-settings into the settings table
  local adminLanguage=$(helperSqlite3 "${WIFI_DB_FILE}" "SELECT setting FROM site_settings WHERE slug = 'language'")
	local reportingEmail=$(helperSqlite3 "${WIFI_DB_FILE}" "SELECT setting FROM site_settings WHERE slug = 'reporting_email'")
	local generalLogo=$(helperSqlite3 "${WIFI_DB_FILE}" "SELECT setting FROM site_settings WHERE slug = 'general_logo'")
	[[ -z "${adminLanguage}" ]] && dataDir="/srv/http/htdocs/media/bluetooth"
	[[ -z "${reportingEmail}" ]] && rejectDelay="48"
	[[ -z "${generalLogo}" ]] && engineLog="/var/log/lightstream/lsbt.log"

	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('admin_langauge', '${localAdminLanguage}')"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('reporting_email', '${reportingEmail}')"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('general_logo', '${generalLogo}')"

	# secure' where the leds do not dance on boot, wifi does not turn on, and the sd copy feature should do nothing
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('system_mode', 'normal')"

	# setup the default followup form
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('form_followup_enabled', 'true')"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('form_followup_id', '0')"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('form_email_enabled', 'false')"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('form_email_stats_enabled', 'false')"

	# the default form ...
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} INTO forms (form_id, title, button_title, icon, image_id, description, submit_success_title, submit_button_title, cancel_button_title, is_deleted) VALUES (0, 'Contact Us', 'Contact Us', '&#xE0B9;', '', '', '', '', '', 0)"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} INTO form_fields (field_id, form_id, sort_order, title, icon, type, options, default_value, is_deleted) VALUES ( 1, 0, 1, 'Name', '&#xE7FD;', 'short', '', '', 0 )"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} INTO form_fields (field_id, form_id, sort_order, title, icon, type, options, default_value, is_deleted) VALUES ( 2, 0, 2, 'Email', '&#xE0BE;', 'short', '', '', 0 )"
	helperSqlite3 "${WIFI_DB_FILE}" "${insertStmt} INTO form_fields (field_id, form_id, sort_order, title, icon, type, options, default_value, is_deleted) VALUES ( 3, 0, 3, 'Message', '&#xE254;', 'long', '', '', 0 )"

	# set some default values, insert or ignore (should not reset the values if they already exist)
	helperSqlite3 "${SATELLITE_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('txfrequency', '7 days');"
	helperSqlite3 "${SATELLITE_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('interface', 'usb');"
	helperSqlite3 "${SATELLITE_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('transmit_test', 'false');"

	if [[ -e "${lsbtConfFile}" ]]; then
		dataDir="$(grep '^data_dir =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		maxPushRetry="$(grep '^max_push_retry =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		rejectDelay="$(grep '^reject_delay =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		engineLog="$(grep '^engine_log =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		engineLogLevel="$(grep '^engine_log_level =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		scanDelay="$(grep '^scan_delay =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		deviceName="$(grep '^device_name =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		scanForDevices="$(grep '^scan_for_devices =' "${lsbtConfFile}" | awk -F' = ' '{print $2}')"
		rm -Rf $(dirname "${lsbtConfFile}")
	fi
    # set some defaults
	[[ -z "${dataDir}" ]] && dataDir="/srv/http/htdocs/media/bluetooth"
	[[ -z "${rejectDelay}" ]] && rejectDelay="48"
	[[ -z "${engineLog}" ]] && engineLog="/var/log/lightstream/lsbt.log"
	[[ -z "${engineLogLevel}" ]] && engineLogLevel="ERROR"
	[[ -z "${scanDelay}" ]] && scanDelay="15"
	[[ -z "${deviceName}" ]] && deviceName=$(_getLightStreamDefaultName)
	[[ -z "${scanForDevices}" ]] && scanForDevices="1"

	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('data_dir', '${dataDir}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('max_push_retry', '${maxPushRetry}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('reject_delay', '${rejectDelay}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('engine_log', '${engineLog}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('engine_log_level', '${engineLogLevel}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('scan_delay', '${scanDelay}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('device_name', '${deviceName}');"
	helperSqlite3 "${BT_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('scan_for_devices', '${scanForDevices}');"

	# sd card settings ...
	helperSqlite3 "${SD_DB_FILE}" "${insertStmt} ${intoSettingsStmt} ('sd_lock_disabled', 'false');"

	# go ahead and write the theme file last because it will also reset the smarty cache which we need to do in the case of reset to defaults
	versionGt 4.0.0
	isGt4=$?
	themeCssFile=$(helperSqlite3 "${WIFI_DB_FILE}"  "SELECT value FROM settings WHERE key = 'themes_css_file'")
	if [[ $isGt4 != 0 ]] || [[ "${themeCssFile}" == "" ]] || [[ "${insertStmt}" == "INSERT OR REPLACE" ]]; then
		php -f "${rootDir}/srv/http/lib/Renew/Utilities/CmdLineRestoreDefaults.php"
	fi

	# delete old tables
	helperSqlite3 "${WIFI_DB_FILE}" "DROP TABLE phinxlog"
	helperSqlite3 "${WIFI_DB_FILE}" "DROP TABLE groups"
	helperSqlite3 "${WIFI_DB_FILE}" "DROP TABLE user_groups"
	helperSqlite3 "${WIFI_DB_FILE}" "DROP TABLE site_settings"
}

function _lsDatabasesInitializeSortOrderTriggers {
	local DB_FILE="${1}"
	local DB_TABLE="${2}"
	local ID_COL="${3}"

	helperSqlite3 "${DB_FILE}" "CREATE TRIGGER IF NOT EXISTS ${DB_TABLE}_sort_order_insert
                              BEFORE INSERT ON ${DB_TABLE}
                              FOR EACH ROW WHEN NEW.is_deleted = 0
                              BEGIN
                                 UPDATE ${DB_TABLE} SET sort_order = sort_order + 1 WHERE sort_order = NEW.sort_order AND ${ID_COL} = NEW.${ID_COL} AND is_deleted = 0;
                              END;"

# WITH NewSequence AS ( SELECT rowid AS this_rowid, (SELECT COUNT(*) FROM ${DB_TABLE} b WHERE a.sort_order > b.sort_order AND b.${ID_COL} = a.${ID_COL} AND b.is_deleted = 0 ) AS sort_order_new FROM ${DB_TABLE} a WHERE a.is_deleted = 0 AND a.${ID_COL} = NEW.${ID_COL} ) UPDATE NewSequence SET sort_order = sort_order_new;
	helperSqlite3 "${DB_FILE}" "CREATE TRIGGER IF NOT EXISTS ${DB_TABLE}_update_sort_order
                              BEFORE UPDATE OF sort_order, is_deleted, ${ID_COL} ON ${DB_TABLE}
                              FOR EACH ROW WHEN NEW.is_deleted = 0
                              BEGIN
                                 UPDATE ${DB_TABLE} SET sort_order = sort_order + 1 WHERE sort_order = NEW.sort_order AND ${ID_COL} = NEW.${ID_COL} AND is_deleted = 0 AND rowid != NEW.rowid;
                              END;"
}


# this allows the first argument to be a function
# the function is called with the other arguments after the function
helperCallFirstArgIfFunctionAndExit $@
