#!/bin/bash

# README: this is the file that gets run and actually performs the update
#######################################################
function preClearLogsIfNeeded {
	# Compare current LightStream version number to 3.3.0
	${THIS_DIR}/rwo_system.sh versionGe 3.3.0
	isGt=$?
	# If current LightStream version is greater or equal to 3.3.0, then there is not a need to clear thelogs
	if [ $isGt == 0 ]; then
		rwoLog "LightStream version is greater than 3.3.0, and will not clear logs at this time"
		return 0
	fi
	# fix problem with overgrown log files versions less than 3.3.0 (CMV 09 Sept 2015)
	rwoLog "Removing nginx logs to prevent excessive consumption of storage space..."
	rm "/var/log/nginx/"*
}

function preConsolidateMediaIntoMasterDirectoryIfNeeded {

	# check if you should consolidate the media ...
	${THIS_DIR}/rwo_system.sh versionLt 4.0.0
	isLt=$?
	if [[ $isLt == 0 ]]; then
		rwoLog "LightStream version is greater or equal to 4.0.0, and media consolidation is not required"
		# Declare indexed array of media directory names.
		declare -a mediaDirs=(bluetooth wifi usb)
		# create the new media directories
		mkdir -p "/srv/http/htdocs/media/master"
		# Iterate through array of media directories.
		for d in ${mediaDirs[*]}; do
			# Define absolute path of current media directory.
			currentMediaDir=/srv/http/htdocs/media/${d}
			if [ ! -d "${currentMediaDir}" ]; then break; fi
			rwoLog "Removing hidden files from ${d} directory on LightStream..."
			find $currentMediaDir -type f -print0 | grep -zZ '\.DS_Store\|\.gitignore\|metadata\.json' | xargs -0I{} rm -f {}
			# Change working directory to current media directory.
			cd $currentMediaDir
			# Iterate through each file under current media directory.
			for f in $(find * | sort | grep -v '/\.'); do
				# Define new absolute path for current file.
				newFilePath="/srv/http/htdocs/media/master/${f}"
				# create the file's directory
				newFileDir=$(dirname "${newFilePath}")
				mkdir -p "${newFileDir}"
				# move the file or create the directory in the master directory
				if [ -d "${f}" ]; then
					rwoLog "Creating path to directory ${f} under 'master' on LightStream..."
					mkdir -p "$newFilePath"
				elif [ -f "${f}" ]; then
					rwoLog "Moving file ${f} to master directory"
					mv -f "${f}" "$newFilePath"
				fi
			done
			# you are now finished with this current media dir
			rm -R "${currentMediaDir}"
		done
	fi

	# check if you create the directory for downloads
	downloadsDirectory="/srv/http/htdocs/media/downloads/"
	if [[ ! -d "${downloadsDirectory}" ]]; then
		mkdir -p "${downloadsDirectory}"
	fi

}

function preGenerateSystemUuidIfNeeded {
    uuidSecretFile=/etc/lightstream/uuid
    if [ ! -f "${uuidSecretFile}" ]; then
        mkdir -p $(dirname "${uuidSecretFile}")
        touch "${uuidSecretFile}"
        echo $(uuidgen | tr -d -) > "${uuidSecretFile}"
    fi
}

# Remove currently installed version of hostapd, if any.
function preRemoveCurrentHostapdFromSystem {
	if [ -n "$(pacman -Q | grep hostapd)" ]; then
		rwoLog "Removing current hostapd package from system..."
		pacman -R --noconfirm --noprogressbar $(pacman -Q | grep hostapd | cut -d' ' -f1)
	else
		rwoLog "No version of hostapd currently installed on LightStream."
	fi
	# remove any custom hostapd unit service file too
	rm /etc/systemd/system/hostapd.service
}

# Stop LightStream services.
function preStopLightStreamServices {
		rwoLog "Stopping some lightstream services..."
		systemdUnitsToStop=( bluetooth dnsmasq dnsmasq.configure incrond iptables lsbt nginx php-fpm rwo_wifi rwo_powerButtonListener-d rwo_wifi@wlan0Ap )
		# Daemon services.
		for systemdUnit in ${systemdUnitsToStop[@]}; do
			rwoLog "Stop systemd unit: ${systemdUnit}"
			systemctl stop "${systemdUnit}"
		done
}

function preRemoveOldAndDepricatedFilesAndSoftware {
	rwoLog "Disable old systemd units"
	systemdUnitsDepricated=( dnsmasq dnsmasq.configure clear_compiled_templates clear_client_date_offset hostapd network-wireless@wlan0 network-wireless@wlan1 network-wireless@eth0 rwo_checkhdspace sshd )
	# disable all the out-of date software
	for systemdUnit in ${systemdUnitsDepricated[@]}; do
		rwoLog "Disable systemd unit:${systemdUnit}"
		systemctl stop "${systemdUnit}"
		systemctl disable "${systemdUnit}"
	done

	# Check whether old media directory exists.
	if [[ -d /srv/http/media/ ]]; then
		# Remove old media directory. It now resides under /srv/http/htdocs/.
		log "Removing old media directory..."
		rm -fr /srv/http/media/
	fi

	# Check whether httpd service is still active.
	if [[ -n $(systemctl status httpd | grep 'Active: active') ]]; then
		log "Stopping the httpd service..."
		systemctl stop httpd
	fi

	# Check for apache packages.
	if [ -n "$(pacman -Q | grep apache)" ]; then
		log "Attempting to remove apache packages..."
		pacman --noconfirm --noprogressbar -Rss apache php-apache
	fi

	# Check for at package.
	if [ -n "$(pacman -Q | grep -w at)" ]; then
		log "Attempting to remove at package..."
		pacman --noconfirm --noprogressbar -Rss at
	fi

	# Check for libnl1 package.
	if [ -n "$(pacman -Q | grep -w libnl1)" ]; then
		log "Attempting to remove libnl1 package..."
		pacman --noconfirm --noprogressbar -Rss libnl1
	fi

	# Check whether httpd configuration directory still exists.
	if [ -d /etc/httpd/ ]; then
		log "Removing residual httpd files..."
		rm -fr /etc/httpd/
	fi

	# remove pacman packages that cause conflict with xorgproto
	pacmanPackagesToRemove=( xf86vidmodeproto renderproto recordproto kbproto inputproto damageproto fixesproto xproto xextproto )
	for packageName in ${pacmanPackagesToRemove[@]}; do
		rwoLog "Removing pacman package:${packageName}"
		pacman --noconfirm -Rdd "${packageName}"
	done

	# move out of place firmware files ... for some reason the firmware files were in /usr/lib/firmware/lib/firmware
	mv /usr/lib/firmware/lib/firmware/* /usr/lib/firmware/

}

function prePreparePlatformSpecificUpdate {
	# Determine the architecture of the platform we're using.
	arch="$(uname -m)"
	arch="${arch%[a-z]}" # trim off the last letter of the architecture .. eg armv6h => armv6
	archDir="${UPDATE_DIR}/.architectures/${arch}"

	# Make sure a valid architecture was found.
	if [ -z "$arch" ]; then
		rwoLog "There was an error trying reading the processor architecture with \$(uname -m)."
	elif [ -d "${archDir}" ]; then
		cp -lR "${archDir}/"* "${UPDATE_DIR}/"
	else
		rwoLog "The architecure '${arch}' does not appear to be supported by this firmware update."
	fi

	# vendor specific update ...
	vendorDir="${UPDATE_DIR}/.vendors/${HWVENDOR}"
	if [ -z "$HWVENDOR" ]; then
		rwoLog "There was an error trying reading hardware vendor"
	elif [ -d "${vendorDir}" ]; then
		cp -lR "${vendorDir}/"* "${UPDATE_DIR}/"
	else
		rwoLog "The vendor '${HWVENDOR}' does not have any vendor specific update files"
	fi
}

# Install new or update existing arch packages.
function midInstallArchPackages {
	# Arch packages for the Raspberry Pi can be downloaded from http://mirror.archlinuxarm.org/
	# Define directory containing Arch packages.
	pkgDir="${UPDATE_DIR}/tmp/pkg"
	# Check whether pkg_dir contains any arch packages.
	if [ -d "$pkgDir" -a -n "$(ls "$pkgDir" | grep '.*\.pkg\.tar\.xz')" ]; then
		# Remove the current hostapd package from the system
		rwoLog "Updating Arch packages..."
		pacman --force --noconfirm --noprogressbar --needed -U "$pkgDir"/*.pkg.tar.xz
		rwoLog "Remove the package dir"
		rm -Rf "${pkgDir}"
	else
		rwoLog "No Arch packages found."
	fi
}

# Update the core LightStream software.
function midInstallLightStreamSoftware {
	# Check whether working directory was successfully changed to the UPDATE_DIR
	cd "${UPDATE_DIR}"
	if [ $? -eq 0 ]; then

		rwoLog "Install files to the root filesystem ... "
		find * -type d -print0 | grep -zZv '^tmp\|^\.' | xargs -0I{} install -o root -g root -d /{}
		find * -type f -print0 | grep -zZv '^tmp\|/\.\|^etc/lightstream/data\|^srv/http/data' | xargs -0I{} install -D -m 644 -o root -g root {} /{}

		rwoLog "Set file mode, owner, and group ..."
		find * -type f -print0 | grep -zZ '^usr/bin' | xargs -0I{} chmod 755 /{}
		chmod 755 /srv/http/lib/Renew/BashScripts/*.sh
		chown -R http:http /srv/http
		chown root:root /srv/http/lib/Renew/BashScripts/*.sh
		chmod 440 /etc/sudoers
		chown root:root /etc/sudoers
		chmod 644 /etc/logrotate.conf
		chown root:root /etc/logrotate.conf
		chmod -R 644 /etc/logrotate.d
		chmod 755 /etc/logrotate.d
		chown -R root:root /etc/logrotate.d
		chmod -R 777 /etc/lightstream

	else
		rwoLog "Unable to change working directory to $UPDATE_DIR."
	fi
}

# run this after all the files are installed
function postMakeBootcmdImg {
	if [ "${HWPLATFORM}" == 'pocket-v1' ]; then
		ubootScriptFile="/boot/bootcmd.scr"
		ubootScriptImage="/boot/bootcmd.img"
		rwoLog "create bootcmd file at:${ubootScriptImage}"
		rm -f "${ubootScriptImage}"
		mkimage -C none -A arm -T script -d "${ubootScriptFile}" "${ubootScriptImage}"
	fi
}

function postSetupDatabases {
	"${UPDATE_DIR}/usr/bin/rwo_system.sh" "lsDatabasesUpdate"
	# remove the ramDb
	rm /tmp/rwo_ramDb.sqlite3
}

# Start LightStream services.
function postSetupLightStreamServices {
	rwoLog "Setting up LightStream Services"

	rwoLog "Load any new kernel modules ..."
	depmod

	rwoLog "Reloading systemd unit files..."
	systemctl daemon-reload

	# these are some old units, and some need to be disabled, but the shoudl be started elsewhere when needed
	systemdUnitsToRemainDisabled=( hostapd lsbt nginx php-fpm clear_client_date_offset rwo_on1MinClock.timer rwo_on15MinClock.timer rwo_onMidnightClock.timer rwo_onBoot rwo_onSysInitTarget rwo_wifi )
	for systemdUnit in ${systemdUnitsToRemainDisabled[@]}; do
		rwoLog "Disable systemd unit:${systemdUnit}"
		systemctl stop "${systemdUnit}"
		systemctl disable "${systemdUnit}"
	done

	# enable these units
	systemdUnitsToEnable=( rwo_onSysInitTarget rwo_onBoot )
	for systemdUnit in ${systemdUnitsToEnable[@]}; do
		rwoLog "Enable systemd unit:${systemdUnit}"
		systemctl enable "${systemdUnit}"
	done

	# start the rwo_onBoot service to treat the install like a fresh boot
	rwoLog "Start LightStream services"

	# these are some old units, and some need to be disabled, but the shoudl be started elsewhere when needed
	systemdUnitsToStart=( rwo_onBoot incrond iptables rwo_powerButtonListener-d )
	for systemdUnit in ${systemdUnitsToStart[@]}; do
		rwoLog "Start systemd unit:${systemdUnit}"
		systemctl start "${systemdUnit}"
	done

	# now reload the udev rules in case they have changed ..
	udevadm control --reload-rules
}

function postScanForAnyMediaChanges {
	"${THIS_DIR}/rwo_system.sh" versionLe "${VERSION_FROM}" '4.3.0'
	isLe=$?
	if [[ $isLe == 1 ]]; then
		rwoLog "the version you came from is <= 4.3.0"
		rwoLog "scan for media changes ..."
		php -f "/srv/http/lib/Renew/Utilities/MediaDbScanUpdate.php"
	fi
}

function postPrintStatusToLightstreamLog {
	df -hH
}

# TODO: Can we remove local-gen?
 function postUpdateLocales {
	# look in the file in this update under etc/locale.gen
	 locale-gen
 }

function postRestartDeviceLauncerService {
	isActive=$(systemctl is-active rwo_deviceLauncher@*)
	if [[ $isActive == "active" ]]; then
		rwoLog "Will restart rwo_deviceLauncher@*"
		systemctl restart rwo_deviceLauncher@*
	fi
}

function postRegenerateCss {
	rwoLog "regenerate user css file"
	php -f "/srv/http/lib/Renew/Utilities/CmdLineRegenerateCss.php"
}

####################### MAIN ################################

# Include logging parallel to this file
THIS_DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
UPDATE_DIR=$(dirname $( dirname "${THIS_DIR}" ) )
HWPLATFORM=$( "${UPDATE_DIR}/usr/bin/rwo_system.sh" getHwPlatform )
HWVENDOR=$( "${UPDATE_DIR}/usr/bin/rwo_system.sh" getHwVendor )
VERSION_FROM=$( "${THIS_DIR}/usr/bin/rwo_system.sh" getVersion)
. "${THIS_DIR}/rwo_includeLogging.sh"
. "${THIS_DIR}/rwo_includeLeds.sh"
chmod -R 777 "${THIS_DIR}/"
# Set internal field separator to newline only.
IFS=$'\n'

# Check if it is being run as a firmware update (i.e. it is in a folder containing the string "UpdateFirmware")
if [ -n "$(echo "${UPDATE_DIR}" | grep 'UpdateFirmware')" ]; then
		rwoLog "============= UPDATE BEGIN ============="
		rwoLog "Found temporary directory $UPDATE_DIR."
		# Clean up the update directory upon any of the given signals.
		trap 'rm -Rf "$UPDATE_DIR"' EXIT INT HUP TERM
		rwoLog "run update on hw platform: ${HWPLATFORM} current version:${VERSION_FROM}"

		if [[ "${HWPLATFORM}" == 'pocket-v1' ]]; then
			ledsWifiUpdateLoopStart
		else
			rwoLog "incorrect hardware platform! .. exiting"
			exit 1
		fi

		# check if the internal card is permanently locked
		if [[ $(sdtool /dev/mmcblk0 status | grep "state: Permanent" | wc -l) -eq 1 ]]; then
			rwoLog "ERROR! SD Card /dev/mmcblk0 is permanently locked!!"
			ledsSdSetLockLed 1
			# sleep for 10 hrs
			sleep 36000
			exit 2
		fi

		preStopLightStreamServices
		preClearLogsIfNeeded
		preRemoveOldAndDepricatedFilesAndSoftware
		preRemoveCurrentHostapdFromSystem
		preConsolidateMediaIntoMasterDirectoryIfNeeded
		prePreparePlatformSpecificUpdate

		midInstallArchPackages
		midInstallLightStreamSoftware

		postMakeBootcmdImg
		postSetupDatabases
		postUpdateLocales
		postScanForAnyMediaChanges
		postSetupLightStreamServices
		postRegenerateCss
		postPrintStatusToLightstreamLog
		postRestartDeviceLauncerService

		if [[ "${HWPLATFORM}" == 'pocket-v1' ]]; then
			ledsWifiUpdateLoopStop
			ledsWifiSetLedsOn 0
		fi
		rwoLog "============= UPDATE END ============="
fi
