#!/bin/bash
# Select the proper hostapd binary, and configuration file to run
# prioritize with the most preferable adapter chipsets at the top

# Include logging to log stderr and stdout to specific file
. "/usr/bin/rwo_includeLogging.sh"
. "/usr/bin/rwo_includeHelpers.sh"
. "/usr/bin/rwo_includeLeds.sh"

function swapWlan0AndWlan1IfNeeded {
	#statements
	# get wlan0 and wlan1 OUI
	wlan0Oui=$(ip link show wlan0 | grep ether | awk '{print $2}' | cut -b 1-9)
	wlan1Oui=$(ip link show wlan1 | grep ether | awk '{print $2}' | cut -b 1-9)
	# to handle the case of the raspberry-pi 3
	# IF wlan0 OUI matched raspberry pi, AND there is a wlan1 THEN swap interface names
	systemctl stop network-wireless@wlan0 # clear out the old wlan0 configuration
	if [[ ${wlan0Oui} == "b8:27:eb:" ]] && [ -n "${wlan1Oui}" ]; then
		swapWlan0AndWlan1
	# this is the case for the pocket
	elif [[ ${wlan0Oui} == "c2:21:0d:" ]] && [[ ${wlan1Oui} == "c0:21:0d:" ]]; then
		swapWlan0AndWlan1
	# the case where you have wlan1 but no wlan0
	elif [ -z "${wlan0Oui}" ] && [ -n "${wlan1Oui}" ]; then
		rwoLog  "will rename wlan1 to wlan0"
		sleep 1.0
		ip link set wlan1 down
		ip link set wlan1 name wlan0
		rwoLog  "done renaming wlan1 to wlan0"
	fi
	sleep 3
	# start up the new wlan0 configuration
	systemctl start network-wireless@wlan0
}

function swapWlan0AndWlan1 {
	rwoLog  "will switch wlan1 and wlan0 interfaces"
	ip link set dev wlan0 down
	sleep 0.1
	ip link set dev wlan1 down
	sleep 0.1
	ip link set wlan0 name wlanTemp
	sleep 0.1
	ip link set wlan1 name wlan0
	sleep 0.1
	ip link set wlanTemp name wlan1
	rwoLog  "done switching wlan0 and wlan1 interfaces"
}

function getSettingStrFromWifiDatabaseWithKey {
	local key=$1
	local value=''
	value=$(helperSqlite3 "/etc/lightstream/data/lswifi.sqlite3" "SELECT value FROM settings WHERE key = '$key'")
	echo "$value"
}

function _selectHostapdBinAndConf {
	# check for the Realtek 8192CU or 8192EU adapter
	# TODO FIXME, this is a weak check b/c it coudl catch any realtek ... find a better check.
	ssid=$(getSettingStrFromWifiDatabaseWithKey 'ssid')
	channel=$(getSettingStrFromWifiDatabaseWithKey 'channel')
	maxWifiUsers=$(getSettingStrFromWifiDatabaseWithKey 'wifi_max_users')
	driver='nl80211'
	htCapab='[SHORT-GI-20]'
	securityBlockStr=''

	# check for random channel
	if [[ $channel == 0 ]]; then
		# generate a random chaneel between 1 and 10
		channel=$((1 + RANDOM % 10))
		rwoLog "generated random wifi channel: ${channel}"
	fi

	if [ "${hwPlatform}" == 'pocket-v1' ]; then
		hostapdBinSelected="${hostapdBinDir}/hostapd"
		htCapab='[SHORT-GI-20]'
		# set the txpower
		# TODO, make the txpower adjustable
		iw reg set US
		iwconfig wlan0 txpower 20
	elif [ -n "$(lsusb | grep Realtek)" ]; then
		rwoLog  "detected Realtek USB adapter"
		hostapdBinSelected="${hostapdBinDir}/hostapd.RTL8192CU"
		driver='rtl871xdrv'
		htCapab='[SHORT-GI-20][MAX-AMSDU-7935]'
	# check for the Ralink RT5370 adapter (the)
	elif [ -n "$(lsusb | grep RT5370)" ]; then
		rwoLog  "detected RT5370 USB adapter"
		hostapdBinSelected="${hostapdBinDir}/hostapd"
		htCapab='[SHORT-GI-20]'
	# as a catch-all for the possibility of unknown chipsets
	else
		rwoLog  "detected OTHER wifi interface, using default config"
		hostapdBinSelected="${hostapdBinDir}/hostapd"
		htCapab='[SHORT-GI-20]'
	fi

	# check for wifi Password, enable if needed
	wifiPasswordEnabled=$(getSettingStrFromWifiDatabaseWithKey 'wifi_password_enabled')
	wifiPassword=$(getSettingStrFromWifiDatabaseWithKey 'wifi_password')
	wifiPasswordLength=$(echo -n "${wifiPassword}" | wc -c)
	securityBlockStr=''
	if [[ $wifiPasswordEnabled == 'true' ]] && [[ $wifiPasswordLength -ge 8 ]] && [[ $wifiPasswordLength -lt 64 ]]; then
		# wpa=2 gives you WPA2 only
		# auth_algs 1=wpa, 2=wep, 3=both
		securityBlockStr="
auth_algs=1
wpa=2
wpa_key_mgmt=WPA-PSK
rsn_pairwise=CCMP
wpa_passphrase=${wifiPassword}"
	fi

	# check if the disable wifi password flag is set
	disableWifiPasswordFlagFile='/tmp/rwo/rwo_wifi@wlan0Ap.startNoPassword'
	if [[ -e "${disableWifiPasswordFlagFile}" ]]; then
		rm "${disableWifiPasswordFlagFile}"
		securityBlockStr=''
	fi

	# sanitize max_wifi_users data
	if [[ ! "$maxWifiUsers" =~ ^[0-9]+$ ]] || [[ "$maxWifiUsers" == "0" ]]; then
		maxWifiUsers=15
	fi

	hostapdConfStr="
interface=wlan0
driver=${driver}
ssid=${ssid}
hw_mode=g
channel=${channel}
auth_algs=1
beacon_int=100
max_num_sta=${maxWifiUsers}
ieee80211n=1
ht_capab=${htCapab}
#supported_rates=60 90 120 180 240 360 480 540
wmm_enabled=1
ctrl_interface=/var/run/hostapd
ctrl_interface_group=0
ap_max_inactivity=300
skip_inactivity_poll=0
max_listen_interval=100
disassoc_low_ack=1
${securityBlockStr}
"

	# write the hostapd to file
	echo "$hostapdConfStr" > "${HOSTAPD_RUNNING_CONF}"
}

function wlan0Ap {
	if [[ "${hwPlatform}" == 'pocket-v1' ]]; then
		# turn on the wifi-on led
		ledsWifiSetSixOclockLed 1
	fi
	# if there is an access point present, then do not start the hotspot
	apState=$(rwo_eth0WifiApState.sh)
	if [[ $apState != "none" ]]; then
		echo "there was an access point detected with state:${apState}"
		echo "exiting hostapd"
		systemd-run --on-active=5 /usr/bin/rwo_eth0WifiApConfigure.sh
		exit 0
	fi

	swapWlan0AndWlan1IfNeeded
	configureAndRestartNginx
	_selectHostapdBinAndConf
	ip link set wlan0 down
	# set wlan1 up so that you can use it for connecting to other networks
	ip link set wlan1 up

	hostapdCliBinSelected="${hostapdBinDir}/hostapd_cli"
	if [[ $hostapdBinSelected == "${hostapdBinDir}/hostapd.RTL8192CU" ]]; then
		hostapdCliBinSelected="${hostapdBinDir}/hostapd_cli.RTL8192CU"
	fi

	# clear out the download download queue sqlite3
	ramDbPath="/tmp/rwo_ramDb.sqlite3"
	rm "/tmp/rwo_ramDb.sqlite3"

	# run the hostapd binary with the proper configuration file
	# redirect stderr and stdout to prevent device ids from showing in logs
	"${hostapdBinSelected}" "${HOSTAPD_RUNNING_CONF}" &>/dev/null &
	PID_HOSTAPD=$!

	# run the node server
	systemctl --no-block start rwo_nodeServer

	# Fix bug where media doesn't load on the first boot
  curl "http://127.0.0.1/"

	# run the CLI after hostapd is up and running
	# so give hostapd some time go get up and running
	sleep 5
	"${hostapdCliBinSelected}" -a "/usr/bin/rwo_wifiOnHostapdChange.sh"
	exitCode=$?

	_onExit
	exit "${exitCode}"
}

function configureAndRestartNginx {
	nginxConfFile="/etc/nginx/nginx.conf"

	systemctl stop nginx

	# remove the old nginx.conf and copy the template into the new
	rm -f "${nginxConfFile}"
	touch "${nginxConfFile}"

	# get the lower case site URL
	siteUrl=$(getSettingStrFromWifiDatabaseWithKey 'site_url')
	siteUrl=$(echo "${siteUrl}" | tr '[:upper:]' '[:lower:]')
	[[ "${siteUrl}" == '' ]] && siteUrl='gowifi.org'
	blockRewriteHttp="
		# admin rewrites
		if (\$request_uri ~* \".*/admin$\"){
			rewrite ^.*$ http://${siteUrl}/admin/index.php;
			break;
		}
		if (\$request_uri ~* \".*/admin/$\"){
			rewrite ^.*$ http://${siteUrl}/admin/index.php;
			break;
		}
		if (\$request_uri ~* \".*/admin2$\"){
			rewrite ^.*$ http://${siteUrl}/admin/login.php?nosetup=true;
			break;
		}
		if (\$request_uri ~* \".*/admin2/$\"){
			rewrite ^.*$ http://${siteUrl}/admin/login.php?nosetup=true;
			break;
		}
		# dont rewrite node endpoints to get around a browser socket limit
		# if (\$request_uri ~* \"^/n/subscribe.*$\"){
		# 	break;
		# }
		if (\$host ~* \"^gowifi.org.*$\") {
			break;
		}
		if (\$host ~* \"^.*192\.168\.211\.1.*$\") {
			break;
		}
		# main host rewrite
		if (\$host != ${siteUrl}) {
			rewrite ^.*$ http://${siteUrl}/;
		}"

		# for https just replace http:// with https://
	blockRewriteHttps="${blockRewriteHttp//http:\/\//https:\/\/}"

	# inserting the serial number into the request should give us a perfroamnce boost!
	# fastcgi_param SERVER_PROTOCOL HTTP/1.0;
	#	fastcgi_param SERVER_PROTOCOL "$server_protocol";
	serialNumber=$(rwo_system.sh getSerialNumber)
	blockFastCgiPortal="
		location / {
			fastcgi_pass  unix:/run/php-fpm/php-fpm.sock;
			fastcgi_index index.php;
			fastcgi_param SERIAL_NUMBER "$serialNumber";
			fastcgi_param HW_PLATFORM "$hwPlatform";
			fastcgi_param CAPTIVE_TYPE \$CAPTIVE_TYPE;
			fastcgi_param SCRIPT_FILENAME /srv/http/htdocs/captive_portal_handler.php;
			include fastcgi_params;
		}
"
	blockFastCgiNormal="
		# Prevent execution of PHP files in upload directories
		location ~ ^/(media|uploads|images|admin/utilities/fine_uploader/files|admin/utilities/fine_uploader/chunks)/.*\.php$ {
			deny all;
			access_log off;
			log_not_found off;
		}

		# Prevent direct access of media directory
		location ~ ^/media/.*$ {
			internal;
		}

		location ~ ^/dist_node/.*$ {
			internal;
		}

		location ~ \.php$ {
			fastcgi_pass  unix:/run/php-fpm/php-fpm.sock;
			fastcgi_index index.php;
			fastcgi_param SERIAL_NUMBER "$serialNumber";
			fastcgi_param HW_PLATFORM "$hwPlatform";
			fastcgi_param CAPTIVE_TYPE \$CAPTIVE_TYPE;
			fastcgi_param SCRIPT_FILENAME \$document_root\$fastcgi_script_name;
			include fastcgi_params;
		}
"
	blockNodeJsProxy="
		proxy_pass http://localhost:3000;
		proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
		proxy_set_header Host \$http_host;
		proxy_set_header X-Forwarded-Proto \$scheme;
		proxy_buffering off;
"

	blockNodeJs="
		location ~ ^/n/subscribe.*$ { 
			$blockNodeJsProxy

			# for SSE
			proxy_set_header Connection '';
			proxy_http_version 1.1;
			chunked_transfer_encoding off;
			proxy_cache off;
		}

		location ~ ^/n/.*$ {
			$blockNodeJsProxy
		}
"

blockSlimPhp="
		location / {
			# index  index.php;
			try_files \$uri /index.php\$is_args\$args; # Slim PHP
		}
"

# this is the string that will get written to nginx.conf
	nginxConfFileBlock="
user http;
worker_processes  2;

error_log  /var/log/nginx/error.log error;

events {
	worker_connections  1024;
}


http {
	include       mime.types;
	default_type  application/octet-stream;
	sendfile      on;
	root /srv/http/htdocs/;

	client_body_timeout   100m;
	fastcgi_read_timeout  100m;

	client_max_body_size       2047M;
	fastcgi_max_temp_file_size 2047M;
	log_format main '\$http_x_forwarded_for - \$remote_user [\$time_local] '
		'\"\$request_method \$scheme://\$host\$request_uri\$is_args\$args \$server_protocol\" '
		'\$status \$body_bytes_sent \"\$http_referer\" '
		'\"\$http_user_agent\" \$request_time';
	access_log  /var/log/nginx/host.access.log main;
	server_names_hash_bucket_size 64;
	#rewrite_log on;

	# add support for gzipping, should gzip javascript and hopefully increase performance
	gzip on;
	gzip_comp_level 2;
	gzip_http_version 1.1;
	gzip_proxied any;
	gzip_min_length 1100;
	gzip_buffers 16 8k;
	gzip_types text/plain text/html text/css application/x-javascript text/xml application/xml application/xml+rss text/javascript;


	# localhost and static server block
	server {
		listen 80;
		set \$CAPTIVE_TYPE 'none';
		server_name
		localhost
		127.0.0.1
		testaurl.org;

$blockSlimPhp

$blockNodeJs

$blockFastCgiNormal
	}


	# HTTPS Server
	server {
		listen 443 ssl;
		set \$CAPTIVE_TYPE 'none';
		# don't catch all server names ...
		server_name ${siteUrl};

$blockRewriteHttps

		ssl_certificate /etc/nginx/ssl/server.crt;
		ssl_certificate_key /etc/nginx/ssl/server.key;

$blockSlimPhp

$blockNodeJs

$blockFastCgiNormal
	}


	# IOS Captive Potal Server Block
	server {
		# list of server names for Apple's captive potal checking
		# TODO, is this list complete?
		# gets passed into the fastcgi as a \$_SERVER variable
		set \$CAPTIVE_TYPE 'ios';
		server_name
				www.appleiphonecell.com
				captive.apple.com
				captive.apple.com
				www.apple.com
				www.itools.info
				www.ibook.info
				www.airport.us
				www.thinkdifferent.us
				;
		listen 80;

		# Non-ios captive portal (!~* means does not match regex)
		#if (\$http_user_agent !~* .*CaptiveNetworkSupport.*) {
		#		rewrite ^.*$ http://192.168.211.1/index.php;
		#		break;
		#}

$blockFastCgiPortal
	}


	# Android Captive Portal SERVER
	# ... Nginx tries to match server blocks by server name
	server {
		# list of server names for google's captive portal checking
		server_name
			clients1.google.com
			clients3.google.com
			clients.l.google.com
			connectivitycheck.android.com
			connectivitycheck.gstatic.com
			gstatic.com
			play.googleapis.com
			google.com
			www.google.com

			# windows
			# msftconnecttest.com
			# ipv6.msftncsi.com
			# www.msftncsi.com
			# dns.msftncsi.com

			# linux
			# connectivity-check.ubuntu.com
			;
		listen 80;
		# gets passed into the fastcgi as a \$_SERVER variable
		set \$CAPTIVE_TYPE 'android';

		 # if the request uri does not match gen_204 or generate_204
		 if (\$request_uri !~* "ge.*_204") {
		 	return 204;
			break;
		 }

		# for all other requests matching gen.*_204
		# process the portal page
		# the portal page will decide what to return (204, 302, etc)
$blockFastCgiPortal
	}


	# General HTTP Server
	server {
		listen 80;
		# gets passed into the fastcgi as a \$_SERVER variable
		set \$CAPTIVE_TYPE 'none';
		# regex to 'catch-all' for server names
		server_name ~^.*$;

$blockRewriteHttp

$blockSlimPhp

$blockNodeJs

$blockFastCgiNormal
	}
}
"
	# replace the nginx.conf block string
	echo "#<BLOCK_NGINX_CONF>" > "${nginxConfFile}"
	gawk -i inplace -v r="$nginxConfFileBlock" '{gsub(/#<BLOCK_NGINX_CONF>/,r)}1' "${nginxConfFile}"

	# turn on iptables, nginx, and php-fpm
	systemctl start iptables
	systemctl start php-fpm
	systemctl start php7.0-fpm
	systemctl start nginx
}

###################### WLAN1 ##########################################

# Status messages:
# failed
# connecting
# obtaining ip address
# connected
function wlan1Connect {

	if [[ ! -e "${WLAN1CONNECT_SSID_FILE}" ]]; then
		rwoLog "connect failed no ssid supplied"
		echo "failed" > ${WLAN1CONNECT_STATUS_FILE}
		exit 1
	fi

	# read the ssid, it should be encoded in hex
	ssid=$(cat "${WLAN1CONNECT_SSID_FILE}")
	password='';
	if [[ -e "${WLAN1CONNECT_PASSWORD_FILE}" ]]; then
		password=$(cat "${WLAN1CONNECT_PASSWORD_FILE}")
	fi

	if [[ "$ssid" == '' ]]; then
		rwoLog "connect failed no ssid supplied"
		echo "failed" > ${WLAN1CONNECT_STATUS_FILE}
		exit 1
	fi

	# get the password
  	echo "connecting" > ${WLAN1CONNECT_STATUS_FILE}

  _wlan1Disconnect
  sleep 1

  # if there is no password, assume it is an open wifi network.
  wpaSupplicantStatus=1
  if [[ "$password" == "" ]]; then
  # ssid is encoded in hex, so don't quote it, this is so the ssid can use special characters
    networkStr="network={
    ssid=${ssid}
    key_mgmt=NONE
		}"
    wpa_supplicant -B -i ${WLAN1IFACE} -c <(echo "${networkStr}")
	wpaSupplicantStatus=$?
  else
	# need to get the psk, but don't want to use the string version of the wifi network
	# reason we do this is because the ssid is encoded in hex so they can more easily use special characters
	# psk=$(wpa_passphrase "${ssid}" "${password}" | grep [^\#]psk= | cut -d= -f 2)
	networkStr="network={
    ssid=${ssid}
    psk=\"${password}\"
		}"
    wpa_supplicant -B -i ${WLAN1IFACE} -c <(echo "${networkStr}")
	wpaSupplicantStatus=$?
  fi

 	 # check if wpa supplicant failed
	if [[ $wpaSupplicantStatus != 0 ]]; then
		rwoLog "wpa_supplicant failed with code ${wpaSupplicantStatus}"
		echo "failed" > ${WLAN1CONNECT_STATUS_FILE}
	fi

  # wait for the network to connect
  for (( i = 0; i < 40; i++ )); do
    sleep 0.5
    _wlan1GetConnectedSsidHex
    # check for connection success
    if [[ "${connectedSsidHex}" == "${ssid}" ]]; then
      # try to get dhcp address
      wlan1FlushIpAddress
      echo "obtaining ip address" > ${WLAN1CONNECT_STATUS_FILE}
      sleep 0.5
      dhcpcd --timeout 20 ${WLAN1IFACE}
      dhcpReqStatus=$?
      # dhcp success!
    	if [ $dhcpReqStatus -eq 0 ]; then
        echo "connected" > "${WLAN1CONNECT_STATUS_FILE}"
    	else
        rwoLog "connect failed - to obtain dhcp ${dhcpReqStatus}"
        echo "failed" > ${WLAN1CONNECT_STATUS_FILE}
      fi
      break
    fi
  done

	printoutStr=$(cat "${WLAN1CONNECT_STATUS_FILE}")
	rwoPrintout "${printoutStr}"

	if [[ "${printoutStr}" != "connected" ]]; then
		_wlan1Disconnect
	else
		while [[ $LOOP == 1 ]]; do
			sleep 0.5
		done
	fi
}

function _wlan1GetConnectedSsid {
  connectedSsid=$(iw dev ${WLAN1IFACE} link | grep "SSID" | sed -e 's/^[[:space:]]*//' | cut -d' ' -f2-)
}

function wlan1GetConnectedSsid {
  _wlan1GetConnectedSsid
  rwoPrintout "${connectedSsid}"
}

function _wlan1GetConnectedSsidHex {
   _wlan1GetConnectedSsid
  connectedSsidHex=$(echo -n "${connectedSsid}" | od -t x1 -A n -w100000 | tr -d ' ')
}

function wlan1GetConnectedSsidHex {
  _wlan1GetConnectedSsidHex
  rwoPrintout "${connectedSsidHex}"
}

function _wlan1GetConnectStatus {
  connectStatus=$(cat "${WLAN1CONNECT_STATUS_FILE}")
	if [[ -z "${connectStatus}" ]]; then
		connectStatus='disconnected'
	fi
}

function wlan1GetConnectStatus {
  _wlan1GetConnectStatus
  rwoPrintout "${connectStatus}"
}

function _wlan1Disconnect {
	wlan1FlushIpAddress
	sudo killall wpa_supplicant
}

function wlan1Disconnect {
	_wlan1Disconnect
	echo 'disconnected' > ${WLAN1CONNECT_STATUS_FILE}
	rwoPrintout 'success'
}

function wlan1FlushIpAddress {
	pkill -9 dhcpcd
	ip addr flush ${WLAN1IFACE}
}

function wlan1Scan {
	# iwlist is in wireless_tools package
	# iwlist is much faster than using the iw command to scan
	# just returns a list of SSIDs separated by newlines
	networksScanned=$(iwlist wlan1 scan | grep "ESSID\|Signal level=\|WPA2 Version" | sed -e 's/^[[:space:]]*//')
	#Quality=48/70  Signal level=-62 dBm
	#ESSID:"Renew"
	#Quality=68/70  Signal level=-42 dBm
	#ESSID:"Renew"
	#IE: IEEE 802.11i/WPA2 Version 1

	while read -r line; do
		if [[ "${line}" =~ ^ESSID: ]]; then
			ssid=$(echo -ne "${line}" | cut -d':' -f2- | sed 's/^"\(.*\)"$/\1/')
			ssidHex=$(echo -ne "${ssid}" | od -t x1 -A n -w100000 | tr -d ' ')
		elif [[ "${line}" =~ ^Quality=.*Signal\ level= ]]; then
			if [[ "${ssid}" != '' ]]; then
				rwoPrintout "${ssid}"
				rwoPrintout "${quality}"
				rwoPrintout "${security}"
				rwoPrintout "${ssidHex}"
			fi
			quality=$(echo "${line}" | cut -d'=' -f2 | cut -d'/' -f1)
			ssid=""
			ssidHex=""
			security="none"
		elif [[ "${line}" =~ ^.*WPA2\ Version ]]; then
			security="wpa2"
		fi
	done <<< "${networksScanned}"

	if [[ "${ssid}" != '' ]]; then
		rwoPrintout "${ssid}"
		rwoPrintout "${quality}"
		rwoPrintout "${security}"
		rwoPrintout "${ssidHex}"
	fi
}


function _onExit {
	hwPlatform=$(/usr/bin/rwo_system.sh getHwPlatform)
	# finished with copying the files
	# check to see if it is a pocket and if so, stop the LEDs
	rwoLog "_onExit ARGUMENTS: $ARGUMENTS"
	if [[ $ARGUMENT_ONE == 'wlan0Ap' ]]; then

		# stop the hostapd pid
		if [[ $PID_HOSTAPD != '' ]]; then
			rwoLog 'stop hostapd binary'
			/bin/kill -HUP $PID_HOSTAPD
			PID_HOSTAPD=''
		fi

		# stop node server
		systemctl --no-block stop rwo_nodeServer

		# shut off all of the leds
		if [[ "${hwPlatform}" == 'pocket-v1' ]]; then
			rwoLog 'shut off the wifi leds'
			ledsWifiSetSixOclockLed 0
			# ledsWifiSetLedsOn 0
		fi

		# remove the address and shutdown the wifi interface
		rwoLog 'flush ip address and wifi interface'
		ip addr flush ${WLAN0IFACE}
		ip link set dev ${WLAN0IFACE} down

	elif [[ $ARGUMENT_ONE == 'wlan1Connect' ]]; then
		LOOP=0
		_wlan1Disconnect
	fi

	exit 0
}

################## MAIN #######################
ARGUMENTS=$@
ARGUMENT_ONE=$1
hwPlatform=$(rwo_system.sh getHwPlatform)
hostapdConfDir="/etc/hostapd"
hostapdBinDir="/usr/bin"
hostapdBinSelected="${hostapdBinDir}/hostapd"
HOSTAPD_RUNNING_CONF="${hostapdConfDir}/hostapd.running.conf"
hostapdConfSelected="${hostapdConfDir}/hostapd.default.conf"
WLAN0IFACE="wlan0" # the interface to be used in the hostapd file ...
# use to send a real sighup to hostapd
PID_HOSTAPD=''
WLAN1IFACE="wlan1"
STATUS_DIR="/tmp/rwo"
[[ ! -d "${STATUS_DIR}" ]] && mkdir -p "${STATUS_DIR}"
WLAN1CONNECT_STATUS_FILE="${STATUS_DIR}/rwo_wifi@wlan1Connect.status"
WLAN1CONNECT_STATUSMESSAGE_FILE="${STATUS_DIR}/rwo_wifi@wlan1Connect.status.msg"
WLAN1CONNECT_SSID_FILE="${STATUS_DIR}/rwo_wifi@wlan1Connect.ssid"
WLAN1CONNECT_PASSWORD_FILE="${STATUS_DIR}/rwo_wifi@wlan1Connect.password"
LOOP=1


# First argument call
# if it did not exit above, lets assume you want to run hostapd
# systemd sends a SIGTERM before exiting
rwoLog "ARGUMENTS: $@"
trap '_onExit' EXIT INT HUP TERM
helperCallFirstArgIfFunctionAndExit $@
# the default
wlan0Ap
